#include "StdAfx.h"
#include "WarMmcSnapin.h"
#include "WmsNodeAuthModules.h"   // class implemented
#include "WarCollector.h"
#include "WarIfServer.h"
#include "WarIfAuthModule.h"
#include "WmsNodeAuthModule.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////


// {27BF7AA6-1CA1-43ae-84CD-3347E6E834D6}
const GUID WmsNodeAuthModules::thisGuid = 
{ 0x27bf7aa6, 0x1ca1, 0x43ae, { 0x84, 0xcd, 0x33, 0x47, 0xe6, 0xe8, 0x34, 0xd6 } };


WmsNodeAuthModules::WmsNodeAuthModules(war_if_site_ptr_t& rSitePtr)
: mSitePtr(rSitePtr) // Deleted by parent
{
}

WmsNodeAuthModules::~WmsNodeAuthModules()
{
}

//============================= LIFECYCLE ====================================
//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
//============================= CALLBACK     ===================================


HRESULT WmsNodeAuthModules::OnShow(IConsole *pConsole, 
                                  BOOL bShow, 
                                  HSCOPEITEM scopeitem)
{
    AFX_MANAGE_STATE(AfxGetStaticModuleState());

    HRESULT      hr = S_OK;
    
    IHeaderCtrl *pHeaderCtrl = NULL;
    IResultData *pResultData = NULL;
    
    if (bShow) 
    {
        hr = pConsole->QueryInterface(IID_IHeaderCtrl, (void **)&pHeaderCtrl);
        _ASSERT(SUCCEEDED(hr));
        
        hr = pConsole->QueryInterface(IID_IResultData, (void **)&pResultData);
        _ASSERT(SUCCEEDED(hr));

        CString hdr1_name, hdr2_name;
        hdr1_name.LoadString(IDS_HDRNAME);
        hdr2_name.LoadString(IDS_HDRAUTTYPE);

        MAKE_WIDEPTR_FROMTSTR(ptr_hdr1, hdr1_name);
        MAKE_WIDEPTR_FROMTSTR(ptr_hdr2, hdr2_name);
        
        // Set the column headers in the results pane
        hr = pHeaderCtrl->InsertColumn( 0, ptr_hdr1, 0, MMCLV_AUTO );
        _ASSERT( S_OK == hr );
        hr = pHeaderCtrl->InsertColumn( 1, ptr_hdr2, 0, MMCLV_AUTO );
        _ASSERT( S_OK == hr );
                
        hr = pResultData->DeleteAllRsltItems();
        _ASSERT( SUCCEEDED(hr) );
        
        pHeaderCtrl->Release();
        pResultData->Release();

         war_if_aut_module_list_t mod_list;
        mSitePtr->EnumAuthModules(mod_list);

        for(war_if_aut_module_list_t::iterator P = mod_list.begin()
            ; P != mod_list.end()
            ; ++P)
        {
            CDelegationBase *pnew_node;
            RESULTDATAITEM rdi;

            pnew_node = new WmsNodeAuthModule(mSitePtr, *P);

            ZeroMemory(&rdi, sizeof(RESULTDATAITEM));
            rdi.mask =  RDI_STR | RDI_IMAGE | RDI_PARAM; 
            rdi.nImage = pnew_node->GetBitmapIndex();
            rdi.str = MMC_CALLBACK;
            rdi.lParam = (LPARAM)pnew_node;     // The cookie
            rdi.nCol = 2;
            
            hr = pResultData->InsertItem( &rdi );
            _ASSERT( SUCCEEDED(hr) );
        }
    }
    
    return hr;
}


//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================

const int WmsNodeAuthModules::GetBitmapIndex()
{
    return NICON_FOLDER;
}

const _TCHAR *WmsNodeAuthModules::GetDisplayName(int nCol)
{
    static _TCHAR szDisplayName[256] = {0};
    
    if (!*szDisplayName)
    {
        LoadString(g_hinst, 
            IDS_NODE_AUTH_MODULES, 
            szDisplayName, 
            sizeof(szDisplayName));
    }
    
    return szDisplayName; 
}

const GUID & WmsNodeAuthModules::getNodeType()
{
    return thisGuid; 
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

