#include "StdAfx.h"
#include "WarMmcSnapin.h"
#include "WmsNodeManagement.h"   // class implemented
#include "WarCollector.h"
#include "WarIfServer.h"
#include "WmsNodeCurrentActivity.h"
#include "WmsNodeLogModules.h"
#include "WmsNodeServerLogs.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////

// {FD0DB811-7B0D-4140-90BB-0DE8AE2EAAEC}
const GUID WmsNodeManagement::thisGuid = 
{ 0xfd0db811, 0x7b0d, 0x4140, { 0x90, 0xbb, 0xd, 0xe8, 0xae, 0x2e, 0xaa, 0xec } };


WmsNodeManagement::WmsNodeManagement(war_if_server_ptr_t& rSvrPtr)
: mServicePtr(rSvrPtr) // Deleted by parent
{
}

WmsNodeManagement::~WmsNodeManagement()
{
}

//============================= LIFECYCLE ====================================
//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
//============================= CALLBACK     ===================================

HRESULT WmsNodeManagement::OnExpand(IConsoleNameSpace *pConsoleNameSpace, 
                                 IConsole *pConsole, 
                                 HSCOPEITEM parent)
{
    if (GetHandle() == NULL) 
        SetHandle((HANDLE)parent);

    int extra_flags = 0;

    if (!bExpanded) 
    {
        // create the child nodes, then expand them
        for(int i = 0; i < 3; i++)
        {
            CDelegationBase *pnew_node;
            SCOPEDATAITEM sdi;

            switch(i)
            {
            case 0:
                pnew_node = new WmsNodeCurrentActivity(mServicePtr);
                break;
            case 1:
                pnew_node = new WmsNodeLogModules(mServicePtr);
                extra_flags = SDI_CHILDREN;
                break;
            case 2:
                pnew_node = new WmsNodeServerLogs(mServicePtr);
                break;
            }

            ZeroMemory(&sdi, sizeof(SCOPEDATAITEM));
            sdi.mask = SDI_STR|   // Displayname is valid
                SDI_PARAM     |   // lParam is valid
                SDI_IMAGE     |   // nImage is valid
                SDI_OPENIMAGE |   // nOpenImage is valid
                SDI_PARENT    |
                extra_flags;
            
            sdi.relativeID  = (HSCOPEITEM)parent;
            sdi.nImage      = pnew_node->GetBitmapIndex();
            sdi.nOpenImage  = pnew_node->GetBitmapIndex();
            sdi.displayname = MMC_CALLBACK;
            sdi.lParam      = (LPARAM)pnew_node;     // The cookie
            
            HRESULT hr = pConsoleNameSpace->InsertItem(&sdi);
            _ASSERT( SUCCEEDED(hr) );
            
            pnew_node->SetHandle((HANDLE)sdi.ID);
        }
    }
 
    return S_OK;   
}

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================

const int WmsNodeManagement::GetBitmapIndex()
{
    return NICON_FOLDER;
}

const _TCHAR *WmsNodeManagement::GetDisplayName(int nCol)
{
    static _TCHAR szDisplayName[256] = {0};
    
    if (!*szDisplayName)
    {
        LoadString(g_hinst, 
            IDS_NODE_MANAGEMENT, 
            szDisplayName, 
            sizeof(szDisplayName));
    }
    
    return szDisplayName; 
}

const GUID & WmsNodeManagement::getNodeType()
{
    return thisGuid; 
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

