

#include "StdAfx.h"
#include <strstream>
#include "WarCollector.h"   // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
//============================= ACESS      ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

/* EXTERNAL REFERENCES */
/** */

WarCollector<char>& operator << (WarCollector<char>& c, 
                           const struct in_addr& val)
{
    const unsigned char *pbytes = (const unsigned char *)&val;
    
    return c << (unsigned)pbytes[0] << '.' 
        << (unsigned)pbytes[1] << '.' 
        << (unsigned)pbytes[2] << '.' 
        << (unsigned)pbytes[3];
}

WarCollector<char>& operator << (WarCollector<char>& c, 
                           const char val)
{
    if (c.MustErase() && !c.mString.empty())
    {
        size_t str_len = c.mString.size();
        size_t len = str_len + 1 + 1;
        char *buf = (char *)alloca(len);
        memcpy(buf, c.mString.c_str(), str_len);
        buf[str_len] = val;
        buf[len -1] = 0;
        c.Reset();
        c.mString = buf;
        memset(buf, 0, len);
    }
    else
        c.mString += val;

    return c;
}

WarCollector<wchar_t>& operator << (WarCollector<wchar_t>& c, 
                           const wchar_t val)
{
    if (c.MustErase() && !c.mString.empty())
    {
        size_t str_len = c.mString.size();
        size_t len = str_len + 1 + 1;
        wchar_t *buf = (wchar_t *)alloca(len * sizeof(wchar_t));
        memcpy(buf, c.mString.c_str(), str_len * sizeof(wchar_t));
        buf[str_len] = val;
        buf[len -1] = 0;
        c.Reset();
        c.mString = buf;
        memset(buf, 0, len * sizeof(wchar_t));
    }
    else
        c.mString += val;
    return c;
}

WarCollector<char>& operator << (WarCollector<char>& c, 
                           const wchar_t val)
{
    char ch = 0;
    if (::wctomb(&ch, val) != 1)
        WarThrow(WarError(WAR_CHRCNV_NO_CONVERSION), NULL);

    return c << ch;
}

WarCollector<wchar_t>& operator << (WarCollector<wchar_t>& c, 
                           const char val)
{
    wchar_t wc = 0;
    if (::mbtowc(&wc, &val, 1) != 1)
        WarThrow(WarError(WAR_CHRCNV_NO_CONVERSION), NULL);

    return c << wc;
}

WarCollector<char>& operator << (WarCollector<char>& c, 
                           const war_ccstr_t val)
{
    if (val && *val)
    {
        if (c.MustErase() && !c.mString.empty())
        {
            size_t str_len = c.mString.size();
            size_t new_str_len = strlen(val);
            size_t len = str_len + new_str_len + 1;
            char *buf = (char *)alloca(len * sizeof(char));
            memcpy(buf, c.mString.c_str(), str_len * sizeof(char));
            memcpy(buf + str_len, val, new_str_len * sizeof(char));
            buf[len -1] = 0;
            c.Reset();
            c.mString = buf;
            memset(buf, 0, len * sizeof(wchar_t));
        }
        else
            c.mString += val;
    }
    return c;
}

WarCollector<char>& operator << (WarCollector<char>& c, 
                                 const war_cwstr_t val)
{
    if (val && *val)
    {
        size_t buflen = ::wcstombs(NULL, val, 0);
        int result = 0;
        if (buflen)
        {
            std::vector<char> buf(buflen +1);
            char *pbuf = (char *)&buf[0];
            
            if (result = ::wcstombs(pbuf, val, buflen +1))
                c << pbuf;

            if (c.MustErase())
                memset(&buf[0], 0, buf.size());
        }

        if (!result)
            WarThrow(WarError(WAR_CHRCNV_NO_CONVERSION), NULL);
    }
    return c;
}

WarCollector<wchar_t>& operator << (WarCollector<wchar_t>& c, 
                           const war_ccstr_t val)
{
    if (val && *val)
    {
        size_t buflen = ::mbstowcs(NULL, val, 0);
        int result = 0;
        if (buflen)
        {
            std::vector<wchar_t> buf(buflen +1);
            wchar_t *pbuf = (wchar_t *)&buf[0];
            
            if (result = ::mbstowcs(pbuf, val, buflen +1))
                c << pbuf;
        
            if (c.MustErase())
                memset(&buf[0], 0, buf.size() * sizeof(wchar_t));
        }

        if (!result)
            WarThrow(WarError(WAR_CHRCNV_NO_CONVERSION), NULL);
    }
    return c;
}

WarCollector<wchar_t>& operator << (WarCollector<wchar_t>& c, 
                           const war_cwstr_t val)
{
    if (val && *val)
    {
        if (c.MustErase() && !c.mString.empty())
        {
            size_t str_len = c.mString.size();
            size_t new_str_len = wcslen(val);
            size_t len = str_len + new_str_len + 1;
            wchar_t *buf = (wchar_t *)alloca(len * sizeof(wchar_t));
            memcpy(buf, c.mString.c_str(), str_len * sizeof(wchar_t));
            memcpy(buf + str_len, val, new_str_len * sizeof(wchar_t));
            buf[len -1] = 0;
            c.Reset();
            c.mString = buf;
            memset(buf, 0, len * sizeof(wchar_t));
        }
        else
            c.mString += val;
    }
    return c;
}

WarCollector<char>& operator << (WarCollector<char>& c, 
                           const std::string& val)
{
    if (c.MustErase() && !c.mString.empty())
        return c << val.c_str();

    c.mString += val;
    return c;
}

WarCollector<char>& operator << (WarCollector<char>& c, 
                           const std::wstring& val)
{
    return c << val.c_str();
}

WarCollector<wchar_t>& operator << (WarCollector<wchar_t>& c, 
                           const std::string& val)
{
    return c << val.c_str();
}

WarCollector<wchar_t>& operator << (WarCollector<wchar_t>& c, 
                           const std::wstring& val)
{
    if (c.MustErase() && !c.mString.empty())
        return c << val.c_str();

    c.mString += val;
    return c;
}
