#include "StdAfx.h"
#include "WarCriticalSection.h"   // class implemented
#include <assert.h>

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

WarCriticalSection::WarCriticalSection(bool allowRecursive)
  throw(WarException)
  : mDoRecursive(allowRecursive)
#ifdef _DEBUG
    , mNumLocks(0)
#endif
{
    int err = ::pthread_mutex_init(&mHandle, allowRecursive 
            ? PTHREAD_MUTEX_RECURSIVE_NP 
            : PTHREAD_MUTEX_FAST_NP);
    if (err)
        WarThrow(WarError(WAR_THREADERR_FAIED_TO_INITIALIZE_MUTEX, err), NULL);
}// WarCriticalSection


WarCriticalSection::~WarCriticalSection()
{
#ifdef _DEBUG
    assert(mNumLocks == 0);
#endif
    ::pthread_mutex_destroy(&mHandle);
}// ~WarCriticalSection


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

void WarCriticalSection::Lock()
  throw(WarException)
{
#ifdef _DEBUG
    ++mNumLocks;
    if (!mDoRecursive)
        assert(mNumLocks == 1);
#endif

    int err = pthread_mutex_lock(&mHandle);
    if (err)
        WarThrow(WarError(WAR_THREADERR_MUTEX_LOCK_FAILED, err), NULL);
}


void WarCriticalSection::Unlock()
  throw(WarException)
{
#ifdef _DEBUG
    --mNumLocks;
    assert(mNumLocks >= 0);
    if (!mDoRecursive)
        assert(mNumLocks == 0);
#endif

    int err = pthread_mutex_unlock(&mHandle);
    if (err)
        WarThrow(WarError(WAR_THREADERR_MUTEX_UNLOCK_FAILED, err), NULL);
}

//============================= ACESS      ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

