#include "StdAfx.h"
#include "WarFunctionProfiler.h"   // class implemented

#ifndef WAR_COLLECTOR_H
#   include "WarCollector.h"
#endif

/////////////////////////////// PUBLIC ///////////////////////////////////////

WarFunctionProfiler *WarFunctionProfiler::mspProfiler;

//============================= LIFECYCLE ====================================

WarFunctionProfiler::WarFunctionProfiler()
{
    memset(&mData, 0, sizeof(mData));
    InitializeCriticalSection(&mhCritSect);
    mspProfiler = this;
    QueryPerformanceFrequency((LARGE_INTEGER *)&mTimerResolution);
}

WarFunctionProfiler::~WarFunctionProfiler()
{
    mspProfiler = NULL;
    DeleteCriticalSection(&mhCritSect);
}


//============================= OPERATORS ====================================

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================

std::string WarFunctionProfiler::Explain() const
{
    WarCollector<char> result;
    const char *names[WAR_FT_INVALID] = 
    {
        "WAR_FP_FTPDATA_PUMP",
        "WAR_FP_FTPDATA_PUMP_FREAD_ASYNCH",
        "WAR_FP_FTPDATA_PUMP_FREAD_SYNCH",
        "WAR_FP_FTPDATA_PUMP_FREAD_CHKEOF",
        "WAR_FP_FTPDATA_PUMP_SCK_OUT",
        "WAR_FP_FTPDATA_PUMP_RECEIVING",
        "WAR_FP_WarSvrProtocolFtpDataConn_ProcessPendingBuffers",
        "WAR_FP_WarSvrProtocolFtpDataConn_OnTransferDone",
        "WAR_FP_WarSvrProtocolFtpDataConn_OnAccept",
        "WAR_FP_WarSvrProtocolFtpDataConn_OnConnect",
        "WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo",
        "WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_SendWithCallback",
        "WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_ERROR",
        "WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_QUEUE",
        "WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_RECV",
        "WAR_FP_WarSvrProtocolFtpDataConn_OnReceived",
        "WAR_FP_WarSvrProtocolFtpDataConn_OnSent",
        "WAR_FP_WarFileDriverFileWin32_ReadWithCallback",
        "WAR_FP_WarFileDriverFileWin32_ReadWithCallback_ALLOC",
        "WAR_FP_WarFileDriverFileWin32_ReadWithCallback_PREP",
        "WAR_FP_WarFileDriverFileWin32_ReadWithCallback_READFILE",
        "WAR_FP_WarFileDriverFileWin32_ReadWithCallback_POSTPRC",
        "WAR_FP_WarFileDriverFileWin32_WriteWithCallback",
        "WAR_FP_WarFileDriverFileWin32_AllocResult_NEW",
        "WAR_FP_WarFileDriverFileWin32_AllocResult_ASSIGN",
        "WAR_FP_WarSocketIoWin32Nt_SendWithCallback",
        "WAR_FP_WarSocketIoWin32Nt_SendWithCallback_ALLOC",
        "WAR_FP_WarSocketIoWin32Nt_SendWithCallback_READFILE",
        "WAR_FP_WarSocketIoWin32Nt_SendWithCallback_POSTPRC",
        "WAR_FP_WarSocketIoWin32Nt_RecvWithCallback",
        "WAR_FP_WarSocketIoWin32Nt_AllocResult_NEW",
        "WAR_FP_WarSocketIoWin32Nt_AllocResult_ASSIGN",
        "WAR_FP_WarUserAuthDataWin32Nt_BeginImpersonate",
        "WAR_FP_WarUserAuthDataWin32Nt_EndImpersonate",
        "WAR_FP_WarSvrProtocolFtpDataConn_OnReceived_SIMPLE",
        "WAR_FP_WarSvrProtocolFtpDataConn_OnReceived_QUEUE",

    };

    for(int index = 0; index < WAR_FT_INVALID; index++)
    {
        char buf [128];

        sprintf(buf, "%8.0lf milliseconds %8.0lf calls %s\n", 
            (double)mData[index].pd_total_time / ((double)mTimerResolution / 1000.0),
            (double)mData[index].pd_counter,
            names[index]);

        result << buf;
    }

    return result.GetValue();
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

