#include "StdAfx.h"
#include "WarLog.h"   // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

const char WarLog::sTypeChr[WARLOG_INVALID + 1] =
{ "DEFISWSNsrcddLOAAACSDDN" };

const char *WarLog::sTypeString[WARLOG_INVALID + 1] = 
{
    {"WARLOG_DEBUG"},
    {"WARLOG_ERROR"},
    {"WARLOG_FILEACC"},
    {"WARLOG_INOUT"},
    {"WARLOG_SECURITY"},
    {"WARLOG_WARNINGS"},
    {"WARLOG_SYSTEM"},
    {"WARLOG_SOCKET"},
    {"WARLOG_SNDFILE"},
    {"WARLOG_RCVFILE"},
    {"WARLOG_CREDIR"},
    {"WARLOG_DELDIR"},
    {"WARLOG_DELFILE"},
    {"WARLOG_LOGIN"},
    {"WARLOG_LOGOUT"},
    {"WARLOG_CREACC"},
    {"WARLOG_DELACC"},
    {"WARLOG_CHGACC"},
    {"WARLOG_CPS"},
    {"WARLOG_INFO"},
    {"WARLOG_THREADS"},
    {"WARLOG_FILES"},
    {"WARLOG_NETWORK"}
};


//============================= LIFECYCLE ====================================

WarLog::WarLog(WarLogEventTypeE logType, war_ccstr_t logFunction) :
mpData(NULL),
mType(logType)
{
    if (WAR_IS_LOGEVENT_IN_USE(mType))
    {
        mpData = new WarLogEventData(mType, logFunction);
    }
}// WarLog


WarLog::~WarLog()
{
    if (mpData)
    {
        delete mpData;
        mpData = NULL;
    }
}// ~WarLog


//============================= OPERATORS ====================================

WarLog::operator bool () const
{
    return WAR_IS_LOGEVENT_IN_USE(mType);
}

WarLogEventTypeE WarLog::Lookup(war_ccstr_t eventName)
{
    for(int i = 0; i < WARLOG_INVALID; i++)
    {
        if (!stricmp(eventName, WarLog::sTypeString[i]))
            break;
    }

    return (WarLogEventTypeE)i;
}

//============================= OPERATIONS ===================================
//============================= ACESS      ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/* Build a log-event ant send it to the event-chain. */
void WarLog::Submit()
{
    if (!mpData)
        return;

    mpData->mTime.Reset();
    mpData->mMessage = GetValue();

    WarLogEngine::GetEngine().LogEvent(new WarLogEvent(*mpData));

    Reset();
    mpData->mError.Reset();
    mpData->mMessage.erase();
    mpData->mInterfaceName.erase();
    mpData->mUser.erase();
}

/////////////////////////////// PRIVATE    ///////////////////////////////////


//============================= EXTERNAL REFERENCES ==========================


void WarLogError(war_ccstr_t func, war_ccstr_t message,
                 WarException *pexception)
{
#if WAR_CATCH_ALL
    try
#endif
    {
        std::string explanation;
        if (pexception)
            explanation = pexception->Explain();

        WarLog err_log(WARLOG_ERROR, "WarLogError");
        err_log << "From " 
            << func 
            << " : " 
            << message
            << " : "
            << explanation
            << war_endl;
    }

#if WAR_CATCH_ALL
    catch(...)
    {
    }
#endif
}

