#include "StdAfx.h"
#include "WarSocketIoWin32Nt.h"   // class implemented

#include "Mswsock.h"

#ifndef WAR_SVR_WIN32_NT_ENGINE_H
#   include "WarSvrWin32NtEngine.h"
#endif

#ifndef WAR_AUTO_LOCK_H
#   include "WarAutoLock.h"
#endif

#ifndef WAR_LOG_H
#   include "WarLog.h"
#endif

#ifndef WAR_AUTO_LOCK_H
#   include "WarAutoLock.h"
#endif

#ifndef WAR_PERFMON_DEF_H
#   include "WarPerfmonDef.h"
#endif

#ifndef WAR_FUNCTION_PROFILER_H
#   include "WarFunctionProfiler.h"
#endif


#define AUTO_LOCK WarAutoLock my_lock(mLock);

/////////////////////////////// PUBLIC ///////////////////////////////////////


WarSocketIoWin32Nt::ol_result_t *WarSocketIoWin32Nt::mspBufferList;
WarCriticalSection WarSocketIoWin32Nt::msBufferLock;

//============================= LIFECYCLE ====================================

WarSocketIoWin32Nt::WarSocketIoWin32Nt()
{
}// WarSocketIoWin32Nt


WarSocketIoWin32Nt::~WarSocketIoWin32Nt()
{
}// ~WarSocketIoWin32Nt


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

void WarSocketIoWin32Nt::SendWithCallback(
    war_transfer_buffer_ptr_t& outBuffer,
    size_t numSegments)
    throw(WarException)
{
    WAR_PROFILE_FUNC(WAR_FP_WarSocketIoWin32Nt_SendWithCallback);

    DWORD dwBytes = 0;
    WarTransferBuffer *pbuffer = &(*outBuffer);
    
    if (numSegments > 1)
    {
        if (NULL == pbuffer->mpLock)
            pbuffer->mpLock = new WarCriticalSection;
    }

    if (pbuffer->mpLock)
        pbuffer->mpLock->Lock();
    
    char *p = pbuffer->GetBuffer();
    size_t segment_size = pbuffer->GetLength() / numSegments;
    size_t bytes = 0;
    size_t bytes_left = pbuffer->GetBytesUsed();
    assert((segment_size * numSegments) == pbuffer->GetLength());

    pbuffer->mBytesUsed = 0; // Prepere for data read/written

    for(unsigned segment = 0; ; segment++)
    {
        ol_result_t *pResult;

        bytes = min(segment_size, bytes_left);
        {
            WAR_PROFILE_FUNC(WAR_FP_WarSocketIoWin32Nt_SendWithCallback_ALLOC);
            pResult = AllocResult(OP_SEND, segment);
        }
        pResult->mBufferPtr = outBuffer;
        pbuffer->mSegmentCnt++;
        int result;
        
        {
            WAR_PROFILE_FUNC(WAR_FP_WarSocketIoWin32Nt_SendWithCallback_READFILE);
            result = WriteFile((HANDLE)mSocket, 
                p,
                bytes,
                &dwBytes,
                &pResult->mOverlappedData);
        }

        WAR_PROFILE_FUNC(WAR_FP_WarSocketIoWin32Nt_SendWithCallback_POSTPRC);

        if (!result)
        {
            WarSystemError system_error;
            
            switch(system_error.SystemError())
            {
            case ERROR_IO_PENDING:
                break; // No action
            case ERROR_HANDLE_EOF:
                if (pbuffer->mpLock)
                    pbuffer->mpLock->Unlock();
                DeleteResult(pResult);
                WarThrow(WarError(WAR_FERR_END_OF_FILE, ERROR_HANDLE_EOF), NULL);
                break;
            default:
                if (pbuffer->mpLock)
                    pbuffer->mpLock->Unlock();
                DeleteResult(pResult);
                WarThrow(system_error, NULL);
            }
        }

        bytes_left -= bytes;
        if (bytes_left == 0)
            break;
        p += bytes;
    }

    if (pbuffer->mpLock)
        pbuffer->mpLock->Unlock();

    WAR_DB_PERFMON_INC(WAR_PRFDEBUG_NUM_SCKPENDINGIO);
}

void WarSocketIoWin32Nt::RecvWithCallback(
                                          war_transfer_buffer_ptr_t& inBuffer,
                                          size_t numSegments)
                                          throw(WarException)
{
    WAR_PROFILE_FUNC(WAR_FP_WarSocketIoWin32Nt_RecvWithCallback);

    DWORD dwBytes = 0;
    
    WarTransferBuffer *pbuffer = &(*inBuffer);
    
    if (numSegments > 1)
    {
        if (NULL == pbuffer->mpLock)
            pbuffer->mpLock = new WarCriticalSection;
    }
    
    if (pbuffer->mpLock)
        pbuffer->mpLock->Lock();
    
    char *p = pbuffer->GetBuffer();
    size_t segment_size = pbuffer->GetLength() / numSegments;
    assert((segment_size * numSegments) == pbuffer->GetLength());

    pbuffer->mBytesUsed = 0; // Prepere for data read/written
    
    for(unsigned segment = 0
        ; segment < numSegments
        ; segment++, p += segment_size)
    {
        ol_result_t *pResult;
        {
            pResult = AllocResult(OP_RECV, segment);
        }
        pResult->mBufferPtr = inBuffer;
        pbuffer->mSegmentCnt++;
        
        if (!ReadFile((HANDLE)mSocket, 
            p,
            segment_size,
            &dwBytes,
            &pResult->mOverlappedData))
        {
            WarSystemError system_error;
            
            switch(system_error.SystemError())
            {
            case ERROR_IO_PENDING:
                break; // No action
            case ERROR_HANDLE_EOF:
                if (pbuffer->mpLock)
                    pbuffer->mpLock->Unlock();
                DeleteResult(pResult);
                WarThrow(WarError(WAR_FERR_END_OF_FILE, ERROR_HANDLE_EOF), NULL);
                break;
            default:
                if (pbuffer->mpLock)
                    pbuffer->mpLock->Unlock();
                DeleteResult(pResult);
                WarThrow(system_error, NULL);
            }
        }
        else if(dwBytes == 0)
        { 
            if (pbuffer->mpLock)
                pbuffer->mpLock->Unlock();
            WarThrow(WarError(WAR_FERR_END_OF_FILE, ERROR_HANDLE_EOF), NULL);
            break;	//No more data
        }
        
    }
    
    if (pbuffer->mpLock)
        pbuffer->mpLock->Unlock();

    WAR_DB_PERFMON_INC(WAR_PRFDEBUG_NUM_SCKPENDINGIO);
}
//============================= ACCESS     ===================================

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////


void WarSocketIoWin32Nt::DoListen(int backlog)
    throw(WarException)
{
    WarLog error_log(WARLOG_ERROR, "WarSocketIoWin32Nt::DoListen");

    AUTO_LOCK

    try
    {
        WarSocketIo::DoListen(backlog);
    }
    catch(WarException& ex)
    {
        error_log << "Failed to initiate listening connections. "
            << ex.Explain()
            << war_endl;

        goto fail;
    }

    {
        // Set up #backlog number of pending connections
        for(int t = 0; t < backlog; t++)
            CreateNewConnectionSocket();
    }

fail:

    if (mListenQueue.empty())
    {
        error_log << "No accept handles aqued. I cannot accept connections in a reaiable manner. Aborting operation."
            << war_endl;

        WarThrow(WarError(WAR_ERR_INTERNAL_DATA_NOT_INITIALIZED), NULL);
    }
    
    if (mListenQueue.size() < backlog)
    {
        error_log << "Continuing with reduced capacity for connections. "
            << (int)mListenQueue.size()
            << " of "
            << (int)backlog
            << " future connection sockets were allocated."
            << war_endl;
    }
}


void WarSocketIoWin32Nt::DoConnect(const struct sockaddr FAR *name, 
                                  int namelen)
    throw(WarException)
{
    WarError throw_err;

    try
    {
        WarSocketIo::DoConnect(name, namelen);
    }
    catch(WarException& ex)
    {
        throw_err = ex;
        if (ex.SystemError() == WSAEWOULDBLOCK)
            goto make_async_connect;
        
        throw ex;
    }

    return;

make_async_connect:
    {
        // Implement I/O Completion port 
        WarSvrWin32EventThreadTask *p = new WarSvrWin32EventThreadTask;
        WarSvrWin32EventThread::task_ptr_t my_ptr = p;
        
        p->mCompletionPort = WarSvrWin32NtEngine::GetEngine().GetPort();
        p->mCompletionKey = (DWORD)FileIOCompletionRoutine;
        p->mpOverlapped = (LPOVERLAPPED)AllocResult(OP_CONNECT, 0);
        p->mSocket = mSocket;
        
        if ((p->mEvent = ::CreateEvent(NULL, false, false, NULL))
            == NULL)
            WarThrowSyserr("CreateEvent()");
        
        if (::WSAEventSelect(mSocket, p->mEvent, FD_CONNECT) != 0)
        {
            WarLog debug_log(WARLOG_DEBUG, "WarSocketIoWin32Nt::DoConnect()");
            WarSystemError sys_err;
            
            if (debug_log)
            {
                debug_log << "Failed to initiate an async connect. (WSAEventSelect). "
                    << sys_err
                    << war_endl;
            }
            
            WarThrow(sys_err, "WSAEventSelect()");
        }
        
        try
        {
            WarSvrWin32NtEngine::GetEngine().QueueRequest(my_ptr);
            WAR_DB_PERFMON_INC(WAR_PRFDEBUG_NUM_SCKCONNPNDIO);
        }
        catch(WarException& ex)
        {
            WarLog debug_log(WARLOG_DEBUG, "WarSocketIoWin32Nt::DoConnect()");
            if (debug_log)
            {
                debug_log << "Failed to queue an async connect. (QueueRequest). "
                    << ex
                    << war_endl;
            }
            
            throw ex;
        }

        WarThrow(throw_err, NULL);
    }
}

void WarSocketIoWin32Nt::DoCreateSocket(int af, int type, int protocol)
    throw(WarException)
{
    WarSocketIo::DoCreateSocket(af, type, protocol);

    assert(mSocket != WAR_INVALID_SOCKET);

    if (::CreateIoCompletionPort((HANDLE)mSocket,
        WarSvrWin32NtEngine::GetEngine().GetPort(),
        (DWORD)FileIOCompletionRoutine,
        0) == NULL)
    {
        WarError system_error;
        system_error.Capture();

        DoClose();

        WarThrow(system_error, "CreateIoCompletionPort()");
    }
}

void WarSocketIoWin32Nt::AssignSocket(war_socket_t sck) 
        throw(WarException)
{
    WarSocketIo::AssignSocket(sck);

    if (::CreateIoCompletionPort((HANDLE)mSocket,
        WarSvrWin32NtEngine::GetEngine().GetPort(),
        (DWORD)FileIOCompletionRoutine,
        0) == NULL)
    {
        WarError system_error;
        system_error.Capture();

        DoClose();

        WarThrow(system_error, "CreateIoCompletionPort()");
    }
}


WarSocketIoWin32Nt::ol_result_def *
WarSocketIoWin32Nt::AllocResult(WarSocketIoWin32Nt::IoOperationE operation,
                                unsigned segmentNum)
{
    WarAutoLock my_lock(msBufferLock);

    ol_result_t *p;

    if (mspBufferList)
    {
        p = mspBufferList;
        mspBufferList = mspBufferList->mpNext;
    }
    else
    {
        WAR_PROFILE_FUNC(WAR_FP_WarSocketIoWin32Nt_AllocResult_NEW);

        p = new ol_result_t;
    }

    WAR_PROFILE_FUNC(WAR_FP_WarSocketIoWin32Nt_AllocResult_ASSIGN);
    p->mpSocket = this;
    p->SocketPtr = mpCompanion; // Protect the socket from destruction
    memset(&(p->mOverlappedData), 0, sizeof(OVERLAPPED));
    p->mOperation = operation;
    p->mAcceptSocket = WAR_INVALID_SOCKET;
    p->mSegmentNum = segmentNum;
    
    return p;
}

void WarSocketIoWin32Nt::DeleteResult(ol_result_def *presult)
{
     WarAutoLock my_lock(msBufferLock);

    // Prepere for re-use
    presult->mpNext = mspBufferList;
    mspBufferList = presult;

    // Release autopointers
    presult->SocketPtr = NULL;
    presult->mBufferPtr = NULL;
}

void WarSocketIoWin32Nt::OnConnect(const WarError& status)
{
    ULONG socket_error = 0;
    int socket_error_len = sizeof(socket_error);
    WarError my_status = status;

    if (!status)
    {
        // Check if we _really_ are connected
        if (!getsockopt(mSocket, SOL_SOCKET, SO_ERROR, 
            (char *)&socket_error, &socket_error_len)
            && (socket_error != 0))
        {
            my_status = WarSystemError();
            if (!my_status)
                my_status = WarError(WAR_ERR_SYSTEM_ERROR);
        }
    }

    WarSocketIo::OnConnect(my_status);
}

/////////////////////////////// PRIVATE    ///////////////////////////////////


VOID CALLBACK WarSocketIoWin32Nt::FileIOCompletionRoutine(
        DWORD dwErrorCode,                // completion code
        DWORD dwNumberOfBytesTransfered,  // number of bytes transferred
        LPOVERLAPPED lpOverlapped         // I/O information buffer
        )
{
    WarLog err_log(WARLOG_ERROR, "WarSocketIoWin32Nt::FileIOCompletionRoutine()");
    ol_result_t *pResult = (ol_result_def *)lpOverlapped;
    WarError err((dwErrorCode != WAR_ERR_OK) 
        ? WAR_ERR_SYSTEM_ERROR : WAR_ERR_OK, (int)dwErrorCode);

    static const char *op_names[OP_INVALID +1] = 
    {
        "OP_RECV",
        "OP_SEND",
        "OP_CONNECT",
        "OP_ACCEPT",
        "OP_INVALID"
    };
    
    war_ccstr_t current_op = op_names[
        WarMin((unsigned)pResult->mOperation, (unsigned)OP_INVALID)];

    if (err && (err.SystemError() == ERROR_HANDLE_EOF))
        err = WAR_FERR_END_OF_FILE;

#ifdef DEBUG
    WarLog net_log(WARLOG_SOCKET, "WarSocketIoWin32Nt::FileIOCompletionRoutine()");

    if (net_log)
    {
        net_log << "Socket event: "
            << current_op
            << " sck: "
            << pResult->mpSocket->GetSeqNumber()
            << " ";
        if (err)
            net_log << err;
        else
            net_log << "OK ";

        WarThread *pthread = WarThread::GetCurrentThread();
        if (pthread)
        {
            net_log << "thd: " 
                << (unsigned)pthread->GetThreadId() 
                << " ";
        }

        net_log << " len: "
            << (unsigned)dwNumberOfBytesTransfered
            << war_endl;
    }
#endif

    try
    {
        switch(pResult->mOperation)
        {
        case OP_RECV:
            {
                WAR_DB_PERFMON_DEC(WAR_PRFDEBUG_NUM_SCKPENDINGIO);

                WarTransferBuffer *pbuffer = &(*pResult->mBufferPtr);
                
                {
                    WarAutoLock my_buffer_lock(pbuffer->mpLock);
                    
                    pResult->mBufferPtr->mBytesUsed 
                        += dwNumberOfBytesTransfered;
                    
                    if (--pbuffer->mSegmentCnt == 0)
                    {
                        if ((pbuffer->GetBytesUsed() > 0)
                            && (WAR_FERR_END_OF_FILE == err.LocalError()))
                            err.Reset(); // Let the higher levels process the data
                    }
                    else
                        break;
                } // unlock buffer lock

                pResult->mpSocket->OnReceived(err, pResult->mBufferPtr);
            }
            break;
        case OP_SEND:
            {
                WAR_DB_PERFMON_DEC(WAR_PRFDEBUG_NUM_SCKPENDINGIO);

                WarTransferBuffer *pbuffer = & (*pResult->mBufferPtr);
                {
                    
                    WarAutoLock my_buffer_lock(pbuffer->mpLock);
                    
                    pResult->mBufferPtr->mBytesUsed 
                        += dwNumberOfBytesTransfered;
                    
                    if (--pbuffer->mSegmentCnt == 0)
                    {
                        if ((pbuffer->GetBytesUsed() > 0)
                            && (WAR_FERR_END_OF_FILE == err.LocalError()))
                            err.Reset(); // Let the higher levels process the data
                    }
                    else
                        break;
                    
                } // unlock buffer lock
                pResult->mpSocket->OnSent(err, pResult->mBufferPtr);
            }
            break;
        case OP_CONNECT:
            WAR_DB_PERFMON_DEC(WAR_PRFDEBUG_NUM_SCKCONNPNDIO);

            pResult->mpSocket->OnConnect(err);
            break;
        case OP_ACCEPT:
            {
                // Handle errors 

                switch (dwErrorCode)
                {
                case ERROR_SUCCESS:
                    break;                    
                case ERROR_OPERATION_ABORTED:
                    err = WarError(WAR_ERR_ABORT_THREAD, dwErrorCode);
                    break;
                default:
                    err = WarError(WAR_ERR_SYSTEM_ERROR, dwErrorCode);
                }
                
                bool do_callback = false;
                pending_ptr_t tmp_holder;
                WarNetAddress addr_local, addr_remote;
                
                if (!dwErrorCode)
                {
                    WarAutoLock my_lock(pResult->mpSocket->mLock);
                    
                    // Replace the used connection handle
                    if (pResult->mpSocket->IsOpen())
                    {
                        pResult->mpSocket->CreateNewConnectionSocket();
                        
                        // Handle the new connection
                        pending_ptr_t search_socket(
                            new WarSocketIoWin32NtPending(
                            pResult->mAcceptSocket));
                        pending_set_t::iterator P = 
                            pResult->mpSocket->mListenQueue.find(search_socket);
                        search_socket->mSocket = WAR_INVALID_SOCKET; // prevent close
                        if (P != pResult->mpSocket->mListenQueue.end())
                        {
                            tmp_holder = (*P);
                            pResult->mpSocket->mListenQueue.erase(P);
                        }
                        
                        WarSocketIoWin32NtPending& rio_buffer = *tmp_holder;
                        struct sockaddr *plocal_sockaddr = NULL, 
                            *premote_sockaddr = NULL;
                        INT local_sockaddr_length = 0;
                        INT remote_sockaddr_length = 0;
                        
                        GetAcceptExSockaddrs(rio_buffer.mBuffer,
                            rio_buffer.GetReadBufferLength(),
                            rio_buffer.GetLocalAddrBufferLength(),
                            rio_buffer.GetRemoteAddrBufferLength(),
                            &plocal_sockaddr,
                            &local_sockaddr_length,
                            &premote_sockaddr,
                            &remote_sockaddr_length);
                        
                        addr_local.AssignSockaddr(plocal_sockaddr, 
                            local_sockaddr_length);
                        addr_remote.AssignSockaddr(premote_sockaddr, 
                            remote_sockaddr_length);
                        
                        do_callback = true;
                    }
                }
                
                if (do_callback)
                {
                    pResult->mpSocket->OnAccept(err, 
                        pResult->mAcceptSocket,
                        addr_remote,
                        addr_local);
                    
                        /* If we got this far, we leave the responsibility for
                        * the new socket to whomever handled the OnAccept()
                        * call. 
                        * In any other case, we would have closed it.
                    */
                    
                    tmp_holder->mSocket = WAR_INVALID_SOCKET;
                }
                
            }
            break;
        default:
            DeleteResult(pResult);
            WarThrow(WarError(WAR_ERR_INVALID_CASE_VALUE), "IoOperationE");
        }
    }
    catch(WarException& ex)
    {
        err_log << "Caught unexcpected exception. Operation was "
            << current_op
            << ' '
            << ex.Explain()
            << war_endl;
    }

#if WAR_CATCH_ALL
    catch(...)
    {
         err_log << "Caught unhandled exception. Operation was "
            << current_op
            << war_endl;
    }
#endif 

    DeleteResult(pResult);
}
        

/* Called from either DoListen() or FileIOCompletionRoutine(). 
 * Both these methods guarantees that the object is locked
 * so that properties can be safely accessed.
 */
void WarSocketIoWin32Nt::CreateNewConnectionSocket()
{
    WarLog error_log(WARLOG_ERROR, 
        "WarSocketIoWin32Nt::CreateNewConnectionSocket");
    
    war_socket_t new_socket = socket(AF_INET, SOCK_STREAM, 0);
    
    if (new_socket == WAR_INVALID_SOCKET)
    {        
        error_log << "Failed to create socket for future connection."
            << WarSystemError() 
            << war_endl; 
        return;
    }
    
    pending_ptr_t sck_ptr(
        new WarSocketIoWin32NtPending(new_socket));
    ol_result_t *polData = AllocResult(OP_ACCEPT, 0);
    polData->mAcceptSocket = new_socket;
    DWORD dwBytesReceived = 0;
    
    // Socket is ok. Initiate accept
    if (!AcceptEx(mSocket, 
        new_socket,
        sck_ptr->mBuffer,
        0,
        sizeof(struct sockaddr) + 16,
        sizeof(struct sockaddr) + 16,
        &dwBytesReceived,
        &polData->mOverlappedData))
    {
        WarSystemError system_err;
        
        if (system_err.SystemError() != ERROR_IO_PENDING)
        {
            error_log << "AcceptEx() failed. " 
                << system_err 
                << war_endl;
            
            delete polData;
            return;
        }
    }
    
    // At this point, everything is swell
    mListenQueue.insert(sck_ptr);
}

