#include "StdAfx.h"
#include "WarSvrProtocolFtp.h"   // class implemented

#ifdef ERROR
#   undef ERROR
#endif

#ifndef WAR_LOG_H
#   include "WarLog.h"
#endif

#ifndef WAR_SVR_PROTOCOL_FTP_MACROS_H
#   include "WarSvrProtocolFtpMacros.h"
#endif

#ifndef WAR_SVR_PROTOCOL_FTP_CMD_ALL_H
#   include "WarSvrProtocolFtpCmdAll.h" 
#endif

#ifndef WAR_AUTO_LOCK_H
#   include "WarAutoLock.h"
#endif

#ifndef WAR_SVR_ENGINE_H
#   include "WarSvrEngine.h"
#endif

#ifndef WAR_TIMER_H
#   include "WarTimer.h"
#endif

#ifndef WAR_PERFMON_DEF_H
#   include "WarPerfmonDef.h"
#endif

#ifndef WAR_USER_AUTH_IMPERSONATE_H
#   include "WarUserAuthImpersonate.h"
#endif

#define AUTO_LOCK WarAutoLock MyLock(mSvrLock);
#define AUTO_DATA_LOCK WarAutoLock my_lock(mSvrLock);

/////////////////////////////// MACROS ///////////////////////////////////////

/*
 * Definitions for the TELNET protocol.
 */
#define	IAC	    255		/* interpret as command: */
#define	DONT	254		/* you are not to use option */
#define	DO	    253		/* please, you use option */
#define	WONT	252		/* I won't use option */
#define	WILL	251		/* I will use option */


/////////////////////////////// PUBLIC ///////////////////////////////////////


//============================= LIFECYCLE ====================================

WarSvrProtocolFtp::WarSvrProtocolFtp(war_socket_io_ptr_t& companionPtr,
                                     WarSvrDefinition& svrDefinion)
: WarSvrProtocol(companionPtr, svrDefinion),
mCurrentpendingBytes(0),
mCurrentInputBytes(0),
mCurrentFileLength(0),
mLastTransferTime((war_time_t)0)
{
    WarLog err_log(WARLOG_ERROR, "WarSvrProtocolFtp::WarSvrProtocolFtp()");

    WAR_DB_PERFMON_INC(WAR_PRFDEBUG_NUM_FTPCTL);

    try
    {
        mReply.AssignToServer(this);
        mReply.InstallMacro(war_macro_ptr_t(new MacroProgramname()));
        mReply.InstallMacro(war_macro_ptr_t(new MacroProgramversion()));
        mReply.InstallMacro(war_macro_ptr_t(new MacroPrgcopyright()));
        mReply.InstallMacro(war_macro_ptr_t(new MacroUser(*this)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroOrigin(mRemoteAddress)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroOsname()));
        mReply.InstallMacro(war_macro_ptr_t(new MacroPwd(*this)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroCurrentfilename(mCurrentFile)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroCurrentpath(mCurrentFile)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroRest(mTransferMode)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroCurrenttype(mTransferMode)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroCurrentstructure(mTransferMode)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroCurrentform(mTransferMode)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroCurrentmode(mTransferMode)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroHelpcmds(GetFtpCmds())));
        mReply.InstallMacro(war_macro_ptr_t(new MacroHostdomainname(mLocalAddress)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroRemotedomainname(mRemoteAddress)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroHostaddress(mLocalAddress)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroRemoteaddress(mRemoteAddress)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroRcvbytes(mFtpStat)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroSndbytes(mFtpStat)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroRcvfiles(mFtpStat)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroSndfiles(mFtpStat)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroNumbytes(mFtpStat)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroNumfiles(mFtpStat)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroTrafficinbytes(mFtpStat)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroTrafficoutbytes(mFtpStat)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroTrafficnumbytes(mFtpStat)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroRcvtransfers(mFtpStat)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroSndtransfers(mFtpStat)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroNumtransfers(mFtpStat)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroDataconn(*this)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroFeatures(*this)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroLocaltime()));
        mReply.InstallMacro(war_macro_ptr_t(new MacroGmttime()));
        mReply.InstallMacro(war_macro_ptr_t(new MacroCurrentcmd(mFtpCommands)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroCurrentfilelen(mCurrentFileLength)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroCurrenttransbytes(mLastTransferBytes)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroCurrenttransrate(mLastTransferCps)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroCurrenttranstime(mLastTransferTime)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroDirmsg(*this)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroEmail(*this)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroExplainerr(mLastError)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroGreeting(*this)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroDiskfree(*this)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroIdletimelm(mIdleTimeSeconds)));
        mReply.InstallMacro(war_macro_ptr_t(new MacroSystemname(GetSiteName())));
    }
    catch(WarException& ex)
    {
        err_log << "Caught exception during macro setup. "
            << ex.Explain()
            << " I'm ignoring the error and continuing."
            << war_endl;
    }

    try
    {
        mFtpCommands.AssignToServer(this);
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdUser(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdPass(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdPort(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdPasv(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdType(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdRetr(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdList(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdNlst(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdCwd(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdCdup(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdPwd(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdQuit(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdStor(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdStou(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdRest(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdSyst(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdHelp(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdAcct(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdSmnt(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdRein(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdStru(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdMode(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdAllo(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdAppe(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdRnfr(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdRnto(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdAbor(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdDele(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdRmd(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdMkd(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdStat(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdNoop(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdSite(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdMdtm(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdSize(*this)));
#if WAR_RFC2389
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdFeat(*this)));
        mFtpCommands.InnstallCommand(war_ftpcmd_ptr_t(new FtpCmdOpts(*this)));
#endif // WAR_RFC2389

        mFtpCommands.InstallAlias("XPWD", "PWD");
        mFtpCommands.InstallAlias("XCWD", "CWD");
        mFtpCommands.InstallAlias("XRMD", "RMD");
        mFtpCommands.InstallAlias("XMKD", "MKD");
    }
    catch(WarException& ex)
    {
        err_log << "Caught exception during command setup. "
            << ex.Explain()
            << " I'm ignoring the error and continuing. (I'm stupid!)"
            << war_endl;
    }

}// WarSvrProtocolFtp


WarSvrProtocolFtp::~WarSvrProtocolFtp()
{
    AUTO_DATA_LOCK
    if (!mDataConnectionPtr.IsEmpty())
    {
        if (mDataConnectionPtr->IsOpen())
            mDataConnectionPtr->Abort();
        
        mDataConnectionPtr = NULL;
    }

    WAR_DB_PERFMON_DEC(WAR_PRFDEBUG_NUM_FTPCTL);
}// ~WarSvrProtocolFtp


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

void WarSvrProtocolFtp::Abort(const WarError& reason)
        throw(WarException)
{
    CloseConnection(reason, true);
}

void WarSvrProtocolFtp::CloseConnection(const WarError& reason,
                                        bool doSayGoodbye)
{
    mLastError = reason;

    if (mDataConnectionPtr)
    {
        try
        {
            if (mDataConnectionPtr->IsOpen())
            {
                mDataConnectionPtr->Abort();
                mDataConnectionPtr = NULL;
            }
        }
        catch(WarException)
        {
            ;
        }
    }

    if (IsOpen())
    {
        if (doSayGoodbye)
        {
            try
            {
                if (reason)
                {
                    // Throws if the socket operation fails
                    mReply << SM_CTRLC_CLOSED_ERR 
                        << reason.Explain() 
                        << FTPR_BYE_421;
                }
                else
                {
                    // Throws if the socket operation fails
                    mReply << SM_GOODBYE_QUIT 
                        << FTPR_BYE_421;
                }
            }
            catch(WarException)
            {
                ;
            }
        }
        Close();
    }

    Logout();
    GetEngine().UnregisterSession(*this);

    // Kill references
    WarTimer::GetTimer().KillEvent(
        (WarPtrWrapper<class WarTimerEvent>&)mTimerPtr);
    mTimerPtr = NULL;

    WarThrow(reason, NULL);
}

void WarSvrProtocolFtp::CreateDataConnection() 
throw(WarException)
{
    war_socket_io_ptr_t io_ptr;
    
    io_ptr = mpCompanion->CreateSame();
    mDataConnectionPtr = new WarSvrProtocolFtpDataConn(io_ptr, *this);
    
}

void WarSvrProtocolFtp::InnstallSiteCommand(war_ftpcmd_ptr_t& command,
                                            bool doReplace)
                                            throw(WarException)
{
    ((FtpCmdSite *)(mFtpCommands.GetCmdPtr("SITE")))->InnstallCommand(
        command, doReplace);
}

void WarSvrProtocolFtp::InstallSiteAlias(war_ccstr_t strAlias, 
                                         war_ccstr_t strCommand)
                                         throw(WarException)
{
    ((FtpCmdSite *)(mFtpCommands.GetCmdPtr("SITE")))->InstallAlias(
        strAlias, strCommand);
}

//============================= ACCESS     ===================================


WarSvrProtocolFtpCmd *WarSvrProtocolFtp::GetCmdPtr(war_ccstr_t cmdName)
{
    return mFtpCommands.GetCmdPtr(cmdName);
}

//============================= INQUIRY    ===================================

#if WAR_RFC2389
const std::string WarSvrProtocolFtp::GetFeatures() const
{
    return mFtpCommands.GetFeatures();
}
#endif // WAR_RFC2389


//============================  Callback ===================================

void WarSvrProtocolFtp::OnTimer()
{
    AUTO_LOCK

    war_datasck_ptr_t my_dataconn_ptr;

    {   
        if (!IsOpen())
            return;
        
        if (IsTransfering() && !mDataConnectionPtr.IsEmpty())
        {
            // Check the data connection
            if (mDataConnectionPtr->GetIdleTimeSince().IsTimeOut())
            {
                mDataConnectionPtr->Abort(WAR_ERR_TIMED_OUT);
            }
        }
        else
        {
            // Check our own timer
            if (mIdleTime.IsTimeOut(mIdleTimeSeconds * 1000))
            {
                try
                {
                    CloseConnection(WarError(WAR_ERR_TIMED_OUT), NULL);
                }
                catch(WarException)
                {
                    return;
                }
            }
        }
    }

    return;
}

void WarSvrProtocolFtp::OnTransferStart(bool isSending, 
                                        bool isTmpFile,
                                        bool isStou)
{
#ifdef DEBUG
    if (isStou)
    {
        assert(false == isSending);
    }
#endif

    // Transfer in progress
    if (isStou)
        mReply << SM_STARTING_STOU;
    else if (isSending)
        mReply << SM_STARTING_RETR;
    else
        mReply << SM_STARTING_SEND;
    
    if (!mDataConnectionPtr.IsEmpty() 
        && mDataConnectionPtr->IsOpen())
        mReply << FTPR_USING_OPEN_CONN_OK_125;
    else
        mReply << FTPR_OPENING_CONN_OK_150;
    
    SetConnectionState(isSending ? PUTFILE : GETFILE);   
}

void WarSvrProtocolFtp::OnTransferDone(const WarError& status)
{
    mLastError = status;

    ResetIdleTimer();

    if (mDataConnectionPtr 
        && mDataConnectionPtr->IsOpen())
        mDataConnectionPtr->Close();

    ConnectionStatesE state = GetConnectionState();
    WarLog debug_log(WARLOG_DEBUG, "WarSvrProtocolFtp::OnTransferDone()");
    WarLog trans_log(state == GETFILE ? WARLOG_RCVFILE : WARLOG_SNDFILE, "WarSvrProtocolFtp::OnTransferDone()");
    std::string status_text;
    int st_offset = ((state == GETFILE) ? 0 : 1)
        + (status ? 4 : 0);

    // Update statistics

    if (mDataConnectionPtr)
    {
        mLastTransferBytes 
            = mDataConnectionPtr->GetBytesReceived() 
            + mDataConnectionPtr->GetBytesSent();
        
        mLastTransferTime = mDataConnectionPtr->GetConnectedTime();
        
        if (mLastTransferTime.mTime)
            mLastTransferCps = (mLastTransferBytes * 1000) / mLastTransferTime.mTime;
        else
            mLastTransferCps = 0;

        if (mDataConnectionPtr->GetBytesReceived() > 0)
            AddToSessionFiles(mCurrentFile);
    }
    else
    {
        mLastTransferBytes = 0;
        mLastTransferCps = 0;
        mLastTransferTime.mTime = 0;
    }

    try
    {
        if (st_offset >= 0)
        {
            mFtpStat.AddValue((WarSvrProtocolFtpStat::StatItemE)
                (WarSvrProtocolFtpStat::ST_FILETRANS_BYTES_IN 
                + st_offset), 
                ((st_offset % 1) == 0)
                ? mDataConnectionPtr->GetBytesSent()
                : mDataConnectionPtr->GetBytesReceived());
            
            mFtpStat.AddValue((WarSvrProtocolFtpStat::StatItemE)
                (WarSvrProtocolFtpStat::ST_FILETRANS_FILES_IN 
                + st_offset),
                1);
        }
    } 
    catch(WarException)
    {
       ;
    }

    // Reply
    try
    {
        if (status)
        {
            if (debug_log)
            {
                status_text = "failed with error; ";
                status_text += status.Explain();
            }
            
            // Failed
            switch(status.LocalError())
            {
            case WAR_ERR_VERSION:
            case WAR_ERR_USAGE:
                mReply << SM_GENERIC_FAIL << FTPR_ABORT_451;
                break;
            case WAR_ERR_NAME_EXIST:
                mReply << SM_NAME_EXIST_DENIED_ACCESS << FTPR_ACCESS_DENIED_550;
                break;
            case WAR_ERR_OBJECT_NOT_FOUND:
            case WAR_FERR_NO_SUCH_PATH:
            case WAR_ERR_ACCESS_DENIED:
                mReply << SM_NO_FILE_OR_N0_ACCESS << FTPR_ACCESS_DENIED_550;
                break;
            case WAR_FERR_PATH_IS_A_DIR:
                mReply << SM_GENERIC_FAIL << status.Explain() << FTPR_ACCESS_DENIED_550;
                break;
            case WAR_FERR_RECURSIVE_LISTINGS_FORBIDDEN:
                mReply << SM_RECURSIVE_LISTINGS_FORBIDDEN << FTPR_ACCESS_DENIED_550;
                break;
            case WAR_ERR_INVALID_PATH:
                mReply << SM_ILLEGAL_PATH << FTPR_ARG_SYNTAX_501;
                break;
            case WAR_FTPDERR_NOASCIIREST:
                mReply << SM_NO_ASCII_RESTART << FTPR_ACCESS_DENIED_550;
                break;
            case WAR_FTPDERR_REST_OUT_OF_RANGE:
                mReply << SM_RESTART_OUT_OF_RANGE << FTPR_ACCESS_DENIED_550;
                break;
            case WAR_FTPERR_BAD_SEQUENCE_OF_CMDS:
                mReply << SM_BAD_SEQ_OF_CMDS << FTPR_BAD_SEQ_503;
                break;
            case WAR_FTPDERR_ABORTED:
                mReply << SM_ABOR_INTERRUPTED << FTPR_TRANSFER_ABORTED_DATACONN_CLOSED_426;
                break;
            default:
                switch(GetConnectionState())
                {
                case GETFILE:
                case PUTFILE:
                    mReply << SM_TRANSFER_FAILED << FTPR_TRANSFER_ABORTED_DATACONN_CLOSED_426;
                    break;
                default:
                    mReply << SM_GENERIC_FAIL << status << FTPR_ABORT_451;
                    break;
                }
            }
        }
        else
        {
            // Transfer successful
            status_text = "successfully completed.";
            
            mReply << SM_TRANSFER_DONE_OK << FTPR_TRANSFER_OK_226;
        }
    }
    catch(WarException)
    {
        ; // The reply will throw an expection if the connection is lost
    }
    
    // Write log entries
    try
    {
        if (mDataConnectionPtr)
        {
            if ((mDataConnectionPtr->GetFileFlags() & WarFile::F_TEMPORARY))
            {
                st_offset += 8; // tmp file
                goto db_log;
            }
            
            switch(state)
            {
            case GETFILE:
            case PUTFILE:
                {   
                    trans_log << "Transfer of \""
                        << mCurrentFile.GetUrl().GetUrl()
                        << "\" "
                        << status_text
                        << war_endl;
                }
                break;
            default:
db_log:
                if (debug_log)
                {
                    debug_log << "Transfer of \""
                        << mCurrentFile.GetUrl().GetUrl()
                        << "\" "
                        << status_text
                        << war_endl;
                }
                break;
            }
        }
        else
        {
            st_offset = -1; // Disable
            
            debug_log << "Transfer of uninitalized file "
                << status_text
                << war_endl;
        }
    }
    catch(WarException)
    {
        ;
    }
    
    // Reset transfer parameters
    mTransferMode.mConnectionType = CONN_ACTIVE; 
    mTransferMode.mTransferOperation = TROP_NORMAL;
    mTransferMode.mPortAddress = mRemoteAddress;
    mTransferMode.mPortAddress.SetPort(mLocalAddress.GetPort());
    mTransferMode.mRestartPos = 0;
    
    SetConnectionState(IDLE);
    
    if (mDataConnectionPtr)
        mDataConnectionPtr = NULL;
}

void WarSvrProtocolFtp::OnSent(const WarError& status, 
        war_transfer_buffer_ptr_t& buffer)
{
    AUTO_LOCK
    
    // Update statistics
    mFtpStat.AddValue(WarSvrProtocolFtpStat::ST_CNTR_BYTES_OUT,
        buffer->GetBytesUsed());
    
    
    WarSvrProtocol::OnSent(status, buffer);
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

void WarSvrProtocolFtp::OnClientConnect()
{
    AUTO_LOCK
    // Update internal variables
    // and start receiving commands from the user

    try
    {
        WarSvrProtocol::OnClientConnect();
    }
    catch(WarException& ex)
    {
        switch(ex.LocalError())
        {
        case WAR_ERR_IP_IS_DENIED:
            mReply << SM_BAD_IP << FTPR_BYE_421;
            break;
        case WAR_ERR_ACCESS_DENIED:
            mReply << SM_ACCESS_DENIED << FTPR_BYE_421;
            break;
        default:
            mReply << SM_GENERIC_FAIL << FTPR_BYE_421;
            break;
        }

        CloseConnection(ex, false);
        throw ex;
    }

    // Install a callback timer
    
    mIdleTimeSeconds = WarMax(GetIntOption("ftpd_IDLETIME"), 10);
    WarTime first_run;
    first_run.mTime += (mIdleTimeSeconds * 1000);

    mTimerPtr = new timer_class_t(this, 
        first_run,
        WarTimerEvent::TIMER_LOOP_RELATIVE,
        mIdleTimeSeconds * 1000);
    WarTimer::GetTimer().AddEvent((WarPtrWrapper<class WarTimerEvent>&)mTimerPtr);

    // RFC 959 3.2 Default data port for the user-process
    mTransferMode.mPortAddress = mRemoteAddress;
    mTransferMode.mPortAddress.SetPort(mLocalAddress.GetPort());

    // Local address for outgoing connections (PORT)
    mTransferMode.mLocalAddress = mLocalAddress;
    int my_port = GetIntOption("ftpd_BINDDATA");
    if (my_port == -1)
        my_port = mTransferMode.mLocalAddress.GetPort() -1;
    
    if ((my_port < 0)
        || (my_port > 0xffff))
    {
        WarLog error_log(WARLOG_ERROR, "WarSvrProtocolFtp::OnClientConnect()");

        error_log << "Configuration error. ftpd_BINDDATA has an invalid value. Setting local port to the default."
            << war_endl;

        my_port = mTransferMode.mLocalAddress.GetPort() -1;
        if (my_port < 0)
            my_port = 0; // Just in case. Should never happen.
    }
    mTransferMode.mLocalAddress.SetPort((war_port_t)my_port);

    if (GetBoolOption("ftpd_STELTHSYST"))
        mReply << SM_WELCOME_STELTH << FTPR_READY_220;
    else
        mReply << SM_WELCOME << FTPR_READY_220;
}

#define GETC() (mCurrentpendingBytes ? \
    mPendingBytes[mCurrentpendingBytes--] \
    : (p < end_of_buffer) ? *p++ : EOF)

// Using reverse order to keep it simple !!!
#define UNGETC(ch) \
{\
    if ((MAX_PENDING_BYTES -1) <= mCurrentpendingBytes) \
        goto buffer_exploit;\
    mPendingBytes[mCurrentpendingBytes++] = (char)(ch & 0xff);\
}

void WarSvrProtocolFtp::OnReceived(const WarError& status, 
        war_transfer_buffer_ptr_t& buffer)
{
    AUTO_LOCK

    WarUserAuthImpersonate my_impersonation(GetLoginAuthData());

    // Update statistics
    mFtpStat.AddValue(WarSvrProtocolFtpStat::ST_CNTR_BYTES_IN,
        buffer->GetBytesUsed());

    WarSvrProtocol::OnReceived(status, buffer);

    if (status)
    {
        WarLog error_log(WARLOG_ERROR, "WarSvrProtocolFtp::OnReceived()");

        error_log << "Error on input. Closing connection"
            << status
            << war_endl;
        try
        {
            CloseConnection(status);
        }
        catch(WarException)
        {
            return;
        }
    }

    war_ccstr_t p = buffer->GetBuffer();
    const war_ccstr_t end_of_buffer = p + buffer->GetBytesUsed();
    
    int ch = EOF;
    while((ch = GETC()) != EOF)
    {   
        ch &= 0377;
        switch(ch)
        {
        case '\n':
            if (mCurrentInputBytes && 
                (mInputData[mCurrentInputBytes -1] == '\r'))
            {
                mInputData[--mCurrentInputBytes] = 0;
                OnCommand(mInputData, mCurrentInputBytes);
                mCurrentInputBytes = 0;
            }
            break;
        case IAC: // Telnet
            if ((ch = GETC()) == EOF)
            {
                UNGETC(IAC);
                goto end_of_input;
            }
            ch &= 0377;
            switch(ch)
            {
            case WILL:
            case WONT:
                if ((ch = GETC()) == EOF)
                {
                    UNGETC(WILL); // Reverse order
                    UNGETC(IAC); // Reverse order
                    goto end_of_input;
                }
                {
                    char telnet_buf[4];
                    sprintf(telnet_buf, "%c%c%c", IAC, DONT, 0377&ch);
                    Send(telnet_buf, 3);
                    FlushOut();
                }
                continue;
            case DO:
            case DONT:
                if ((ch = GETC()) == EOF)
                {
                    UNGETC(DO); // Reverse order
                    UNGETC(IAC); // Reverse order
                    goto end_of_input;
                }
                {
                    char telnet_buf[4];
                    sprintf(telnet_buf, "%c%c%c", IAC, WONT, 0377&ch);
                    Send(telnet_buf, 3);
                    FlushOut();
                }
                continue;
            case IAC:
                break;
            default:
                continue;	/* ignore command */
            }
            break; // case IAC
            //  Fall trough
        default:
            if ((MAX_INPUT_BUFFER - 1) <= mCurrentInputBytes)
                goto buffer_exploit;

            mInputData[mCurrentInputBytes] = (char)ch;
            mCurrentInputBytes++;
        }
    }

end_of_input:
    if (IsOpen() && mIoInBufferPtr)
    {
        mIoInBufferPtr->Reset();
        RecvWithCallback(mIoInBufferPtr);
    }
    return;

buffer_exploit:
    mReply << SM_HACKER_BUFFER_ATTACK << FTPR_BYE_421;

    try
    {
        CloseConnection(WarError(WAR_ERR_SECURITY_VIOLATION), false);
    }
    catch(WarException)
    {
        return;
    }

}

void WarSvrProtocolFtp::OnCommand(war_ccstr_t cmdBuffer,  
                                  const size_t bufferLength)
                                  throw(WarException)
{
    WarLog debug_log(WARLOG_DEBUG, "WarSvrProtocolFtp::OnCommand()");
    char cmd_name[MAX_FTP_CMD_LEN + 1];
    char *pp = cmd_name;
    const war_ccstr_t buffer_limit = cmd_name + MAX_FTP_CMD_LEN;
    war_ccstr_t p = cmdBuffer;

    while(*p && (*p != ' '))
    {
        if (buffer_limit <= pp)
        {
            mReply << SM_UNKNOWN_CMD << FTPR_SYNTAX_ERR_500;
            return;
        }
        *pp++ = toupper(*p++);
    }
    *pp = 0;

    if (debug_log)
    {
        if (!strcmp(cmd_name, "PASS"))
            debug_log << cmd_name << " ********" << war_endl;
        else
            debug_log << "FTP command: " << cmdBuffer << war_endl;
    }

    if ((pp - cmd_name) < MIN_FTP_CMD_LEN)
    {
        mReply << SM_UNKNOWN_CMD << FTPR_SYNTAX_ERR_500;
        return;
    }

    mFtpCommands.OnCommand(cmd_name, cmdBuffer, bufferLength);
}

#undef GETC
#undef UNGETC

/////////////////////////////// PRIVATE    ///////////////////////////////////

