
#include "StdAfx.h"
#include "WarSvrProtocolFtpCmdAll.h"   // classes implemented

#ifndef WAR_LOG_H
#   include "WarLog.h"
#endif

#ifndef WAR_USER_ENGINE_H
#   include "WarUserEngine.h"
#endif

#ifndef WAR_SVR_PROTOCOL_FTP_DATA_CONN_H
#   include "WarSvrProtocolFtpDataConn.h"
#endif

#ifndef WAR_SVR_DIR_LISTING_H
#   include "WarSvrDirListing.h"
#endif

#ifndef WAR_DIRFMT_UNIX_LS_H
#   include "WarDirfmtUnixLs.h"
#endif

#ifndef WAR_ARGS_H
#   include "WarArgs.h"
#endif

#ifndef WAR_ATOI_H
#   include "WarAtoi.h"
#endif

#ifndef WAR_AUTO_LOCK_H
#   include "WarAutoLock.h"
#endif

#ifndef WAR_FSYS_H
#   include "WarFsys.h"
#endif


#include <fstream>

using namespace std;


#define AUTO_DATA_LOCK \
        WarAutoLock my_lock(GetServer().mFtpDataLock);


#if 0 // Impl. template
/////////////////////////////// XX ///////////////////////////////////////

XX::XX(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("COMMAND", 
                       "commandExplanation", 
                       "commandSyntax",
                       "commandReference",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE XX::ExecuteCommand()
{
    return CMD_SUCCESS;
}
#endif // 0

/////////////////////////////// USER AND PASS ///////////////////////////////////////


FtpCmdAuthBase::FtpCmdAuthBase(war_ccstr_t commandName, 
                               war_ccstr_t commandExplanation, /* Human help*/
                               war_ccstr_t commandSyntax, /* Formal */
                               war_ccstr_t commandReference, /* RFC */
                               WarSvrProtocolFtp& serverCompanion)
                               :WarSvrProtocolFtpCmd(commandName, 
                               commandExplanation,
                               commandSyntax,
                               commandReference,
                               serverCompanion)
{
}

bool FtpCmdAuthBase::Login(war_ccstr_t virtualHost, 
                           war_ccstr_t userName, 
                           war_ccstr_t userPasswd)
{
    WarLog err_log(WARLOG_ERROR, "FtpCmdAuthBase::Login()");
    war_auth_ptr_t auth_module_ptr;
    war_authdata_ptr_t session_data_ptr;

    WarLoginResultE result = GetServer().Login(virtualHost, 
        userName, 
        userPasswd);

    switch(result)
    {
        
    case LF_OK:
        {
            WarLog auth_log(WARLOG_LOGIN, "FtpCmdAuthBase::Login()");

            try
            {
                GetServer().SetCwd();
            }
            catch(WarException& ex)
            {
                err_log << "Failed to login after a successful authentication "
                    "because it was impossible to set the users current "
                    "(home) directory. Please verify the user_HOME option for "
                    "this user."
                    << ex
                    << war_endl;

                *mpReply << SM_GENERIC_FAIL << FTPR_BYE_421;
                GetServer().CloseConnection(WarError(), false);
                break;
            }
            
            auth_log << GetServer()
                << " User logged in."
                << war_endl;
            
            *mpReply << SM_LOGGED_IN << FTPR_LOGIN_OK_230;
            GetServer().SetConnectionState(IDLE);
        }
        return true;
    case LF_NEED_PASSWORD:
fool_me:
        *mpReply << SM_NEED_PASSWD << FTPR_NEED_PASSWORD_331;
        break;
    case LF_NEED_EMAIL_AS_PWD:
        *mpReply << SM_NEED_EMAIL << FTPR_NEED_PASSWORD_331;
        break;
    case LF_TOO_LONG_USERNAME:
    case LF_TOO_LONG_PASSWORD:
    case LF_TOO_LONG_HOSTNAME:
    case LF_INTERNAL_ERROR:
        err_log << GetArgument() 
            << " - Login() failed due to error FtpSvrMsgsE::SM_#"
            << (int)result
            << war_endl;
        
        *mpReply << SM_ERR_BYE << FTPR_BYE_421;
        GetServer().CloseConnection(WarError(), false);
        break;
    case LF_THROW_USER_OFF:
        *mpReply << SM_DONT_KNOW_YOU << FTPR_BYE_421;
        GetServer().CloseConnection(WarError(), false); 
        break;
    case LF_TIME_OF_DAY_DENIAL:
        *mpReply << SM_RIGHT_PLACE_WRONG_TIME << FTPR_BYE_421;
        GetServer().CloseConnection(WarError(), false);
        break;
    case LF_IP_SHITLISTED:
        *mpReply << SM_BAD_IP << FTPR_BYE_421;
        GetServer().CloseConnection(WarError(), false);
        break;
    case LF_IP_MARKED_AS_HACKER:
        *mpReply << SM_HACKERS_NOT_WELCOME << FTPR_BYE_421;
        GetServer().CloseConnection(WarError(), false);
        break;
    case LF_TOO_MANY_FAILED_LOGINS:
        *mpReply << SM_TOO_MANY_FAILED_LOGINS << FTPR_BYE_421;
        GetServer().CloseConnection(WarError(), false);
        break;
    case LF_TOO_MANY_CONNECTIONS:
        *mpReply << SM_TMC_ACCOUNT << FTPR_BYE_421;
        GetServer().CloseConnection(WarError(), false);
        break;
    case LF_TOO_MANY_CONNECTIONS_TOTAL:
         *mpReply << SM_TMC_ACCOUNT << FTPR_BYE_421;
        GetServer().CloseConnection(WarError(), false);
        break;
    case LF_TOO_MANY_CONNECTIONS_FROM_IP:
        *mpReply << SM_TMC_IP << FTPR_BYE_421;
        GetServer().CloseConnection(WarError(), false);
        break;
    case LF_SYSTEM_SHUTDOWN:
        *mpReply << SM_SHUTTING_DOWN << FTPR_BYE_421;
        GetServer().CloseConnection(WarError(), false);
        break;
    case LF_NO_ANONYMOUS_USERS:
        *mpReply << SM_ANON_DENIED << FTPR_BYE_421;
        GetServer().CloseConnection(WarError(), false);
        break;
    case LF_CONFIGURATION_ERROR:
        *mpReply << SM_SERVER_CONFIGURATION_ERROR << FTPR_BYE_421;
        GetServer().CloseConnection(WarError(), false);
        break;
    case LF_CALLER_NOT_AUTHENTICATED:
    default:
        if (userPasswd)
            *mpReply << SM_PASSWD_NOT_ACCEPTED << FTPR_NOT_LOGGED_IN_530;
        else
            goto fool_me;  
        break;
    }

    return false;
}

/////////////////////////////// USER ///////////////////////////////////////

FtpCmdUser::FtpCmdUser(WarSvrProtocolFtp& serverCompanion)
: FtpCmdAuthBase("USER", 
                 "Log on with username", 
                 "USER <SP> <username> <CRLF>",
                 "RFC 959",
                 serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdUser::ExecuteCommand()
{

    if (Login(NULL, 
        GetArgument(), 
        NULL))
    {
        return CMD_SUCCESS;
    }

    return CMD_FAIL;
}

/////////////////////////////// PASS ///////////////////////////////////////

FtpCmdPass::FtpCmdPass(WarSvrProtocolFtp& serverCompanion)
: FtpCmdAuthBase("PASS", 
                 "Login with password", 
                 "PASS <SP> <password> <CRLF>",
                 "RFC 959",
                 serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdPass::ExecuteCommand()
{
    FtpCmdUser *pUserCmd = (FtpCmdUser *)GetServer().GetCmdPtr("USER");

    if (!pUserCmd)
        WarThrow(WarError(WAR_ERR_INTERNAL_DATA_NOT_INITIALIZED), "missing FtpCmdUser");

    if (Login(NULL, 
        pUserCmd->GetArgument(),
        GetArgument()))
    {
        return CMD_SUCCESS;
    }

    return CMD_FAIL;
}

/////////////////////////////// PORT ///////////////////////////////////////


FtpCmdPort::FtpCmdPort(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("PORT", 
                       "Data port", 
                       "<SP> <host-port> <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdPort::ExecuteCommand()
{
    if (!GetServer().GetBoolOption("ftpd_DOPORT"))
	{
		if (GetServer().GetBoolOption("ftpd_DOPASV"))
            *mpReply << SM_PORT_DISABLED << FTPR_NOT_IMPL_502;
		else
		{
            *mpReply << "Server misconfiguration. PORT and PASV commands are both disabled." 
                << FTPR_NOT_IMPL_502;

            WarLog err_log(WARLOG_ERROR, "FtpCmdPort::ExecuteCommand()");
            err_log << "***SERVER MISCONFIGURATION. Both PORT and PASV commands are disabled. Please enable at least one of the modes! No files can be transfered until the error is corrected!"
                << war_endl;
		}
		return CMD_FAIL;
	}

    sockaddr Sockaddr;
    memset(&Sockaddr, 0, sizeof(Sockaddr));
	sockaddr_in *s_in = (sockaddr_in *)&Sockaddr;
	int i1, i2, i3, i4, i5, i6;
	
	if (sscanf(GetArgument(),"%d,%d,%d,%d,%d,%d", 
        &i1, &i2, &i3, &i4, &i5, &i6) != 6)
	{
        *mpReply << GetArgument() 
            << ": Must be 6 digits. (IPv4)" 
            << FTPR_ARG_SYNTAX_501;
		return CMD_FAIL;
	}

    unsigned char *p = (unsigned char *)&s_in->sin_addr;
    *p++ = (unsigned char)i1;
    *p++ = (unsigned char)i2;
    *p++ = (unsigned char)i3;
    *p++ = (unsigned char)i4;
	s_in->sin_port = htons((i5 << 8) | i6);
    s_in->sin_family = AF_INET;

#if WAR_RFC2577
    // Check network address
    if ((htonl(INADDR_ANY) == s_in->sin_addr.s_addr)
        || (htonl(INADDR_NONE) == s_in->sin_addr.s_addr)
        || (htonl(INADDR_BROADCAST) == s_in->sin_addr.s_addr))
    {
        WarLog sec_log(WARLOG_SECURITY, "FtpCmdPort::ExecuteCommand()");
		
        sec_log << GetArgument() 
            << ": refused PORT command due to invalid IP address: "
            << inet_ntoa(s_in->sin_addr)
            << war_endl;

        *mpReply << SM_INVALID_IP_ADDRESS << FTPR_CMD_NOT_IMPL_ARG_504;
        return CMD_FAIL;
    }

    const struct in_addr& remote_host = GetServer().GetRemoteAddress().GetInAddr();
    if ((s_in->sin_addr.s_addr != remote_host.s_addr)
        // Special case - on the local machine, the loopback and real address max mix
        && !((remote_host.s_addr == htonl(INADDR_LOOPBACK)) && GetServer().IsLocalAddress(s_in->sin_addr))
        && !GetServer().GetBoolOption("ftpd_ALLOWFTPPROXY"))
    {
        WarLog sec_log(WARLOG_SECURITY, "FtpCmdPort::ExecuteCommand()");
		
        sec_log << GetArgument() 
            << ": refused PORT command. Server to server (proxy) is disabled (ftpd_ALLOWFTPPROXY option): "
            << inet_ntoa(s_in->sin_addr)
            << war_endl;

        *mpReply << SM_NO_FTP_PROXY << FTPR_CMD_NOT_IMPL_ARG_504;
        return CMD_FAIL;
    }

    // Check port
	if (GetServer().GetBoolOption("ftpd_DOPRIVPORT") &&
        ((ntohs(s_in->sin_port) < 1024)))
	{
        WarLog sec_log(WARLOG_SECURITY, "FtpCmdPort::ExecuteCommand()");
		
        sec_log << GetArgument() 
            << ": refused PORT command due to ftpd_DOPRIVPORT setting"
            << war_endl;

        *mpReply << SM_PORT_RESTRICTED << FTPR_CMD_NOT_IMPL_ARG_504;
        return CMD_FAIL;
	}
#endif // WAR_RFC2577

    *mpReply << SM_PORT_OK << FTPR_OK_200;
    
    GetServer().mTransferMode.mPortAddress.AssignSockaddr(
        &Sockaddr, sizeof(Sockaddr));
    GetServer().mTransferMode.mConnectionType = CONN_ACTIVE;
    
    WarLog debug_log(WARLOG_DEBUG, "FtpCmdPort::ExecuteCommand()");
    if (debug_log)
    {
        debug_log << "PORT " 
            << GetServer().mTransferMode.mPortAddress.Explain()
            << war_endl;
    }

    return CMD_SUCCESS;
}

FtpCmdPasv::FtpCmdPasv(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("PASV", 
                       "Request a passive connection (the server listens for a connection)", 
                       "PASV <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdPasv::ExecuteCommand()
{
    WarNetAddress local_addr;

    if (!GetServer().GetBoolOption("ftpd_DOPASV"))
    {
        if (GetServer().GetBoolOption("ftpd_DOPORT"))
            *mpReply << SM_PASV_DISABLED << FTPR_NOT_IMPL_502;
        else
        {
            *mpReply << "Server misconfiguration. PORT and PASV commands are both disabled." 
                << FTPR_NOT_IMPL_502;
            
            WarLog err_log(WARLOG_ERROR, "FtpCmdPasv::ExecuteCommand()");
            err_log << "***SERVER MISCONFIGURATION. Both PORT and PASV commands are disabled. Please enable at least one of the modes! No files can be transfered until the error is corrected!"
                << war_endl;
        }
        return CMD_FAIL;
    }

    if (!GetServer().mDataConnectionPtr.IsEmpty())
    {
        /* Some DOS attacks may involve flooding the server with
           PASV commands. We will never listen to more than one of these
           at one time, but creating and setting up listening sockets may
           be expensive. We will therefore not accept a PASV command 
           until the pervious file transfer has completed, or the 
           passive socket timed out.
        */
        
        *mpReply << SM_PASV_FLOOD; 
        if (GetServer().GetBoolOption("ftpd_STRICTREPLY"))
            *mpReply << FTPR_SYNTAX_ERR_500;
        else
            *mpReply << FTPR_ABORT_451; // Not RFC 959 return code !! 
        return CMD_FAIL;
    }

    try
    {
        GetServer().CreateDataConnection();
        WarSvrProtocolFtpDataConn& rSck = *(GetServer().mDataConnectionPtr);

        local_addr = GetServer().GetLocalAddress();
        local_addr.SetPort(0);
        pair<war_port_t, war_port_t> range, *prange = NULL;

        string port_range = GetServer().GetOption("ftpd_PASVRANGE");
        if (!port_range.empty())
        {
            unsigned i1 = 0, i2 = 0;
            if ((sscanf(port_range.c_str(), "%u-%u", &i1, &i2) == 2)
                && (i1 < i2) && (i2 <= 0xffff))
            {
                range.first = (war_port_t)i1;
                range.second = (war_port_t)i2;
                prange = &range;
            }
            else
            {
                WarLog err_log(WARLOG_ERROR, "FtpCmdPasv::ExecuteCommand()");

                err_log << "Invalid ftpd_PASVRANGE value: " 
                    << port_range
                    << ". Ignoring this setting!"
                    << war_endl;
            }
        }
        rSck.Listen(local_addr, 1, prange);
    }
    catch(WarException e)
    {
        *mpReply << "Failed to open passive socket. " 
            << e.Explain() ;
        
        if (GetServer().GetBoolOption("ftpd_STRICTREPLY"))
            *mpReply << FTPR_SYNTAX_ERR_500;
        else
            *mpReply << FTPR_ABORT_451; // Not RFC 959 return code !! 
        return CMD_FAIL;
    }

    try
    {
        const struct sockaddr_in& rsckaddr_in 
            = GetServer().mDataConnectionPtr->GetLocalAddress().GetSockAddrIn();
        
        const unsigned char *paddr = (unsigned char *)&rsckaddr_in.sin_addr;
        const unsigned char *pport = (unsigned char *)&rsckaddr_in.sin_port;
        
        *mpReply << SM_PASV_OK 
            << " ("
            << (int)(unsigned)paddr[0] << ","
            << (int)(unsigned)paddr[1] << ","
            << (int)(unsigned)paddr[2] << ","
            << (int)(unsigned)paddr[3] << ","
            << (int)(unsigned)pport[0] << ","
            << (int)(unsigned)pport[1]
            << ")"
            << FTPR_PASV_OK_227;
    }
    catch(WarException e)
    {
        *mpReply << "Failed to express PASV reply. " 
            << e.Explain();

        if (GetServer().GetBoolOption("ftpd_STRICTREPLY"))
            *mpReply << FTPR_SYNTAX_ERR_500;
        else
            *mpReply << FTPR_ABORT_451; // Not RFC 959 return code !! 
        return CMD_FAIL;
    }

    GetServer().mTransferMode.mConnectionType = CONN_PASSIVE;

    WarLog debug_log(WARLOG_DEBUG, "FtpCmdPort::ExecuteCommand()");
    if (debug_log)
    {
        debug_log << "PASV " 
            << GetServer().mTransferMode.mPortAddress.Explain()
            << war_endl;
    }

    return CMD_SUCCESS;
}

/////////////////////////////// TYPE ///////////////////////////////////////

FtpCmdType::FtpCmdType(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("TYPE", 
                       "Select data type", 
                       "TYPE <SP> <type-code> <CRLF>",
                       "RFC 950",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdType::ExecuteCommand()
{
    war_ccstr_t p = GetArgument();
    {
        while(isspace(*p))
            ++p;

        if (!*p)
        {
            *mpReply << SM_MISSING_PARAMETER_IN_CMD << FTPR_ARG_SYNTAX_501;
            return CMD_FAIL;
        }

        switch(toupper(*p))
        {
        case 'A':
            while(isspace(*p))
                ++p;
            switch(toupper(*p))
            {
            case 'T':
            case 'C':
                *mpReply << SM_PARAMETER_NOT_SUPPORTED 
                    << FTPR_CMD_NOT_IMPL_ARG_504;
                return CMD_FAIL;
            case 'N':
            default:
                *mpReply << SM_TYPE_A << FTPR_OK_200;
                GetServer().mTransferMode.mType = TYPE_ASCII;
                GetServer().mTransferMode.mForm = FORM_NONPRINT;
                return CMD_SUCCESS;
            }
        case 'E':
            *mpReply << SM_TYPE_E << FTPR_CMD_NOT_IMPL_ARG_504;
            return CMD_FAIL;
        case 'I':
            *mpReply << SM_TYPE_I << FTPR_OK_200;
            GetServer().mTransferMode.mType = TYPE_IMAGE;
            return CMD_SUCCESS;
        case 'L':
             while(isspace(*p))
                ++p;
             if (*p && (atoi(p) == 8))
             {
                 *mpReply  << SM_TYPE_L8 << FTPR_OK_200;
                 GetServer().mTransferMode.mType = TYPE_LOCALB;
                 return CMD_SUCCESS;
             }
             *mpReply << SM_PARAMETER_NOT_SUPPORTED 
                 << FTPR_CMD_NOT_IMPL_ARG_504;
             return CMD_FAIL;
        }
    }
    return CMD_FAIL; // Compiler food
}

/////////////////////////////// TRANSFER ///////////////////////////////////////

FtpCmdTransfer::FtpCmdTransfer(war_ccstr_t commandName, 
                               war_ccstr_t commandExplanation, /* Human help*/
                               war_ccstr_t commandSyntax, /* Formal */
                               war_ccstr_t commandReference, /* RFC */
                               WarSvrProtocolFtp& serverCompanion)
                               : WarSvrProtocolFtpCmd(commandName, 
                               commandExplanation,
                               commandSyntax,
                               commandReference,
                               serverCompanion)
{
}

war_svrfile_ptr_t 
FtpCmdTransfer::OpenFile(const WarSvrPath& svrFile, 
                         war_uint32_t flags)
                         throw(WarException)
{

    if ((flags & (F_WRITE | F_APPEND | F_CREATE))
        && svrFile.IsSharedUpload()
        && !GetServer().IsFileOwnedBySession(svrFile))
    { 
        if (flags & F_MUSTEXIST)
        {
            WarLog db_log(WARLOG_DEBUG, "FtpCmdTransfer::OpenFile()");
            
            db_log << "Denying access to overwrite of file in shared upload folder."
                << svrFile
                << war_endl;
            
            WarThrow(WarError(WAR_ERR_ACCESS_DENIED), NULL);
        }
        flags |= F_NOTEXIST;
        flags &= ~F_TRUNCATE;
    }
    else if ((flags & F_READ) && svrFile.IsSharedUpload())
    {
        WarLog db_log(WARLOG_DEBUG, "FtpCmdTransfer::OpenFile()");

        db_log << "Denying access to download from shared upload folder."
            << svrFile
            << war_endl;

        WarThrow(WarError(WAR_ERR_ACCESS_DENIED), NULL);
    }

    war_svrfile_ptr_t file_ptr 
        = GetServer().OpenFile(svrFile, flags);

    const war_flen_t rest_pos = GetServer().mTransferMode.mRestartPos;

    if (0 < rest_pos)
    {
        if (stricmp(GetServer().GetFtpCmds().GetPreviousCommand().GetName().c_str(),
            "REST"))
        {
            // draft-ietf-ftpext-mlst-11 demands that REST is the last
            // command to issue before a transfer command.
            WarThrow(WarError(WAR_FTPERR_BAD_SEQUENCE_OF_CMDS), NULL);
        }

        if ((flags & F_TEXT)
            && !GetServer().GetBoolOption("ftpd_ALLOWASCIIRESTART"))
            WarThrow(WarError(WAR_FTPDERR_NOASCIIREST), NULL);

        if (rest_pos > file_ptr->GetLength())
            WarThrow(WarError(WAR_FTPDERR_REST_OUT_OF_RANGE), NULL);

        file_ptr->Seek(rest_pos);
    }

    if (flags & F_TEMPORARY)
        GetServer().mCurrentFileLength = -1;
    else
        GetServer().mCurrentFileLength = file_ptr->GetLength();
    
    return file_ptr;
}

void FtpCmdTransfer::SendFile(
        const war_ccstr_t filePath)
        throw(WarException)
{
    GetServer().ResolvPath(filePath);

    war_uint32_t my_flags = F_READ | F_SHREAD 
        | F_MUSTEXIST | F_SEQUENTIAL;

    if (TYPE_ASCII == GetServer().mTransferMode.mType)
        my_flags |= WarFileEnums::F_TEXT;

    war_svrfile_ptr_t my_file_ptr 
        = OpenFile(GetServer().mCurrentFile, my_flags);

    GetServer().OnTransferStart(true, false);
    Connect();
    GetServer().mDataConnectionPtr->SendFile(my_file_ptr);
}

void FtpCmdTransfer::ReceiveFile(
        const war_ccstr_t filePath,
        bool withUniqueName,
        bool doAppend)
        throw(WarException)
{
    war_uint32_t my_flags = 0;
    GetServer().ResolvPath(filePath);

    if (withUniqueName)
    {
        if (!GetServer().GetBoolOption("ftpd_ALLOWSTOU"))
        {
            *mpReply << SM_COMMAND_FORBIDDEN << FTPR_NOT_IMPL_502;
            return;
        }
        
        /* Unique filename
           This is tricky, as this command is rarely used,
           but _are_ used in some special applications.
           The normal approach is to count upwards 
           
             sprintf(buf, "name.%d", ++counter);
           
           and test if a file with that number exists. If the
           counter exeeds a (tiny) limit, the command fails.
           This approach is cheap, but saves system resources.

           We try another approach here, by searching for 
           files with our name, but without the number,
           and simply adding one to the last number found!
    
             dir_list --> "file.?????.ext"

           Where ????? is a number.
           This makes this command extreemly expensive, but
           usable in industrial applications, as it can handle
           up to 99.999 STOU commands in one directory. The bad
           news is that we only look for the highest number, and
           not holes (deleted files) in the series.

           The command should be disabled on most systems.
        */

        // We don't allow pattern chacaters here, as we will use
        // these for lookups
        for(war_svrpath_ccstr_t p = GetServer().mCurrentFile.GetAlias().GetPath()
            ; *p
            ; p++)
        {
            if ((*p == '*') || (*p == '?') || (*p == '['))
                WarThrow(WarError(WAR_ERR_INVALID_PATH), NULL);
        }

        war_svrpath_t my_path = GetServer().mCurrentFile.GetAlias().GetPathname();
        war_svrpath_t my_prefix = GetServer().mCurrentFile.GetAlias().GetFilename();
        war_svrpath_t my_postfix = GetServer().mCurrentFile.GetAlias().GetExtension();
        if (!my_postfix.IsEmpty())
            my_prefix.CutAtLastCh('.');

        war_svrpath_t my_pattern;
        my_pattern << my_prefix.GetPath() 
            << ".?????";
        if (!my_postfix.IsEmpty())
            my_pattern << "."
            << my_postfix.GetPath();
        
        WarSvrDirList dir_list(0, GetServer(), false);
        dir_list.mPatternList.push_back(my_pattern);
        dir_list.Create(WarUtf8(my_path.GetPath()).GetUtf8().c_str());

        size_t counter = 1;
        const size_t match_punkts = my_postfix.IsEmpty() ? 1 : 2;
        const char match_last = (match_punkts == 2) ? '.' : 0;
        for(WarDirList::nodelist_t::reverse_iterator P 
            = dir_list.mList.rbegin()
            ; P != dir_list.mList.rend()
            ; P++)
        {
            war_ccstr_t p_start = P->Data().mName, 
                p = p_start + P->Data().mNameLen -1;

            int match = 0;
            for(; p > p_start; p--)
            {
                if (('.' == *p) && (match_punkts == ++match))
                {
                    if (isdigit(p[1])
                        && isdigit(p[2])
                        && isdigit(p[3])
                        && isdigit(p[4])
                        && isdigit(p[5])
                        && (match_last == p[6]))
                    {
                        // We got the highest match!
                        counter = atoi(p + 1) + 1;
                        goto do_stou;
                    }
                }
            }
        }

do_stou:
        char buf[32];
        sprintf(buf, ".%05d", counter);

        war_svrpath_t my_unique_path;
        my_unique_path << my_path.GetPath() 
            << WAR_SYSSLASH 
            << my_prefix.GetPath() 
            << buf;
        if (!my_postfix.IsEmpty())
            my_unique_path << "." << my_postfix.GetPath();

        GetServer().ResolvPath(WarUtf8(my_unique_path.GetPath()).GetUtf8().c_str());

        my_flags = F_CREATE | F_WRITE | F_NOTEXIST | F_SEQUENTIAL;

    }
    else
    {
        if (GetServer().mTransferMode.mRestartPos > 0)
            my_flags = F_WRITE | F_MUSTEXIST | F_SEQUENTIAL | F_SHREAD;
        else if (doAppend)
            my_flags = F_WRITE | F_CREATE | F_APPEND | F_SEQUENTIAL | F_SHREAD;
        else
            my_flags = F_WRITE | F_TRUNCATE | F_SEQUENTIAL | F_SHREAD;
    }

    if (TYPE_ASCII == GetServer().mTransferMode.mType)
        my_flags |= WarFileEnums::F_TEXT;

    war_svrfile_ptr_t my_file_ptr 
        = OpenFile(GetServer().mCurrentFile, my_flags);

    GetServer().OnTransferStart(false, false, withUniqueName);
    Connect();
    GetServer().mDataConnectionPtr->ReceiveFile(my_file_ptr);
}

void FtpCmdTransfer::Connect()
    throw(WarException)
{
    WarSvrProtocolFtp& rsvr = GetServer();
    if (CONN_ACTIVE == rsvr.mTransferMode.mConnectionType)
    {

        // Have to initiate a connectiopn to the client...
        if (!rsvr.mDataConnectionPtr.IsEmpty())
            rsvr.mDataConnectionPtr = NULL;
        
        rsvr.CreateDataConnection();
        rsvr.mDataConnectionPtr->SetAsyncMode();
        rsvr.mDataConnectionPtr->Connect(
            WAR_SOCK_STREAM,
            rsvr.mTransferMode.mPortAddress,
            &rsvr.mTransferMode.mLocalAddress);
    }
    else
    {
        // Passive connection.
        if (rsvr.mDataConnectionPtr.IsEmpty())
        {
            WarLog err_log(WARLOG_ERROR, "FtpCmdTransfer::Connect()");

            err_log << "No socket for passive transfer!. Internal error."
                << war_endl;

            WarThrow(WarError(WAR_FTPDERR_NO_DATA_CONNECTION), NULL);
        }
    }
}



/////////////////////////////// RETR ///////////////////////////////////////

FtpCmdRetr::FtpCmdRetr(WarSvrProtocolFtp& serverCompanion)
: FtpCmdTransfer("RETR", 
                 "Retrieve (download) a file", 
                 "RETR <SP> <pathname> <CRLF>",
                 "RFC 959",
                 serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdRetr::ExecuteCommand()
{
    try
    {
        SendFile(GetArgument());
    }
    catch(WarException& ex)
    {
        GetServer().OnTransferDone(ex);
        return CMD_FAIL;
    }

    return CMD_SUCCESS;
}

/////////////////////////////// LIST ///////////////////////////////////////


FtpCmdList::FtpCmdList(WarSvrProtocolFtp& serverCompanion)
: FtpCmdTransfer("LIST", 
                 "List directory", 
                 "LIST [<SP> <pathname>] <CRLF>",
                 "RFC 959",
                 serverCompanion),
                 mDoForceSimple(false)
{
}

FtpCmdList::FtpCmdList(war_ccstr_t commandName, 
                     war_ccstr_t commandExplanation, /* Human help*/
                     war_ccstr_t commandSyntax, /* Formal */
                     war_ccstr_t commandReference, /* RFC */
                     WarSvrProtocolFtp& serverCompanion)
                     : FtpCmdTransfer(commandName, 
                       commandExplanation, 
                       commandSyntax,
                       commandReference,
                       serverCompanion),
                       mDoForceSimple(false)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdList::ExecuteCommand()
{
    int list_flags = 0;
    GetServer().mCurrentFileLength = -1;

    WarDirfmtUnixLs<ofstream> UnixLs;

    string tmp_dir_str = GetServer().GetOption("core_TMPDIR");

    const char *tmp_dir = tmp_dir_str.c_str();
    char *temp_name = NULL;

    if (!tmp_dir || !*tmp_dir)
    {
        if ((tmp_dir = ::getenv("TEMP")) == NULL)
            tmp_dir = ::getenv("TMP");
    }

    if (tmp_dir && !*tmp_dir)
        tmp_dir = NULL;

    if ((temp_name = tempnam(tmp_dir, "dirlist.tmp")) == NULL)
    {
        WarThrow(WarError(WAR_ERR_OUT_OF_MEMORY), "tempnam");
    }

    string tmpname_url = "file:///";
    string tmpname_str = temp_name;
    tmpname_url += temp_name;
    
    try
    {
        // Build a dir-list in a file (for now)
        ofstream out_file(tmpname_str.c_str());

        WarSvrPath tmp_path("ls -l", tmpname_url.c_str(),
            ALLOW_READ | ALLOW_WRITE | ALLOW_DELETE);

        // Create a dir-listing
        WarDirfmtUnixLs<ofstream>::seq_file_list_t file_list;

        WarArgs<char> my_args;
        my_args.mArgMode = true;

        if (mDoForceSimple)
            my_args << "ls" << "-1" << "--" << GetArgument();
        else
            my_args << "ls" << GetArgument();

        UnixLs.ParseArgs(my_args.GetArgc(), 
            my_args.GetArgv(), 
            file_list);

        WarSvrDirListing<WarDirfmtUnixLs<ofstream>, ofstream> 
            my_listing(
            UnixLs, 
            out_file, 
            list_flags, 
            GetServer());

        if (file_list.empty())
            file_list.push_front(WarUtf8(GetServer().GetCwd().GetAlias().GetPath()).GetUtf8());

        my_listing.Create(file_list.front().c_str());
        out_file.close();

        // Send the file
        GetServer().mCurrentFile = WarSvrPath("ls -l", 
            tmpname_url.c_str(), 0);

        war_svrfile_ptr_t listfile_ptr = new WarSvrFile;
        listfile_ptr->Open(tmp_path,
            WarFile::F_READ
            | WarFile::F_WRITE
            | WarFile::F_TEMPORARY
            | WarFile::F_MUSTEXIST
            | WarFile::F_SEQUENTIAL
            | WarFile::F_CALLBACK);

        GetServer().OnTransferStart(true, false);
        Connect();
        GetServer().mDataConnectionPtr->SendFile(listfile_ptr);
    }
    catch(WarException& ex)
    {
        if (!UnixLs.mErrorMessage.empty())
            *mpReply << UnixLs.mErrorMessage;
        
        GetServer().OnTransferDone(ex);
        return CMD_FAIL;
    }

    return CMD_SUCCESS;
}

/////////////////////////////// FtpCmdNlst ///////////////////////////////////////

FtpCmdNlst::FtpCmdNlst(WarSvrProtocolFtp& serverCompanion)
: FtpCmdList("NLST", 
             "List directory", 
             "NLST [<SP> <pathname>] <CRLF>",
             "commandReference",
             serverCompanion)
{
    mDoForceSimple = true;
}


/////////////////////////////// FtpCmdCwd ///////////////////////////////////////

FtpCmdCwd::FtpCmdCwd(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("CWD", 
                       "Change the current directory", 
                       "CWD <SP> <pathname> <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

FtpCmdCwd::FtpCmdCwd(war_ccstr_t commandName, 
                     war_ccstr_t commandExplanation, /* Human help*/
                     war_ccstr_t commandSyntax, /* Formal */
                     war_ccstr_t commandReference, /* RFC */
                     WarSvrProtocolFtp& serverCompanion)
                     : WarSvrProtocolFtpCmd(commandName, 
                       commandExplanation, 
                       commandSyntax,
                       commandReference,
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdCwd::ExecuteCommand()
{
    return Cwd(GetArgument());
}
    
WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdCwd::Cwd(war_ccstr_t newCwd)
{
    try
    {
        GetServer().SetCwd(newCwd);
    }
    catch(WarException& e)
    {
        switch(e.LocalError())
        {
        case WAR_FERR_NOT_A_DIRECTORY:
        case WAR_FERR_NO_SUCH_PATH:
        case WAR_ERR_ACCESS_DENIED:
            *mpReply << SM_CWD_FAIL << FTPR_ACCESS_DENIED_550;
            break;
        case WAR_ERR_INVALID_PATH:
            *mpReply << SM_ILLEGAL_PATH << FTPR_ACCESS_DENIED_550;
            break;
        default:
            *mpReply << SM_CWD_FAIL
                << e.Explain()
                << FTPR_ACCESS_DENIED_550;
        }

        return CMD_FAIL;
    }

    *mpReply << SM_CWD_OK << FTPR_FILE_ACTION_OK_250;

    return CMD_SUCCESS;
}

/////////////////////////////// FtpCmdPwd ///////////////////////////////////////

FtpCmdPwd::FtpCmdPwd(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("PWD", 
                       "Prints the current working directory", 
                       "PWD <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdPwd::ExecuteCommand()
{
    *mpReply 
        << SM_PWD_OK
        << FTPR_PATHNAME_OK_257;
    
    return CMD_SUCCESS;
}


/////////////////////////////// FtpCmdCdup ///////////////////////////////////////

FtpCmdCdup::FtpCmdCdup(WarSvrProtocolFtp& serverCompanion)
: FtpCmdCwd("CDUP", 
                       "Change the current directory", 
                       "CDUP <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdCdup::ExecuteCommand()
{
    return Cwd("..");
}


/////////////////////////////// FtpCmdQuit ///////////////////////////////////////

FtpCmdQuit::FtpCmdQuit(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("QUIT", 
                       "Logs the user off from the FTP server", 
                       "QUIT <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdQuit::ExecuteCommand()
{
    WarLog logout_log(WARLOG_LOGOUT, "WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdQuit::ExecuteCommand()");

    logout_log << "FTP user "
        << GetServer()
        << " Logged out"
        << war_endl;

    *mpReply << SM_GOODBYE_QUIT << FTPR_QUIT_221;
        GetServer().CloseConnection(WarError(), false);
    return CMD_SUCCESS;
}


/////////////////////////////// FtpCmdStor ///////////////////////////////////////

FtpCmdStor::FtpCmdStor(WarSvrProtocolFtp& serverCompanion)
: FtpCmdTransfer("STOR", 
                       "Stores a file on the server", 
                       "STOR <SP> <pathname> <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdStor::ExecuteCommand()
{
    try
    {
        ReceiveFile(GetArgument());
    }
    catch(WarException& ex)
    {
        GetServer().OnTransferDone(ex);
        return CMD_FAIL;
    }

    return CMD_SUCCESS;
}

/////////////////////////////// FtpCmdStou ///////////////////////////////////////

FtpCmdStou::FtpCmdStou(WarSvrProtocolFtp& serverCompanion)
: FtpCmdTransfer("STOU", 
                       "Stores a file on the server with a unique name", 
                       "STOU [<SP>pathname]<CRLF>",
                       "RFC 959, RFC 1123; extended to common convensions",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdStou::ExecuteCommand()
{
    if (GetServer().mTransferMode.mRestartPos > 0)
    {
        *mpReply << "Ignoring previous REST command. STOU require a new file, and cannot resume a broken transfer.";
        GetServer().mTransferMode.mRestartPos = 0;
    }

    try
    {
        string my_name = GetArgument() ? GetArgument() : "";
        
        if (my_name.empty())
        {
            my_name = GetServer().GetOption("ftpd_STOUNAME");
            if (my_name.empty())
                my_name = "stou";
        }

        ReceiveFile(my_name.c_str(), true);
    }
    catch(WarException& ex)
    {
        GetServer().OnTransferDone(ex);
        return CMD_FAIL;
    }

    return CMD_SUCCESS;
}

/////////////////////////////// FtpCmdRnfr ///////////////////////////////////////

FtpCmdRest::FtpCmdRest(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("REST", 
                       "Restart a broken transfer", 
                       "REST <SP> <marker> <CRLF>",
                       "RFC 959, draft-ietf-ftpext-mlst-11",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdRest::ExecuteCommand()
{
    
    GetServer().mTransferMode.mRestartPos = WarAtoi<war_flen_t>(GetArgument());

    *mpReply << SM_REST_OK << FTPR_PENDING_OK_350;

    return CMD_SUCCESS;
}

/////////////////////////////// FtpCmdSyst ///////////////////////////////////////

FtpCmdSyst::FtpCmdSyst(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("SYST", 
                       "System identification", 
                       "SYST <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdSyst::ExecuteCommand()
{
    FtpSvrMsgsE my_msg = SM_SYST_OK_IDLE;

    if (GetServer().GetBoolOption("ftpd_STELTHSYST"))
        my_msg = SM_SYST_OK_IDLE_STELTH;

    *mpReply << my_msg << FTPR_NAME_OK_215;
    return CMD_SUCCESS;
}


/////////////////////////////// FtpCmdHelp ///////////////////////////////////////

FtpCmdHelp::FtpCmdHelp(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("HELP", 
                       "Gives help on the commands avialable", 
                       "HELP [<SP> <string>] <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdHelp::ExecuteCommand()
{
    war_ccstr_t help_on_cmd = GetArgument();
    
    *mpReply << SM_HELP_ALL << FTPR_HELP_214;

    return CMD_SUCCESS;
}

/////////////////////////////// FtpCmdAppe ///////////////////////////////////////

FtpCmdAppe::FtpCmdAppe(WarSvrProtocolFtp& serverCompanion)
: FtpCmdTransfer("APPE", 
                 "Appends a file to an existing file", 
                 "APPE <SP> <pathname> <CRLF>",
                 "RFC 959",
                 serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdAppe::ExecuteCommand()
{
    if (GetServer().mTransferMode.mRestartPos > 0)
    {
        *mpReply << "Ignoring previous REST command. APPE cannot resume a broken transfer.";
        GetServer().mTransferMode.mRestartPos = 0;
    }

    try
    {
        ReceiveFile(GetArgument(), false, true);
    }
    catch(WarException& ex)
    {
        GetServer().OnTransferDone(ex);
        return CMD_FAIL;
    }

    return CMD_SUCCESS;
}

/////////////////////////////// FtpCmdAllo ///////////////////////////////////////

FtpCmdAllo::FtpCmdAllo(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("ALLO", 
        "Allocate storage", 
        "ALLO <SP> <decimal-integer> [<SP> R <SP> <decimal-integer>] <CRLF>",
        "RFC 959",
        serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdAllo::ExecuteCommand()
{
    war_ccstr_t p = GetArgument(), digit_str = p;
    LastCommandresultE return_val = CMD_FAIL;

    while(isdigit(*p))
        ++p;
    
    if (*p == 0)
    {
        war_flen_t length = WarAtoi<war_flen_t>(digit_str),
            available = 0;

        try
        {
            WarFileDriver::driver_ptr_t driver_ptr = 
                WarFileEngine::GetEngine().GetDriver(
                GetServer().GetCwd().GetUrl().GetDriverName().c_str());

            available = driver_ptr->WarGetFreeSpaceOnPath(GetServer().GetCwd().GetUrl());

            if (available >= length)
            {
                *mpReply << SM_ALLO_OK << FTPR_OK_200;
                return_val = CMD_SUCCESS;
            }
            else
                *mpReply << SM_ALLO_FAIL << FTPR_SYNTAX_ERR_500;

        }
        catch(WarException& e)
        {
            *mpReply 
                << SM_GENERIC_FAIL 
                << e
                << FTPR_NOT_IMPLEMENTED_202;
        }
    }
    else
        *mpReply << SM_ALLO_NOT_IMPLEMENTED_PARAM << FTPR_NOT_IMPLEMENTED_202;
    
    return return_val;
}

/////////////////////////////// FtpCmdRnfr ///////////////////////////////////////

FtpCmdRnfr::FtpCmdRnfr(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("RNFR", 
                       "Rename file from", 
                       "RNFR <SP> <pathname> <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdRnfr::ExecuteCommand()
{
    try
    {
        GetServer().ResolvPath(GetArgument(), mPath);
        
        war_stat_t st;
        WarFsysStat(mPath.GetUrl(), st);

        if (mPath.IsSharedUpload()
            && !GetServer().IsFileOwnedBySession(mPath))
            WarThrow(WarError(WAR_ERR_ACCESS_DENIED), NULL);

        mPath.VerifyHost(GetServer().GetRemoteAddress().GetInAddr());
        mPath.VerifyRnfr();
    }
    catch(WarException& e)
    {
        switch(e.LocalError())
        {
        case WAR_ERR_ACCESS_DENIED:
        case WAR_ERR_OBJECT_NOT_FOUND:
        case WAR_FERR_NO_SUCH_PATH:
            *mpReply << SM_NO_FILE_OR_N0_ACCESS << FTPR_ACCESS_DENIED_550;
            break;
        default:
            *mpReply << SM_GENERIC_FAIL << e << FTPR_ACCESS_DENIED_550;
        }

        return CMD_FAIL;
    }

    // If we got here, the file is ok
    *mpReply << SM_RNFR_OK << FTPR_PENDING_OK_350;

    return CMD_SUCCESS;
}

/////////////////////////////// FtpCmdRnto ///////////////////////////////////////

FtpCmdRnto::FtpCmdRnto(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("RNTO", 
                       "Rename file to a new name. Works with RNFR", 
                       "RNTO <SP> <pathname> <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdRnto::ExecuteCommand()
{
    WarSvrPath from_path, to_path;
    WarFileDriver::driver_ptr_t my_driver_ptr;
    bool src_is_dir = false;
    bool dest_is_dir = false;
    bool src_is_shared_upload = false;
    bool dst_is_shared_upload = false;
    
    // Check that the source file information is ok.
    try
    {
        const WarSvrProtocolFtpCmd& prev_cmd = 
            GetServer().GetFtpCmds().GetPreviousCommand();

        if ((prev_cmd.GetLastResult() != CMD_SUCCESS)
            || stricmp(prev_cmd.GetName().c_str(), "RNFR"))
            WarThrow(WarError(WAR_ERR_COMMAND_FAILED), NULL);

        from_path = ((FtpCmdRnfr &)prev_cmd).GetRnfrPath();
    }
    catch(WarException)
    {
        *mpReply << SM_BAD_SEQUENCE_OF_COMMANDS << FTPR_BAD_SEQ_503;
        return CMD_FAIL;
    }

    // Verify perms
    if ((src_is_shared_upload = from_path.IsSharedUpload())
        && !GetServer().IsFileOwnedBySession(from_path))
    {
        *mpReply << SM_NO_FILE_OR_N0_ACCESS << FTPR_ACCESS_DENIED_550;
        return CMD_FAIL;
    }

    // Check that the source exist, and that the destination does not
    // exist, or is a directory. Also check that the source and
    // destination is using the same file-driver.
    try
    {
        GetServer().ResolvPath(GetArgument(), to_path);

        dst_is_shared_upload = to_path.IsSharedUpload();
        
        from_path.VerifyHost(GetServer().GetRemoteAddress().GetInAddr());
        to_path.VerifyHost(GetServer().GetRemoteAddress().GetInAddr());
        
        if (from_path.GetUrl().GetDriverName() != to_path.GetUrl().GetDriverName())
        {
            *mpReply << SM_RNTO_NOT_SAME_DEVICE << FTPR_ACCESS_DENIED_550;
            return CMD_FAIL;
        }
        
        my_driver_ptr = WarFileEngine::GetEngine().GetDriver(
            to_path.GetUrl().GetDriverName().c_str());
                
        // Get info about the source
        war_stat_t st;
        my_driver_ptr->WarStat(from_path.GetUrl(), st);
        src_is_dir = ((st.st_mode & S_IFDIR) == S_IFDIR);

        // Get info about the destination
        switch(my_driver_ptr->WarExamine(to_path.GetUrl(), st))
        {
        case FT_DIR:
            if (src_is_dir)
                goto dest_exist;
            dest_is_dir = true;
            break;
        case FT_VOID:
            break;
        default:
dest_exist:
            *mpReply << SM_DESTINATION_EXIST << FTPR_ACCESS_DENIED_550;
            return CMD_FAIL;
        }
    }
    catch(WarException& e)
    {
        switch(e.LocalError())
        {
        case WAR_ERR_ACCESS_DENIED:
        case WAR_FERR_NO_SUCH_PATH:
            *mpReply << SM_NO_FILE_OR_N0_ACCESS << FTPR_ACCESS_DENIED_550;
            break;
        default:
            *mpReply << SM_GENERIC_FAIL << e << FTPR_ACCESS_DENIED_550;
        }

        return CMD_FAIL;
    }

    // Do the job
    try
    {
        if (src_is_dir)
        {

            from_path.VerifyRnfrDir();
            to_path.VerifyRntoDir();

            my_driver_ptr->WarRenameDirectory(
                from_path.GetUrl(), to_path.GetUrl());
        }
        else
        {
            from_path.VerifyRnfr();
            to_path.VerifyRnto();

            my_driver_ptr->WarGenericMoveFile(
                from_path.GetUrl(), to_path.GetUrl());
        }
    }
    catch(WarException& e)
    {
        switch(e.LocalError())
        {
        case WAR_ERR_ACCESS_DENIED:
        case WAR_ERR_OBJECT_NOT_FOUND:
        case WAR_FERR_NO_SUCH_PATH:
            *mpReply << SM_NO_FILE_OR_N0_ACCESS << FTPR_ACCESS_DENIED_550;
            break;
        default:
            *mpReply << SM_GENERIC_FAIL << e << FTPR_ACCESS_DENIED_550;
        }

        return CMD_FAIL;
    }

    if (src_is_shared_upload)
        GetServer().DeleteFromSessionFiles(from_path);

    if (dst_is_shared_upload)
        GetServer().AddToSessionFiles(to_path);
    
    if (src_is_shared_upload && !dst_is_shared_upload)
    {
        WarLog warn_log(WARLOG_WARNINGS, "FtpCmdRnto::ExecuteCommand()()");
        
        warn_log << "Moved a file \""
            << from_path 
            << "\" from a shared upload directory to a location \""
            << to_path
            << "\" that is not a shared upload directory. "
            << war_endl;
    }

    *mpReply << SM_RNTO_OK << FTPR_FILE_ACTION_OK_250;

    return CMD_SUCCESS;
}

/////////////////////////////// FtpCmdAbor ///////////////////////////////////////

FtpCmdAbor::FtpCmdAbor(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("ABOR", 
                       "Aborts the current transfer", 
                       "ABOR <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdAbor::ExecuteCommand()
{
    switch(GetServer().GetConnectionState())
    {
        case GETFILE:
        case PUTFILE:
            // Abort the transfer.
            try
            {
                GetServer().mDataConnectionPtr->Abort();
            }
            catch(WarException& e)
            {
                WarLog err_log(WARLOG_ERROR, "FtpCmdAbor::ExecuteCommand()");

                err_log << "Failed to ABORt an ongoing transfer properly."
                    << e << war_endl;
                
                *mpReply << SM_GENERIC_FAIL 
                    << e
                    << FTPR_TRANSFER_ABORTED_DATACONN_CLOSED_426;
                
                GetServer().SetConnectionState(IDLE);
            }
        default:
            ; // Nothing
    }

    *mpReply << SM_ABOR_OK << FTPR_TRANSFER_OK_226;
    
    return CMD_SUCCESS;
}

/////////////////////////////// FtpCmdDele ///////////////////////////////////////

FtpCmdDele::FtpCmdDele(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("DELE", 
                       "Delete a file", 
                       "DELE <SP> <pathname> <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdDele::ExecuteCommand()
{
    WarSvrPath my_path;

    try
    {
        GetServer().ResolvPath(GetArgument(), my_path);
        
        WarFileDriver::driver_ptr_t my_driver_ptr = 
            WarFileEngine::GetEngine().GetDriver(
            my_path.GetUrl().GetDriverName().c_str());

        war_stat_t st;
        switch(my_driver_ptr->WarExamine(my_path.GetUrl(), st))
        {
            
        case FT_LINK:
        case FT_SPECIAL:
        case FT_NORMAL:
            if (my_path.IsSharedUpload() 
                && !GetServer().IsFileOwnedBySession(my_path))
                goto nope;
            
            my_path.VerifyDelete();

            my_driver_ptr->WarDeleteFile(my_path.GetUrl());
            break;

        case FT_DIR:
            *mpReply << SM_DELE_FAILED_IS_DIR << FTPR_ACCESS_DENIED_550;
            return CMD_FAIL;
        default:
nope:
            // Nope!
            *mpReply << SM_NO_FILE_OR_N0_ACCESS << FTPR_ACCESS_DENIED_550;
            return CMD_FAIL;
        }
    }
    catch(WarException& e)
    {
        switch(e.LocalError())
        {
        case WAR_ERR_ACCESS_DENIED:
        case WAR_FERR_NO_SUCH_PATH:
            *mpReply << SM_NO_FILE_OR_N0_ACCESS << FTPR_ACCESS_DENIED_550;
            break;
        default:
            *mpReply << e << SM_DELE_FAILED << FTPR_ACCESS_DENIED_550;
            break;
        }
        return CMD_FAIL;
    }

    GetServer().DeleteFromSessionFiles(my_path);

    *mpReply << SM_DELETE_OK << FTPR_FILE_ACTION_OK_250;
    return CMD_SUCCESS;
}

/////////////////////////////// FtpCmdRmd ///////////////////////////////////////

FtpCmdRmd::FtpCmdRmd(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("RMD", 
                       "Deletes an empty directory", 
                       "RMD  <SP> <pathname> <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdRmd::ExecuteCommand()
{
    try
    {
        WarSvrPath my_path;
        GetServer().ResolvPath(GetArgument(), my_path);

        my_path.VerifyHost(GetServer().GetRemoteAddress().GetInAddr());
        my_path.VerifyRmd();
        
        WarFileDriver::driver_ptr_t my_driver_ptr = 
            WarFileEngine::GetEngine().GetDriver(
            my_path.GetUrl().GetDriverName().c_str());

        war_stat_t st;
        switch(my_driver_ptr->WarExamine(my_path.GetUrl(), st))
        {
        case FT_SPECIAL:
        case FT_NORMAL:
            *mpReply << SM_RMD_FAILED_IS_FILE << FTPR_ACCESS_DENIED_550;
            return CMD_FAIL;
            break;

        case FT_LINK:
        case FT_DIR:
            if (my_path.IsSharedUpload() 
                && !GetServer().IsFileOwnedBySession(my_path))
                goto nope;
            my_driver_ptr->WarDeleteDirectory(my_path.GetUrl());
            break;
        default:
nope:
            // Nope!
            *mpReply << SM_NO_FILE_OR_N0_ACCESS << FTPR_ACCESS_DENIED_550;
            return CMD_FAIL;
        }
    }
    catch(WarException& e)
    {
        switch(e.LocalError())
        {
        case WAR_ERR_ACCESS_DENIED:
        case WAR_FERR_NO_SUCH_PATH:
            *mpReply << SM_NO_FILE_OR_N0_ACCESS << FTPR_ACCESS_DENIED_550;
            break;
        default:
            *mpReply << e << SM_RMD_FAILED << FTPR_ACCESS_DENIED_550;
            break;
        }
        return CMD_FAIL;
    }

    *mpReply << SM_RMD_OK << FTPR_FILE_ACTION_OK_250;
    return CMD_SUCCESS;
}

/////////////////////////////// FtpCmdMkd ///////////////////////////////////////

FtpCmdMkd::FtpCmdMkd(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("MKD", 
                       "Creates a directory", 
                       "MKD <SP> <pathname> <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdMkd::ExecuteCommand()
{
    try
    {
        GetServer().ResolvPath(GetArgument());

        GetServer().mCurrentFile.VerifyHost(GetServer().GetRemoteAddress().GetInAddr());
        GetServer().mCurrentFile.VerifyMkd();
        
        WarFileDriver::driver_ptr_t my_driver_ptr = 
            WarFileEngine::GetEngine().GetDriver(
            GetServer().mCurrentFile.GetUrl().GetDriverName().c_str());

        war_stat_t st;
        switch(my_driver_ptr->WarExamine(
            GetServer().mCurrentFile.GetUrl(), st))
        {
        case FT_VOID:
            WarFsysCreateDirectory(GetServer().mCurrentFile.GetUrl());
            break;
        case FT_SPECIAL:
        case FT_NORMAL:
        case FT_LINK:
        case FT_DIR:
            *mpReply << SM_MKD_FAILED_PATH_EXIST << FTPR_ACCESS_DENIED_550;
            return CMD_FAIL;
            break;
        default:
            // Nope!
            *mpReply << SM_NO_FILE_OR_N0_ACCESS << FTPR_ACCESS_DENIED_550;
            return CMD_FAIL;
        }
    }
    catch(WarException& e)
    {
        switch(e.LocalError())
        {
        case WAR_ERR_ACCESS_DENIED:
        case WAR_FERR_NO_SUCH_PATH:
            *mpReply << SM_NO_FILE_OR_N0_ACCESS << FTPR_ACCESS_DENIED_550;
            break;
        default:
            *mpReply << e << SM_MKD_FAILED << FTPR_ACCESS_DENIED_550;
            break;
        }
        return CMD_FAIL;
    }

    *mpReply << SM_MKD_OK << FTPR_PATHNAME_OK_257;
    return CMD_SUCCESS;
}

/////////////////////////////// FtpCmdStat ///////////////////////////////////////

FtpCmdStat::FtpCmdStat(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("STAT", 
                       "Returns the current status", 
                       "STAT [<SP> <pathname>] <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdStat::ExecuteCommand()
{
    if (HaveArgument())
    {
        try
        {
            GetServer().ResolvPath(GetArgument());
            war_stat_t st;
            switch(WarFsysExamine(GetServer().mCurrentFile.GetUrl(), st))
            {
            case FT_NORMAL:
                *mpReply << SM_STATUS_WITH_ARGUMENT_IS_FILE << FTPR_STAT_FILE_213;
                break;
            case FT_DIR:
                *mpReply << SM_STATUS_WITH_ARGUMENT_IS_DIR << FTPR_STAT_DIR_212;
                break;
            case FT_VOID:
                *mpReply << SM_STATUS_WITH_ARGUMENT_FAILED_VOID;
                goto failed_denied;
            default:
                *mpReply << SM_STATUS_WITH_ARGUMENT_FAILED_TYPE;
                goto failed_denied;
            }
        }
        catch(WarException& e)
        {
            switch(e.LocalError())
            {
            case WAR_ERR_ACCESS_DENIED:
            case WAR_FERR_NO_SUCH_PATH:
                *mpReply << SM_NO_FILE_OR_N0_ACCESS;
                break;
            default:
                *mpReply << e << SM_MKD_FAILED;
                break;
            }
            goto failed_denied;
        }
    }
    else
    {
        switch(GetServer().GetConnectionState())
        {
        case IDLE:
            if (GetServer().GetBoolOption("ftpd_STELTHSYST"))
                *mpReply << SM_STATUS_IDLE_STELTH << FTPR_STAT_211;
            else
                *mpReply << SM_STATUS_IDLE << FTPR_STAT_211;
            break;
        case GETFILE:
        case PUTFILE:
            *mpReply << SM_STATUS_TRANSFER << FTPR_STAT_211;
            break;
        default:
            *mpReply << SM_GENERIC_FAIL << FTPR_SYNTAX_ERR_500;
        }
    }

    return CMD_SUCCESS;
    
failed_denied:
    if (GetServer().GetBoolOption("ftpd_STRICTREPLY"))
        *mpReply << FTPR_SYNTAX_ERR_500;
    else
        *mpReply << FTPR_ACCESS_DENIED_550;

    return CMD_FAIL;
}

/////////////////////////////// FtpCmdNoop ///////////////////////////////////////

FtpCmdNoop::FtpCmdNoop(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("NOOP", 
                       "No operation, used to keep a session open", 
                       "NOOP <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdNoop::ExecuteCommand()
{
    if (GetServer().GetBoolOption("ftpd_NOOP"))
    {
        *mpReply << SM_NOOP_OK << FTPR_OK_200;
        WarSvrProtocolFtpCmd::OnResetTimer();
    }
    else
        *mpReply << SM_NOOP_OK_NORESET << FTPR_OK_200;

    return CMD_SUCCESS;
}

/////////////////////////////// FtpCmdSite ///////////////////////////////////////

FtpCmdSite::FtpCmdSite(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("SITE", 
                       "Nonstandard commands", 
                       "SITE <SP> <string> <CRLF>",
                       "RFC 959",
                       serverCompanion)
{
    mCommands.AssignToServer(&serverCompanion);
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdSite::ExecuteCommand()
{
    war_ccstr_t p = GetArgument();
    string cmd_name;
    LastCommandresultE result = CMD_FAIL;

    while(*p)
    {
        if (*p == ' ')
            break;
        cmd_name += *p++;
    }

    mCommands.OnCommand(cmd_name.c_str(), p, strlen(p));

    try
    {
        result = mCommands.GetCurrentCommand().GetLastResult();
    }
    catch(WarException)
    {
    }

    return result;
}

#if WAR_RFC2389
/////////////////////////////// FtpCmdFeat ///////////////////////////////////////

FtpCmdFeat::FtpCmdFeat(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("FEAT", 
                       "Lists extended commands and features", 
                       "FEAT <CRLF>",
                       "RFC 2389",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdFeat::ExecuteCommand()
{
    *mpReply << SM_FEAT_OK << FTPR_STAT_211;
    return CMD_SUCCESS;
}


/////////////////////////////// XX ///////////////////////////////////////

FtpCmdOpts::FtpCmdOpts(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("OPTS", 
                       "Negotiate an option with the server", 
                       "OPTS command-name [SP command-option] <CRLF>",
                       "RFC 2389",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdOpts::ExecuteCommand()
{
    war_ccstr_t p = GetArgument();
    std::string my_command_name;
    LastCommandresultE result = CMD_FAIL;
    FtpSvrMsgsE reply_msg = SM_OPTS_FAILED;
    WarFTPDReplyCodesE reply_code = FTPR_ARG_SYNTAX_501;

    while(*p)
    {
        if (*p == ' ')
        {
            ++p;
            break;
        }

        my_command_name += *p++;
    }

    try
    {
        WarSvrProtocolFtpCmd *my_cmd = GetServer().GetCmdPtr(
            my_command_name.c_str());
        
        if (my_cmd)
        {
            my_cmd->OnOpts(my_command_name.c_str(), 
                p, result, reply_msg, reply_code);
        }
    }   
    catch(WarException)
    {
    }

    *mpReply << reply_msg << reply_code;
    return result;
}

#endif // WAR_RFC2389

/////////////////////////////// XX ///////////////////////////////////////

FtpCmdMdtm::FtpCmdMdtm(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("MDTM", 
                       "Shows modification time for a file", 
                       "MDTM <SP> path <CRLF>",
                       "draft-ietf-ftpext-mlst-11",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdMdtm::ExecuteCommand()
{
    try
    {
        // MDTM is only supposed to work on files we can
        // send. We will therefore open file file to make
        // sure that we have the required access-level

        GetServer().ResolvPath(GetArgument());
        GetServer().mCurrentFile.VerifyHost(GetServer().GetRemoteAddress().GetInAddr());
        GetServer().mCurrentFile.VerifyList();

        war_uint32_t my_flags = F_READ | F_SHREAD | F_SHWRITE | F_MUSTEXIST;
        war_svrfile_ptr_t file_ptr = GetServer().OpenFile(
            GetServer().mCurrentFile, my_flags);

        // Ok
        size_t precision = 1; // Precision in sec/n where n == 1 is one second
        WarTime mod_time;
        WarFsysGetFileModTime(GetServer().mCurrentFile.GetUrl(), 
            mod_time, precision);

        *mpReply << mod_time.FormatMDTM(precision) << FTPR_STAT_FILE_213;
        return CMD_SUCCESS;
    }
    catch(WarException& e)
    {
        switch(e.LocalError())
        {
        case WAR_ERR_ACCESS_DENIED:
        case WAR_FERR_NO_SUCH_PATH:
            *mpReply << SM_NO_FILE_OR_N0_ACCESS << FTPR_ACCESS_DENIED_550;
            break;
        case WAR_ERR_INVALID_PATH:
            *mpReply << SM_ILLEGAL_PATH << FTPR_ARG_SYNTAX_501;
            break;
        case WAR_FERR_PATH_IS_A_DIR:
            *mpReply << SM_THE_PATH_IS_A_DIR << FTPR_ACCESS_DENIED_550;
            break;
        default:
            *mpReply << e << SM_GENERIC_FAIL << FTPR_SYNTAX_ERR_500;
            break;
        }
    }

    return CMD_FAIL;
}

/////////////////////////////// FtpCmdSize ///////////////////////////////////////

FtpCmdSize::FtpCmdSize(WarSvrProtocolFtp& serverCompanion)
: WarSvrProtocolFtpCmd("SIZE", 
                       "The size of the file in the current text/binary mode", 
                       "SIZE <SP> path <CRLF>",
                       "draft-ietf-ftpext-mlst-11",
                       serverCompanion)
{
}

WarSvrProtocolFtpCmd::LastCommandresultE FtpCmdSize::ExecuteCommand()
{
    war_flen_t file_length = 0;

    try
    {
        war_stat_t st;

        GetServer().ResolvPath(GetArgument());
        GetServer().mCurrentFile.VerifyHost(GetServer().GetRemoteAddress().GetInAddr());
        GetServer().mCurrentFile.VerifyList();

        // Check binary size
        switch(WarFsysExamine(GetServer().mCurrentFile.GetUrl(), st))
        {
        case FT_NORMAL:
            file_length = st.st_size;
            break;
        case FT_DIR:
            WarThrow(WarError(WAR_FERR_PATH_IS_A_DIR), NULL);
            break;
        default:
            WarThrow(WarError(WAR_FERR_NOT_A_FILE), NULL);
        }
        

        // If this is an ascii file, we must calculate its 
        // size. We only do that up to a certain size to
        // avoid DoS attacks, using ths command on large
        // binary files.
        if (TYPE_ASCII == GetServer().mTransferMode.mType)
        {
            war_flen_t max_len = GetServer().GetIntOption("ftpd_SIZEMAXASCII");
            if ((max_len > 0) && (file_length > max_len))
            {
                *mpReply << SM_THE_FILE_IS_TOO_LARGE_FOR_ASCII << FTPR_ACCESS_DENIED_550;
                return CMD_FAIL;
            }

            war_uint32_t my_flags = F_READ | F_SHREAD | F_SHWRITE | F_MUSTEXIST | F_TEXT;
            war_svrfile_ptr_t file_ptr = GetServer().OpenFile(
                GetServer().mCurrentFile, my_flags);
            
            file_length = file_ptr->GetLength();
        }
    }
    catch(WarException& e)
    {
        switch(e.LocalError())
        {
        case WAR_ERR_ACCESS_DENIED:
        case WAR_FERR_NO_SUCH_PATH:
            *mpReply << SM_NO_FILE_OR_N0_ACCESS << FTPR_ACCESS_DENIED_550;
            break;
        case WAR_ERR_INVALID_PATH:
            *mpReply << SM_ILLEGAL_PATH << FTPR_ARG_SYNTAX_501;
            break;
        case WAR_FERR_PATH_IS_A_DIR:
            *mpReply << SM_THE_PATH_IS_A_DIR << FTPR_ACCESS_DENIED_550;
            break;
        case WAR_FERR_NOT_A_FILE:
            *mpReply << SM_NOT_A_REGULAR_FILE << FTPR_ACCESS_DENIED_550;
            break;
        default:
            *mpReply << e << SM_GENERIC_FAIL << FTPR_SYNTAX_ERR_500;
            break;
        }

        return CMD_FAIL;
    }

    *mpReply << file_length << FTPR_STAT_FILE_213;
    return CMD_SUCCESS;
}

