#include "StdAfx.h"
#include "WarSvrProtocolFtpDataConn.h"   // class implemented

#ifndef WAR_AUTO_LOCK_H
#   include "WarAutoLock.h"
#endif

#ifndef WAR_SVR_PROTOCOL_FTP_H
#   include "WarSvrProtocolFtp.h"
#endif

#ifndef WAR_FILE_DRIVER_FILE_H
#   include "WarFileDriverFile.h"
#endif

#ifndef WAR_LOG_H
#   include "WarLog.h"
#endif

#ifndef WAR_PERFMON_DEF_H
#   include "WarPerfmonDef.h"
#endif

#ifndef WAR_FUNCTION_PROFILER_H
#   include "WarFunctionProfiler.h"
#endif

#ifndef WAR_SOCKET_CPS_LIMIT_H
#   include "WarSocketCpsLimit.h"
#endif

#define AUTO_LOCK \
        war_datasck_ptr_t my_ref_ptr((WarSvrProtocolFtpDataConn *)this); \
        WarAutoLock my_lock((WarCriticalSection &)mrServer.mSvrLock)

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

WarSvrProtocolFtpDataConn::WarSvrProtocolFtpDataConn(
    war_socket_io_ptr_t& companionPtr,
    WarSvrProtocolFtp& rServer)
: WarSocket(companionPtr),
mrServer(rServer),
mDirection(D_INVALID),
mHaveSeenEof(false),
mIoSerialLast(0),
mIoSerialFirst(0),
mHasFinished(false),
mUseFileCallback(false),
mIsInitialized(false),
mTimeOutSeconds(120),
mNumPendingNetIoRequests(0),
mNumPendingFileIoRequests(0)
{
    WarLog debug_log(WARLOG_DEBUG, 
        "WarSvrProtocolFtpDataConn::WarSvrProtocolFtpDataConn()");

    WAR_DB_PERFMON_INC(WAR_PRFDEBUG_NUM_FTPDATA);
    
    if (debug_log)
    {
        debug_log << "Creating FTP data socket "
            << GetSeqNumber()
            << war_endl;
    }
}// WarSvrProtocolFtpDataConn


WarSvrProtocolFtpDataConn::~WarSvrProtocolFtpDataConn()
{
    WarLog debug_log(WARLOG_DEBUG, 
        "WarSvrProtocolFtpDataConn::~WarSvrProtocolFtpDataConn()");

    if (debug_log)
    {
        debug_log << "Destroying FTP data socket "
            << GetSeqNumber()
            << war_endl;
    }

    WAR_DB_PERFMON_DEC(WAR_PRFDEBUG_NUM_FTPDATA);

}// ~WarSvrProtocolFtpDataConn


//============================= OPERATORS ====================================



//============================= OPERATIONS ===================================


void WarSvrProtocolFtpDataConn::SendFile(war_svrfile_ptr_t& filePtr)
{
    mDirection = D_SENDING;

    int cps = mrServer.GetIntOption("ftpd_CPSOUTMAXCPS");
    if (cps > 0)
    {
        try
        {
            WarSocketCpsLimit::GetModule().EnableCps(
                this, 
                mrServer.GetIntOption("ftpd_CPSOUTSAMPLEDELAY"),
                mrServer.GetIntOption("ftpd_CPSOUTHOLDDELAY"),
                mrServer.GetIntOption("ftpd_CPSOUTMINCNT"),
                mrServer.GetIntOption("ftpd_CPSOUTMAXCNT"),
                mrServer.GetIntOption("ftpd_CPSOUTMINCPS"),
                cps,
                WarSocketCpsLimit::SENDING);
        }
        catch(WarException& ex)
        {
            WarLog err_log(WARLOG_ERROR, "WarSvrProtocolFtpDataConn::SendFile()");

            err_log << "Failed to set CPS limit. "
                << ex
                << war_endl;
        }
    }

    StartTransfer(filePtr);
}

void WarSvrProtocolFtpDataConn::ReceiveFile(war_svrfile_ptr_t& filePtr)
{
    mDirection = D_RECEIVING;

    int cps = mrServer.GetIntOption("ftpd_CPSINMAXCPS");
    if (cps > 0)
    {
        try
        {
            WarSocketCpsLimit::GetModule().EnableCps(
                this, 
                mrServer.GetIntOption("ftpd_CPSINSAMPLEDELAY"),
                mrServer.GetIntOption("ftpd_CPSINHOLDDELAY"),
                mrServer.GetIntOption("ftpd_CPSINMINCNT"),
                mrServer.GetIntOption("ftpd_CPSINMAXCNT"),
                mrServer.GetIntOption("ftpd_CPSINMINCPS"),
                cps,
                WarSocketCpsLimit::RECEIVING);
        }
        catch(WarException& ex)
        {
            WarLog err_log(WARLOG_ERROR, "WarSvrProtocolFtpDataConn::ReceiveFile()");

            err_log << "Failed to set CPS limit. "
                << ex
                << war_endl;
        }
    }
    
    StartTransfer(filePtr);
}

void WarSvrProtocolFtpDataConn::Abort(const war_error_definitions errReason)
{
    OnTransferDone(WarError(errReason));
}

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================

// called by the control socket
bool WarSvrProtocolFtpDataConn::IsTimeOut() const
{
    return GetIdleTimeSince().IsTimeOut(mTimeOutSeconds);
}

int WarSvrProtocolFtpDataConn::GetNumBuffers()
{
    return mNumPendingNetIoRequests
        + mNumPendingFileIoRequests
        + mFreeQueue.size()
        + mPendingInQueue.size();
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

void WarSvrProtocolFtpDataConn::StartTransfer(war_svrfile_ptr_t& filePtr)
{
    WarLog debug_log(WARLOG_DEBUG, 
        "WarSvrProtocolFtpDataConn::StartTransfer()");

    if (debug_log)
    {
        debug_log << "Starting transfer on FTP data socket "
            << GetSeqNumber()
            << " "
            << filePtr->GetUrl()
            << war_endl;
    }

    WAR_PERFMON_INC(WAR_PRFSVR_TRANSFERS_NOW)

    // Allocate transfer buffers
    {
        mNumBufSegments = mrServer.GetIntOption("ftpd_DATABUFSEGMENTS"); 
        if (mNumBufSegments < 1)
            mNumBufSegments = 1;

        mNumBuffers = mrServer.GetIntOption("ftpd_DATABUFCNT");
        if (mNumBuffers < 1)
            mNumBuffers = 1;
        
        mBufferLen = mrServer.GetIntOption("ftpd_DATABUFSIZE");
        if (mBufferLen < 512)
            mBufferLen = 512;

        filePtr->GetFileDriverFile().OnFtpBufferSize(
            mNumBuffers,
            mBufferLen,
            mNumBufSegments);

        mSegmentLen = mBufferLen / mNumBufSegments;
        
        for(int i = 0; i < mNumBuffers; i++)
            mFreeQueue.push_back(
                new WarTransferBuffer(mBufferLen, this, i));
        
        mFilePtr = filePtr;
        if (mFilePtr->GetFlags() & WarFileEnums::F_CALLBACK)
        {
            mFilePtr->AssignCallback(CallbackFunc, this);
            mUseFileCallback = true;
        }

        mIsInitialized = true;
        
        if (mpCompanion->GetCurrentState() != WAR_SCKSTATE_CONNECTED)
            return;
    }

    mTimeOutSeconds = mrServer.GetIntOption("ftpd_DATATIMEOUT");

    PumpData();
}

void WarSvrProtocolFtpDataConn::KillBuffers()
{
    while(!mFreeQueue.empty())
    {
        mFreeQueue.pop_front();
    }

    while(!mPendingInQueue.empty())
    {
        mPendingInQueue.erase(mPendingInQueue.begin());
    }

    WarLog debug_log(WARLOG_DEBUG, "WarSvrProtocolFtpDataConn::KillBuffers()");

    if (debug_log)
    {
        debug_log << GetNumBuffers() << " buffers and " 
            << mNumPendingNetIoRequests
            << " + "
            << mNumPendingFileIoRequests
            << " IO requests left after kill."
            << war_endl;
    }
}

void WarSvrProtocolFtpDataConn::PumpData()
{
    WAR_PROFILE_FUNC(WAR_FP_FTPDATA_PUMP);

    if (!mIsInitialized)
        return;

    bool do_continue = true;
    
    while(!mHaveSeenEof)
    {
        if (D_SENDING == mDirection)
        {
            // Send data
            try
            {
                if (mUseFileCallback)
                {
                    WAR_PROFILE_FUNC(WAR_FP_FTPDATA_PUMP_FREAD_ASYNCH);
                    while(QueueFileRead())
                        ;
                    break;
                }
                else
                {
                    WAR_PROFILE_FUNC(WAR_FP_FTPDATA_PUMP_FREAD_SYNCH);
                    
                    AUTO_LOCK;

                    if (mHaveSeenEof || mFreeQueue.empty())
                        break;

                    WarTransferBuffer *pbuf = &(*mFreeQueue.front());

                    pbuf->mBytesUsed = mFilePtr->Read(
                        pbuf->mBuffer, 
                        pbuf->GetLength());
                    
                    if (0 == pbuf->mBytesUsed)
                    {
                        WAR_PROFILE_FUNC(WAR_FP_FTPDATA_PUMP_FREAD_CHKEOF);
                        if (mFilePtr->IsEof())
                            WarThrow(WarError(WAR_FERR_END_OF_FILE), NULL);
                        else
                        {
                            // No data fetched, - not callback-mode
                            // and no error. 
                            // We must handle this as a error-situation
                            WarLog err_log(WARLOG_ERROR, "WarSvrProtocolFtpDataConn::PumpData()");
                            
                            err_log << "Read() from file returen zero bytes in non-callback mode. No error was reported. This should never happen!. Aborting transfer."
                                << war_endl;
                            
                            WarThrow(WarError(WAR_ERR_SYSTEM_ERROR), NULL);
                        }
                    }

                    {
                        WAR_PROFILE_FUNC(WAR_FP_FTPDATA_PUMP_SCK_OUT);

                        QueueNetWrite(mFreeQueue.front());
                        mFreeQueue.pop_front();
                    }
                }
            }
            catch(WarException& ex)
            {
                if (ex.LocalError() == WAR_FERR_END_OF_FILE)
                {
                    AUTO_LOCK;
                    mHaveSeenEof = true;
                    break;
                }
                
                // Abort
                WarLog warn_log(WARLOG_WARNINGS, "WarSvrProtocolFtpDataConn::PumpData()");
                warn_log << "Caught unexpected exception "
                    << ex.Explain()
                    << " Aborting transfer"
                    << war_endl;
                OnTransferDone(ex);
                return;
            }
        }
        else // Direction
        {
            // Receiving 

            WAR_PROFILE_FUNC(WAR_FP_FTPDATA_PUMP_RECEIVING);

            // Receive data
            try
            {
                while(QueueNetRead())
                        ;
                    break;
            }
            catch(WarException& ex)
            {
                if (ex.LocalError() == WAR_FERR_END_OF_FILE)
                {
                    mHaveSeenEof = true;
                    break;
                }
                
                // Abort
                WarLog warn_log(WARLOG_WARNINGS, "WarSvrProtocolFtpDataConn::PumpData()");
                warn_log << "Caught unexpected exception from QueueNetRead() "
                    << ex.Explain()
                    << " Aborting transfer"
                    << war_endl;
                OnTransferDone(ex);
                return;
            }
        }
        
    } // while(!mFreeQueue.empty())

    AUTO_LOCK;
    ProcessPendingBuffers();
        
    if (mHaveSeenEof)
    {
#ifdef DEBUG
        WarLog db_log(WARLOG_DEBUG, "WarSvrProtocolFtpDataConn::PumpData()");

        if (db_log)
        {
            db_log << "EOF has been set. "
                << GetReferenceCount() << " references left and "
                << mNumPendingNetIoRequests
                << " + "
                << mNumPendingFileIoRequests
                << " pendig IO requests and "
                << (int)mPendingInQueue.size()
                << " buffers pending for full sequence."
                << war_endl;
        }
#endif
        if (D_SENDING == mDirection)
        {
            if ((0 >= mNumPendingFileIoRequests)
                && (0 >= mNumPendingNetIoRequests)
                && mPendingInQueue.empty())
            {
                OnTransferDone(WarError());
            }
        }
        else
        {
             if ((0 >= mNumPendingFileIoRequests)
                 && (0 >= mNumPendingNetIoRequests)
                 && mPendingInQueue.empty())
             {
                 OnTransferDone(WarError());
             }
        }
    }
}

void WarSvrProtocolFtpDataConn::OnTransferDone(const WarError& status)
{
    WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnTransferDone);

    war_datasck_ptr_t my_ptr = this;

    AUTO_LOCK;

    if (mHasFinished)
        return;

    // Kill transfer plugins
    GetSocketIo().RemovePlugins(WarSocketIo::PLUGIN_ON_RECEIVED);
    GetSocketIo().RemovePlugins(WarSocketIo::PLUGIN_ON_SENT);
    
    // Clean up databuffers
    KillBuffers();

    WAR_PERFMON_DEC(WAR_PRFSVR_TRANSFERS_NOW)

    mHasFinished = true;
    mrServer.OnTransferDone(status);
    if (mFilePtr->IsOpen())
        mFilePtr->Close();
    mFilePtr = NULL;

    if (status)
    {
        WAR_PERFMON_INC(WAR_PRFSVR_TRANSFERS_FAILED);
    }
    else
    {
        WAR_PERFMON_INC(WAR_PRFSVR_TRANSFERS_TOTAL);
    }
    
    // Cleanup
    assert(IsOpen() == false);
}

void WarSvrProtocolFtpDataConn::OnAccept(const WarError& status,
        war_socket_t newSocket,
        const WarNetAddress& remoteAddress,
        const WarNetAddress& localAddress)
{
    WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnAccept);

    AUTO_LOCK;

    if (status)
    {
        OnTransferDone(status);
        return;
    }

    // We take over the new socket, and call OnConnect()
    mpCompanion->Close();
    mpCompanion->AssignSocket(newSocket);
    SetCurrentState(WAR_SCKSTATE_CONNECTED);
    OnConnect(status);
}

void WarSvrProtocolFtpDataConn::OnConnect(const WarError& status)
{
    WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnConnect);

    WarLog debug_log(WARLOG_DEBUG, "WarSvrProtocolFtpDataConn::OnConnect()");

    AUTO_LOCK;

    if (status)
    {
        if (debug_log)
        {
            debug_log << "Data connection with peer failed."
                << status
                << war_endl;
        }
        OnTransferDone(status);
        return;
    }

    if (debug_log)
    {
        debug_log << "Data connection with peer is established."
            << war_endl;
    }

    PumpData();
}

void WarSvrProtocolFtpDataConn::CallbackFunc(
    war_cptr_t ptrToObject, 
    const WarError&, 
    war_transfer_buffer_ptr_t& bufferPtr)
{
    WarSvrProtocolFtpDataConn *pObject = 
        (WarSvrProtocolFtpDataConn *)ptrToObject;

    pObject->OnFileIo(WarError(), bufferPtr);
}

void WarSvrProtocolFtpDataConn::OnFileIo(const WarError& status, 
                                         war_transfer_buffer_ptr_t& bufferPtr)
{
    WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo);
    
    WarError my_status = status;
    WarLog debug_log(WARLOG_DEBUG, "WarSvrProtocolFtpDataConn::OnFileIo()");
     
    if (mHasFinished)
    {
        // Just ignore the event
        if (debug_log)
        {
            AUTO_LOCK;
            
            --mNumPendingFileIoRequests;

            debug_log << "Trashing I/O callback. The connection is already closed. "
                << GetReferenceCount() << " references left and "
                << mNumPendingNetIoRequests
                << " + "
                << mNumPendingFileIoRequests
                << " pendig IO requests."
                << war_endl;
        }
        return;
    }
    
    if (D_SENDING == mDirection)
    {
        // Sending to client
        // Pass the buffer to the socket
        if (my_status)
        {
            WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_ERROR);
            
            if (my_status.LocalError() == WAR_FERR_END_OF_FILE)
            {
                AUTO_LOCK;
                
                --mNumPendingFileIoRequests;

                mHaveSeenEof = true;
            }
            else
            {
                goto abort_transfer;
            }
        }
        else
        {
            AUTO_LOCK;
            if (bufferPtr->mSerialNum == mIoSerialFirst)
            {
                WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_SendWithCallback);
                ++mIoSerialFirst;
                
                try
                {
                    QueueNetWrite(bufferPtr);
                }
                catch(WarException& ex)
                {
                    WarLog warn_log(WARLOG_WARNINGS, "WarSvrProtocolFtpDataConn::OnFileIo()");
                    warn_log << "Caught exception during QueueNetWrite() "
                        << ex.Explain()
                        << " Aborting transfer"
                        << war_endl;
                    
                    my_status = ex;
                    goto abort_transfer;
                }
            }
            else
            {
                WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_QUEUE);
                mPendingInQueue.insert(bufferPtr);
            }

            --mNumPendingFileIoRequests;
        }
    }
    else
    {
        // Receiving from client
        WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_RECV);
        if (my_status)
            goto abort_transfer;
        
        AUTO_LOCK;
        
        if (!mHaveSeenEof)
        {
            // Add the buffer to the free-queue
            mFreeQueue.push_front(bufferPtr);
        }

        --mNumPendingFileIoRequests;
    }

    
    PumpData();
    return;
        
abort_transfer:
    {
        // Fatal error. Abort the transfer!
        try
        {
            --mNumPendingFileIoRequests;

            OnTransferDone(my_status);
        }
        catch(WarException& ex)
        {
            WarLog warn_log(WARLOG_WARNINGS, "WarSvrProtocolFtpDataConn::OnFileIo()");
            warn_log << "Caught exception during OnTransferDone() "
                << ex.Explain()
                << " Ignoring"
                << war_endl;
        }
        return;
    }   
}

void WarSvrProtocolFtpDataConn::OnReceived(const WarError& status, 
        war_transfer_buffer_ptr_t& buffer)
{
    WarLog debug_log(WARLOG_DEBUG, "WarSvrProtocolFtpDataConn::OnReceived()");

    if (D_RECEIVING != mDirection)
    {
        WarLog warn_log(WARLOG_WARNINGS, "WarSvrProtocolFtpDataConn::OnReceived()");
        warn_log << "Received input data in sendmode. This should not happen. Aborting transfer."
            << war_endl;

        OnTransferDone(WarError(WAR_ERR_NOT_OPEN_FOR_READ));
        return;
    }

    
    {
        WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnReceived);
        
        AUTO_LOCK;

        if (mHasFinished)
        {   
            
            --mNumPendingNetIoRequests;

            // Just ignore the event
            if (debug_log)
            {
                debug_log << "Trashing I/O callback. The connection is already closed. "
                    << GetReferenceCount() << " references left and "
                    << mNumPendingNetIoRequests
                    << " + "
                    << mNumPendingFileIoRequests
                    << " pendig IO requests."
                    << war_endl;
            }
            return;
        }
        
        if (status)
        {
            if (status.LocalError() == WAR_FERR_END_OF_FILE)
            {
                if (debug_log)
                {
                    debug_log << "EOF on input file. "
                        << GetReferenceCount() << " references left and "
                        << mNumPendingNetIoRequests
                        << " + "
                        << mNumPendingFileIoRequests
                        << " pendig IO requests. "
                        << "Buffer serial # "
                        << buffer->mSerialNum 
                        << ", first serial # "
                        << mIoSerialFirst
                        << ", last serial # "
                        << mIoSerialLast
                        << war_endl;
                }
                
                mHaveSeenEof = true;
            }
            else
            {
                if (debug_log)
                {
                    debug_log << "Error on input file Aborting the transfer. "
                        << GetReferenceCount() << " references left and "
                        << mNumPendingNetIoRequests
                        << " + "
                        << mNumPendingFileIoRequests
                        << " pendig IO requests."
                        << war_endl;
                }
                
                // Fatal error. Abort the transfer!
                OnTransferDone(status);
            }
        }
        else // status
        {   
            // No error
            if (buffer->mSerialNum == mIoSerialFirst)
            {
                WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnReceived_SIMPLE);
                ++mIoSerialFirst;
                
                WriteToFile(buffer);
            }
            else
            {
                // Buffer is out of sync. Queue it.
                
                WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnReceived_QUEUE);
                mPendingInQueue.insert(buffer);
            }
        } 
        
        --mNumPendingNetIoRequests;
    }

    PumpData();
}

void WarSvrProtocolFtpDataConn::WriteToFile(war_transfer_buffer_ptr_t& buffer) 
{
    try
    {
        if (mUseFileCallback)
        {
            QueueFileWrite(buffer);
        }
        else
        {
            mFilePtr->Write(buffer->mBuffer, buffer->mBytesUsed);
            mFreeQueue.push_front(buffer);
        }
    }
    catch(WarException& ex)
    {
        WarLog err_log(WARLOG_ERROR, "WarSvrProtocolFtpDataConn::WriteToFile()");
        
        err_log << "Failed to write to destination file "
            << ex.Explain()
            << " Aborting transfer."
            << war_endl;
        
        OnTransferDone(ex);
        throw ex;
    }
}

void WarSvrProtocolFtpDataConn::ProcessPendingBuffers()
{
    WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_ProcessPendingBuffers);
    
    while(!mPendingInQueue.empty()
        && ((*mPendingInQueue.begin())->mSerialNum 
        == mIoSerialFirst))
    {
        ++mIoSerialFirst;

        war_transfer_buffer_ptr_t buffer = *mPendingInQueue.begin();
        mPendingInQueue.erase(mPendingInQueue.begin());

        if (D_SENDING == mDirection)
        {
            // Sending
            QueueNetWrite(buffer);
        }
        else
        {
            // Receiving
            WriteToFile(buffer);
        }
    }
}
    
void WarSvrProtocolFtpDataConn::OnSent(const WarError& status,
        war_transfer_buffer_ptr_t& buffer)
{
    {
        WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnSent);
        
        assert(D_SENDING == mDirection);
        
        AUTO_LOCK;
        
        WarLog debug_log(WARLOG_DEBUG, "WarSvrProtocolFtpDataConn::OnSent()");
                
        if (mHasFinished)
        {
            --mNumPendingNetIoRequests;
            
            // Just ignore the event
            if (debug_log)
            {
                debug_log << "Trashing I/O callback. The connection is already closed. "
                    << GetReferenceCount() << " references left and "
                    << mNumPendingNetIoRequests
                    << " + "
                    << mNumPendingFileIoRequests
                    << " pendig IO requests."
                    << war_endl;
            }
            return;
        }
        
        if (status)
        {
            // Fatal error. Abort the transfer!
            
            --mNumPendingNetIoRequests;
            
            OnTransferDone(status);
            return;
        }
        
        if (!mHaveSeenEof)
        {
            mFreeQueue.push_front(buffer);
        }
        
        --mNumPendingNetIoRequests;
    }
    
    PumpData();
}

bool WarSvrProtocolFtpDataConn::QueueNetRead()
{
    war_transfer_buffer_ptr_t buffer;
 
    AUTO_LOCK;
    
    if (mFreeQueue.empty() || mHaveSeenEof)
        return false;
    
    buffer = mFreeQueue.front();
    buffer->mSerialNum = mIoSerialLast++;
    mFreeQueue.pop_front();
    
    RecvWithCallback(buffer, mNumBufSegments);
    
    ++mNumPendingNetIoRequests;
    
    return true;
}

void WarSvrProtocolFtpDataConn::QueueNetWrite(war_transfer_buffer_ptr_t& buffer)
{
    AUTO_LOCK;

    SendWithCallback(buffer, mNumBufSegments); 
    
    ++mNumPendingNetIoRequests;
}

bool WarSvrProtocolFtpDataConn::QueueFileRead()
{
    war_transfer_buffer_ptr_t buffer;
    
    AUTO_LOCK;
    if (mFreeQueue.empty() || mHaveSeenEof)
        return false;
    
    buffer = mFreeQueue.front();
    buffer->mSerialNum = mIoSerialLast++;
    mFreeQueue.pop_front();
    
    mFilePtr->ReadWithCallback(buffer);
    
    ++mNumPendingFileIoRequests;
    
    return true;
}

void WarSvrProtocolFtpDataConn::QueueFileWrite(war_transfer_buffer_ptr_t& buffer)
{
    mFilePtr->WriteWithCallback(buffer);
    ++mNumPendingFileIoRequests;
}

/////////////////////////////// PRIVATE    ///////////////////////////////////

