/** A fast locking class for
 *  code and data locks.
 *
 *
 * #include "WarCriticalSection.h" <BR>
 *
 *  
 */

#ifndef WAR_CRITICAL_SECTION_H
#define WAR_CRITICAL_SECTION_H

/* SYSTEM INCLUDES */

#ifndef WAR_ASSERT_H_INCLUDED
#   define WAR_ASSERT_H_INCLUDED
#   include <assert.h>
#endif

/* PROJECT INCLUDES */

#ifndef WAR_EXCEPTION_H
#   include "WarException.h"
#endif


/* LOCAL INCLUDES */


/* FORWARD REFERENCES */

class WarThreadEvent;

#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/


/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/

#ifdef __cplusplus

class WarCriticalSection 
{
public:
    // LIFECYCLE
    
    /**
    * Default constructor.
    *
    * @exception WarException on error
    */
    WarCriticalSection(bool allowRecursive = false) throw(WarException);
    
    /**
    * Destructor.
    */
    ~WarCriticalSection(void);
    
    // OPERATORS    

    /** */
    inline void Lock() throw(WarException)
    {
        ::EnterCriticalSection(&mHandle);
#ifdef _DEBUG
        ++mNumLocks;
        if (!mDoRecursive)
            assert(mNumLocks == 1);
#endif
    }

    /** */
    inline void Unlock() throw(WarException)
    {
#ifdef _DEBUG
        --mNumLocks;
        assert(mNumLocks >= 0);
        if (!mDoRecursive)
            assert(mNumLocks == 0);
#endif
        
        ::LeaveCriticalSection(&mHandle);
    }

    // OPERATIONS                       
    // ACCESS
    // INQUIRY
    
protected:
private:
    friend WarThreadEvent;
    war_critsec_t mHandle;
    bool mDoRecursive; // Recursive locs on a therad

#ifdef _DEBUG
    int mNumLocks;
#endif
};

/* INLINE METHODS */

/* EXTERNAL REFERENCES */

#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* _WAR_CRITICAL_SECTION_H_ */

