/** 
*/

#ifndef WAR_DIRFMT_UNIX_LS_H
#define WAR_DIRFMT_UNIX_LS_H

/* SYSTEM INCLUDES */

#ifndef WAR_INCLUDED_LIMITS_H
#   define WAR_INCLUDED_LIMITS_H
#   include <limits.h>
#endif

/* PROJECT INCLUDES */

#ifndef WAR_SVR_DIRLIST_H
#   include "WarSvrDirList.h"
#endif

#ifndef WAR_GETOPT_H
#   include "WarGetopt.h"
#endif

#ifndef WAR_ESCAPE_STR_H
#   include "WarEscapeStr.h"
#endif

/* LOCAL INCLUDES */


/* FORWARD REFERENCES */



#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/

#ifndef INODE_DIGITS
#   define INODE_DIGITS 7
#endif

#ifndef S_IRUSR
#   define S_IRWXU 00700 // mask for user (file owner) permissions
#   define S_IRUSR 00400 //user has read permission
#   define S_IWUSR 00200 //user has write permission
#   define S_IXUSR 00100 //user has execute permission
#   define S_IRWXG 00070 //mask for group permissions
#   define S_IRGRP 00040 //group has read permission
#   define S_IWGRP 00020 //group has write permission
#   define S_IXGRP 00010 //group has execute permission
#   define S_IRWXO 00007 //mask for permissions for others (not in group)
#   define S_IROTH 00004 //others have read permission
#   define S_IWOTH 00002 //others have write permisson
#   define S_IXOTH 00001 //others have execute permission
#   define S_IXUGO (S_IXUSR | S_IXGRP | S_IXOTH)
#endif // S_IRUSR

#ifndef S_ISSOCK
#   define S_ISSOCK(a) (0)
#endif

#ifndef S_ISDOOR
#   define S_ISDOOR(a) (0)
#endif

#define WAR_ISREG(a) (a.mType == WarDirListNode::FT_NORMAL)
#define WAR_ISDIR(a) (a.mType == WarDirListNode::FT_DIR)
#define WAR_ISLNK(a) (a.mType == WarDirListNode::FT_LINK)
#define WAR_ISFIFO(a) (a.mType == WarDirListNode::FT_SPECIAL)
#define WAR_ISSOCK(a) S_ISSOCK(a.mMode)
#define WAR_ISDOOR(a) S_ISDOOR(a.mMode)


/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/

#ifdef __cplusplus

template <class outputT>
class WarDirfmtUnixLs : public WarFileEnums
{
public:
    typedef std::list<std::string> seq_file_list_t;
    typedef std::list<std::string> ignore_patterns_t;

    ///
    enum format
    {
        /// -l (el)
        LONG_FORMAT,	
        ///  -1 (one)
        ONE_PER_LINE,	
        /// -C
        MANY_PER_LINE,	
        /// -x
        HORIZONTAL,	
        /// -m
        WITH_COMMAS	
    };

    ///
    enum sort_type
    {
        /// -U 
        SORT_NONE,
        /// default
        SORT_NAME,
        /// -X
        SORT_EXTENSION,
        /// -t
        SORT_TIME,
        /// -S
        SORT_SIZE,
        /// -v
        SORT_VERSION
    };
    
    ///
    enum time_type
    {
        /// default
        TIME_MTIME,
        /// -c
        TIME_CTIME,
        /// -u
        TIME_ATIME
    };

    enum indicator_style
    {
        /// --indicator-style=none
        IS_NONE,
        /// -F, --indicator-style=classify
        IS_CLASSIFY,
        /// -p, --indicator-style=file-type
        IS_FILE_TYPE
    };
    
    // LIFECYCLE
    /** Constructor */
    WarDirfmtUnixLs()
        : mDoPrintBlockSize(false),
        mDoPrintInodes(false),
        mDoPrintBlockSizeAll(false),
        mDoNumericIds(false),
        mDoHideGroup(false),
        mDoFullTime(false),
        mDoAllFiles(false),
        mDoReallyAllFiles(false),
        mDoRecursive(false),
        mDoPrintDirName(false),
        mDoImmediateDirs(false),
        mDoQmarkFunnyChars(false),
        mDoReverseSort(false),
        mDoTraceLinks(false),
        mListFormat(LONG_FORMAT),
        mSortBy(SORT_NAME),
        mTimeType(TIME_MTIME),
        mIndicatorStyle(IS_NONE),
        mOutputBlockSize(0),
        mpDir(NULL),
        mDoQuoteNames(false)
    {
        strcpy(mEndOfLine, "\n");
    }

        
    ///
    virtual ~WarDirfmtUnixLs()
    {};

    // OPERATORS
    
     //* Interface to \Ref{WarDirListing}. */
    outputT& List (outputT& Out, const WarSvrDirList& Dir)
    {
        mpDir = &Dir;
        if ((LONG_FORMAT == mListFormat) || mDoPrintBlockSize)
        {
            if (mDoPrintDirName)
                Out << mEndOfLine 
                << Dir.GetRelativePath() 
                << ":" 
                << mEndOfLine;
            Out << "total " 
                << Dir.GetBlocks() 
                << mEndOfLine;
        }

        if (mSortBy == SORT_NAME)
            DoNameSort(Out, Dir);
        else
            DoOtherSort(Out, Dir);
        mpDir = NULL;
        mDoPrintDirName = true; // In case there are more dirs...
        return Out;
    }

    /** Parse a unix 'ls' command line.
    */
    void ParseArgs(int argc, char **argv, seq_file_list_t& List)
    {
        WarGetopt my_opts;
        static const struct option options[] =
        {
            {"all",no_argument, 0, 'a'},
                //  {"escape",no_argument, 0, 'b'},
            {"directory",no_argument, 0, 'd'},
            //  {"dired",no_argument, 0, 'D'},
            {"full-time",no_argument, 0, 1}, // NB m_bFullTime
            {"human-readable",no_argument, 0, 'h'},
            {"inode",no_argument, 0, 'i'},
            {"kilobytes",no_argument, 0, 'k'},
            {"numeric-uid-gid",no_argument, 0, 'n'},
            {"no-group",no_argument, 0, 'G'},
            {"hide-control-chars",no_argument, 0, 'q'},
            {"reverse",no_argument, 0, 'r'},
            {"size",no_argument, 0, 's'},
            {"width",required_argument, 0, 'w'},
            {"almost-all",no_argument, 0, 'A'},
            {"ignore-backups",no_argument, 0, 'B'},
            {"classify",no_argument, 0, 'F'},
            {"file-type",no_argument, 0, 'p'},
            {"si",no_argument, 0, 'H'},
            {"ignore",required_argument, 0, 'I'},
            {"indicator-style",required_argument, 0, 14},
            {"dereference",no_argument, 0, 'L'},
            {"literal",no_argument, 0, 'N'},
            {"quote-name",no_argument, 0, 'Q'},
            //  {"quoting-style",required_argument, 0, 15},
            {"recursive",no_argument, 0, 'R'},
            {"format",required_argument, 0, 12},
            {"show-control-chars",no_argument, 0, 16},
            {"sort",required_argument, 0, 10},
            //  {"tabsize",required_argument, 0, 'T'},
            {"time",required_argument, 0, 11},
            {"color",optional_argument, 0, 13},
            {"block-size",required_argument, 0, 17},
            {"help",no_argument, 0, 250},
            {"version",no_argument, 0, 251},
            {NULL, 0, NULL, 0}
        };
        
        int c = 0;    
        while ((c = my_opts.GetoptLong(argc, 
            argv, 
            "acdefghiklmnoqrstuvw:xABCFGHI:LNRSUX1",
            options, NULL)) > 0)
        {
            switch (c)
            {
            case 'a':
                mDoAllFiles = true;
                mDoReallyAllFiles = true;
                break;
                
                //        case 'b':
                //            set_quoting_style (NULL, escape_quoting_style);
                //            break;
                
            case 'c':
                mTimeType = TIME_CTIME;
                mSortBy = SORT_TIME;
                break;
                
            case 'd':
                mDoImmediateDirs = true;
                break;
                
            case 'f':
                /* Same as enabling -a -U and disabling -l -s.  */
                mDoAllFiles = 1;
                mDoReallyAllFiles = 1;
                mSortBy = SORT_NONE;
                /* disable -l */
                if (mListFormat == LONG_FORMAT)
                    mListFormat = ONE_PER_LINE;
                mDoPrintBlockSize = false;	/* disable -s */
                break;
                
            case 'g':
                /* No effect.  For BSD compatibility. */
                break;
                
            case 'h':
                mOutputBlockSize = -1024;
                break;
                
            case 'H':
                mOutputBlockSize = -1000;
                break;
                
            case 'i':
                mDoPrintInodes = true;
                break;
                
            case 'k':
                mOutputBlockSize = 1024;
                break;
                
            case 'l':
                mListFormat = LONG_FORMAT;
                break;
                
            case 'm':
                mListFormat = WITH_COMMAS;
                break;
                
            case 'n':
                mDoNumericIds = 1;
                break;
                
            case 'o':  /* Just like -l, but don't display group info.  */
                mListFormat = LONG_FORMAT;
                mDoHideGroup = true;
                break;
                
            case 'p':
                mIndicatorStyle = IS_FILE_TYPE;
                break;
                
            case 'q':
                mDoQmarkFunnyChars = true;
                break;
                
            case 'r':
                mDoReverseSort = 1;
                break;
                
            case 's':
                mDoPrintBlockSizeAll = 1;
                break;
                
            case 't':
                mSortBy = SORT_TIME;
                break;
                
            case 'u':
                mSortBy = SORT_TIME;
                mTimeType = TIME_ATIME;
                break;
                
            case 'v':
                mSortBy = SORT_VERSION;
                break;
                
            case 'w': // Ignore
                break;
                
            case 'x':
                mListFormat = HORIZONTAL;
                break;
                
            case 'A':
                mDoReallyAllFiles = 0;
                mDoAllFiles = 1;
                break;
                
            case 'B':
                mIgnorePatterns.push_back("*~");
                mIgnorePatterns.push_back(".*~");
                break;
                
            case 'C':
                mListFormat = MANY_PER_LINE;
                break;
                
                //        case 'D':
                //            dired = 1;
                //            break;
                
            case 'F':
                mIndicatorStyle = IS_CLASSIFY;
                break;
                
            case 'G':		/* inhibit display of group info */
                mDoHideGroup = 1;
                break;
                
            case 'I':
                mIgnorePatterns.push_back(my_opts.optarg);
                break;
                
            case 'L':
                mDoTraceLinks = true;
                break;
                
                //        case 'N':
                //            set_quoting_style (NULL, literal_quoting_style);
                //            break;
                
            case 'Q':
                mDoQuoteNames = true;
                break;
                
            case 'R':
                mDoRecursive = true;
                break;
                
            case 'S':
                mSortBy = SORT_SIZE;
                break;
                
                //        case 'T':
                //            break;
                
            case 'U':
                mSortBy = SORT_NONE;
                break;
                
            case 'X':
                mSortBy = SORT_EXTENSION;
                break;
                
            case '1':
                mListFormat = ONE_PER_LINE;
                break;
                
            case 10:		/* --sort */
                if (!strcmp(my_opts.optarg, "none"))
                    mSortBy = SORT_NONE;
                else if (!strcmp(my_opts.optarg, "time"))
                    mSortBy = SORT_TIME;
                else if (!strcmp(my_opts.optarg, "extension"))
                    mSortBy = SORT_EXTENSION;
                else if (!strcmp(my_opts.optarg, "version"))
                    mSortBy = SORT_VERSION;
                break;            
                
            case 11:		/* --time */
                if (!strcmp(my_opts.optarg, "atime"))
                    mTimeType = TIME_ATIME;
                if (!strcmp(my_opts.optarg, "access"))
                    mTimeType = TIME_ATIME;
                if (!strcmp(my_opts.optarg, "use"))
                    mTimeType = TIME_ATIME;
                if (!strcmp(my_opts.optarg, "ctime"))
                    mTimeType = TIME_CTIME;
                if (!strcmp(my_opts.optarg, "status"))
                    mTimeType = TIME_CTIME;
                break;            
                
            case 12:		/* --format */
                if (!strcmp(my_opts.optarg, "verbose"))
                    mListFormat = LONG_FORMAT;
                if (!strcmp(my_opts.optarg, "long"))
                    mListFormat = LONG_FORMAT;
                if (!strcmp(my_opts.optarg, "commas"))
                    mListFormat = WITH_COMMAS;
                if (!strcmp(my_opts.optarg, "HORIZONTAL"))
                    mListFormat = HORIZONTAL;
                if (!strcmp(my_opts.optarg, "across"))
                    mListFormat = LONG_FORMAT;
                if (!strcmp(my_opts.optarg, "vertical"))
                    mListFormat = MANY_PER_LINE;
                if (!strcmp(my_opts.optarg, "single-column"))
                    mListFormat = ONE_PER_LINE;
                break;
                
            case 13:		/* --color */
                // Just ignore
                break;
                
            case 14:		/* --indicator-style */
                if (!strcmp(my_opts.optarg, "none"))
                    mIndicatorStyle = IS_NONE;
                if (!strcmp(my_opts.optarg, "classify"))
                    mIndicatorStyle = IS_CLASSIFY;
                if (!strcmp(my_opts.optarg, "file_type"))
                    mIndicatorStyle = IS_FILE_TYPE;
                break;
                
                //        case 15:		/* --quoting-style */
                //            break;
                
            case 16:
                mDoQmarkFunnyChars = false;
                break;
                
            case 17:
                mOutputBlockSize = atoi(
                    my_opts.optarg ? my_opts.optarg : "0");
                break;
            case 250:
                Usage();
            case 251:
                mErrorMessage = "WarDirfmtUnixLs v. 3.0/Generic";
                mErrorMessage += mEndOfLine;
                WarThrow(WarError(WAR_ERR_VERSION), NULL);
            }    
        }
        
        for (;my_opts.optind < argc; my_opts.optind++)
            List.push_back(argv[my_opts.optind]);
    }

    /** Outputs to mErrorMessage and throws */
    void Usage() throw (WarException)
    {
        std::strstream buf;
    
        buf << "Usage: ls [OPTION]... [FILE]... " << mEndOfLine;
        buf << "In FTP servers: LIST [OPTION]... [FILE]" << mEndOfLine;
        buf << "List information about the FILEs (the current directory by default)." << mEndOfLine;
        buf << "Sort entries alphabetically if none of -cftuSUX nor --sort." << mEndOfLine;
        buf << mEndOfLine;
        buf << "-a, --all                  do not hide entries starting with ." << mEndOfLine;
        buf << "-A, --almost-all           do not list implied . and .." << mEndOfLine;
        //buf << "-b, --escape               print octal escapes for nongraphic characters" << mEndOfLine;
        buf << "    --block-size=SIZE      use SIZE-byte blocks" << mEndOfLine;
        buf << "-B, --ignore-backups       do not list implied entries ending with ~" << mEndOfLine;
        buf << "-c                         sort by change time; with -l: show ctime" << mEndOfLine;
        buf << "-C                         list entries by columns" << mEndOfLine;
        //buf << "    --color[=WHEN]         control whether color is used to distinguish file" << mEndOfLine;
        buf << "                           types.  WHEN may be `never', `always', or `auto'" << mEndOfLine;
        buf << "-d, --directory            list directory entries instead of contents" << mEndOfLine;
        //buf << "-D, --dired                generate output designed for Emacs' dired mode" << mEndOfLine;
        buf << "-f                         do not sort, enable -aU, disable -lst" << mEndOfLine;
        buf << "-F, --classify             append indicator (one of */=@|) to entries" << mEndOfLine;
        buf << "    --format=WORD          across -x, commas -m, HORIZONTAL -x, long -l," << mEndOfLine;
        buf << "                           single-column -1, verbose -l, vertical -C" << mEndOfLine;
        buf << "    --full-time            list both full date and full time\n";
        buf << "-g                         (ignored)" << mEndOfLine;
        buf << "-G, --no-group             inhibit display of group information" << mEndOfLine;
        buf << "-h, --human-readable       print sizes in human readable format (e.g., 1K 234M 2G)" << mEndOfLine;
        buf << "-H, --si                   likewise, but use powers of 1000 not 1024" << mEndOfLine;
        buf << "    --indicator-style=WORD append indicator with style WORD to entry names:" << mEndOfLine;
        buf << "                           none (default), classify (-F), file-type (-p)" << mEndOfLine;
        buf << "-i, --inode                print index number of each file" << mEndOfLine;
        buf << "-I, --ignore=PATTERN       do not list implied entries matching shell PATTERN" << mEndOfLine;
        buf << "-k, --kilobytes            like --block-size=1024" << mEndOfLine;
        buf << "-l                         use a long listing format" << mEndOfLine;
        buf << "-L, --dereference          list entries pointed to by symbolic links" << mEndOfLine;
        buf << "-m                         fill width with a comma separated list of entries" << mEndOfLine;
        buf << "-n, --numeric-uid-gid      list numeric UIDs and GIDs instead of names" << mEndOfLine;
        buf << "-N, --literal              print raw entry names (don't treat e.g. control" << mEndOfLine;
        buf << "                           characters specially)" << mEndOfLine;
        buf << "-o                         use long listing format without group info" << mEndOfLine;
        buf << "-p, --file-type            append indicator (one of /=@|) to entries" << mEndOfLine;
        buf << "-q, --hide-control-chars   print ? instead of non graphic characters" << mEndOfLine;
        buf << "    --show-control-chars   show non graphic characters as-is (default)" << mEndOfLine;
        buf << "-Q, --quote-name           enclose entry names in double quotes" << mEndOfLine;
        //buf << "    --quoting-style=WORD   use quoting style WORD for entry names:" << mEndOfLine;
        //buf << "                           literal, shell, shell-always, c, escape" << mEndOfLine;
        buf << "-r, --reverse              reverse order while sorting" << mEndOfLine;
        buf << "-R, --recursive            list subdirectories recursively" << mEndOfLine;
        buf << "-s, --size                 print size of each file, in blocks\n";
        buf << "-S                         sort by file size" << mEndOfLine;
        buf << "    --sort=WORD            extension -X, none -U, size -S, time -t," << mEndOfLine;
        buf << "                           version -v" << mEndOfLine;
        buf << "                           status -c, time -t, atime -u, access -u, use -u" << mEndOfLine;
        buf << "    --time=WORD            show time as WORD instead of modification time:" << mEndOfLine;
        buf << "                           atime, access, use, ctime or status; use" << mEndOfLine;
        buf << "                           specified time as sort key if --sort=time" << mEndOfLine;
        buf << "-t                         sort by modification time" << mEndOfLine;
        //buf << "-T, --tabsize=COLS         assume tab stops at each COLS instead of 8" << mEndOfLine;
        buf << "-u                         sort by last access time; with -l: show atime" << mEndOfLine;
        buf << "-U                         do not sort; list entries in directory order" << mEndOfLine;
        buf << "-v                         sort by version" << mEndOfLine;
        buf << "-w, --width=COLS           assume screen width instead of current value" << mEndOfLine;
        buf << "-x                         list entries by lines instead of by columns" << mEndOfLine;
        buf << "-X                         sort alphabetically by entry extension" << mEndOfLine;
        buf << "-1                         list one file per line" << mEndOfLine;
        buf << "    --help                 display this help and exit" << mEndOfLine;
        buf << "    --version              output version information and exit" << mEndOfLine;
        buf << '\0';
        
        mErrorMessage = buf.str();
        buf.freeze(false);
        
        WarThrow(WarError(WAR_ERR_USAGE), NULL);
    }

    // OPERATIONS    
    

    void InvalidPath(const std::string& path)
    {
    }

    // ACCESS
    // INQUIRY

    /// Export flags to WarSvrDirListing
    int ExportFlags()
    {
        int my_flags = DIRF_TRY_SAFE_PATTERNS;
        if (mDoRecursive)
            my_flags|= DIRF_RECURSIVE;
    
        if (!mDoReallyAllFiles)
            my_flags|= DIRF_HIDE_DOT_DOTDOT;
        if (!mDoReallyAllFiles || !mDoAllFiles)
            my_flags|= DIRF_HIDE_DOT_FILES;

        return my_flags;
    }

    void ExportExcludelist(WarDirList::patterns_t& excludeList)
    {
        for(ignore_patterns_t::const_iterator P = mIgnorePatterns.begin()
            ; P != mIgnorePatterns.end()
            ; P++)
        {
            excludeList.push_back(war_svrpath_t(WarUtf8(P->c_str()).GetUnicode().c_str()));
        }
    }
    
    /**@name Options, normally set via arguments to ParseArgs()
        
        Since some arguments applies for the scannig process as well,
        they may have to be passed on to \Ref{WarDirList} before
        the directory is scanned.
    */
    //@{
    ///
    bool mDoPrintBlockSize; 
    ///
    bool mDoPrintInodes;
    /// print for each node
    bool mDoPrintBlockSizeAll;
    ///
    bool mDoNumericIds;
    ///
    bool mDoHideGroup;
    ///
    bool mDoFullTime;
    ///
    bool mDoAllFiles;
    ///
    bool mDoReallyAllFiles;
    ///
    bool mDoRecursive;
    ///
    bool mDoPrintDirName;
    ///
    bool mDoImmediateDirs;
    ///
    bool mDoQmarkFunnyChars;
    ///
    bool mDoReverseSort;
    ///
    bool mDoTraceLinks;
    ///
    format mListFormat;
    ///
    sort_type mSortBy;
    ///
    time_type mTimeType;
    ///
    indicator_style mIndicatorStyle;
    ///
    ignore_patterns_t mIgnorePatterns;
    /// Keeps the sequence used to terminate a line
    char mEndOfLine[4];
    ///
    int mOutputBlockSize;
    ///
    bool mDoQuoteNames;
    //@}
    /// Returns error information if Usage() is called, or \Ref{List} fails.
    std::string mErrorMessage;
    
protected:
    const WarDirList *mpDir;
private:

    template <class outputT>
    void DoNameSort(outputT& Out, const WarDirList& Dir)
    {
        for(WarDirList::nodelist_t::const_iterator P = Dir.GetList().begin()
            ; P != Dir.GetList().end()
            ; P++)
        {
            OutNode(Out, P->Data());
        }

        //Out << (war_ccstr_t)Buffer;
    }

    void DoOtherSort(outputT& Out, const WarDirList& Dir)
    {
        assert(FALSE);
    }

    void OutNode(outputT& Out, const WarDirListNode& Node)
    {
        if (Node.mType > WarDirListNode::FT_LINK)
            return; // Hidden

        switch(mListFormat)
        {
        case LONG_FORMAT:
            OutLong(Out, Node);
            break;
        case ONE_PER_LINE:
            OutShort(Out, Node);
            break;
        case MANY_PER_LINE:
        case HORIZONTAL:
        case WITH_COMMAS:
             assert(FALSE);
        }
    }

    void OutShort(outputT& Out, const WarDirListNode& Node)
    {
        OutNameAndStuff(Out, Node);
        Out << mEndOfLine;
    }

    void OutNameAndStuff(outputT& Out, const WarDirListNode& Node)
    {
        if (mDoPrintInodes)
            OutInode(Out, Node);

        if (mDoPrintBlockSizeAll)
            OutBlocksize(Out, Node);

        OutQuotedName(Out, Node);
    }

    void OutQuotedName(outputT& Out, const WarDirListNode& Node)
    {

        if (mDoQuoteNames)
        {
            Out << '\"';
            Out << WarEscapeStr<char>(Node.mName, '\"');
            Out << '\"';
        }
        else
            Out << Node.mName;

        char my_indicator = 0;

        switch(mIndicatorStyle)
        {
        case IS_NONE:
            break;
        case IS_CLASSIFY:
            if (WAR_ISREG(Node))
            {
                if (Node.mMode & S_IXUGO)
                    my_indicator = '*';
                break;
            }
            // Fall trough
        case IS_FILE_TYPE:
            if (WAR_ISDIR (Node))
                my_indicator = '/';
            else if (WAR_ISLNK (Node))
                my_indicator = '@';
            else if (WAR_ISFIFO (Node))
                my_indicator = '|';
            else if (WAR_ISSOCK (Node))
                my_indicator = '=';
            else if (WAR_ISDOOR (Node))
                my_indicator = '>';
            break;
        }

        if (my_indicator)
            Out << my_indicator;
    }


    void OutLong(outputT& Out, const WarDirListNode& Node)
    {
        static const char spaces[] = {"        "};

        OutMode(Out, Node);

        if (mDoPrintInodes)
            OutInode(Out, Node);

        if (mDoPrintBlockSizeAll)
            OutBlocksize(Out, Node);

        OutLinks(Out, Node);

        std::string user_name /* = (mDoNumericIds ? "" : GetUser(Node.mUid))*/;
        if (user_name.empty())
            WarItoa(user_name, Node.mUid);

        if (user_name.length() < 8)
            user_name += &spaces[user_name.length() % 8];
        else
            user_name.erase(8);

        Out << user_name;

        if (!mDoHideGroup)
        {
            std::string group_name /*= (mDoNumericIds ? "" : GetGroup(Node.mGid))*/;
            if (group_name.empty())
                WarItoa(group_name, Node.mGid);
            
            if (group_name.length() < 8)
                group_name += &spaces[group_name.length() % 8];
            else
                group_name.erase(8);

            Out << group_name;
        }

        OutLength(Out, Node);
        OutDate(Out, Node);
        OutQuotedName(Out, Node);
        Out << mEndOfLine;
    }

    void OutLength(outputT& Out, const WarDirListNode& Node)
    {
        OutHumanDigit(Out, Node.mSize, 8);
    }

    // This can be done better...
    template <class intT>
    void OutHumanDigit(outputT& Out, const intT& value, size_t colWidth)
    {
        static const char *suffixes[] =
        {
            "",	/* not used */
            "k",	/* kilo */
            "M",	/* Mega */
            "G",	/* Giga */
            "T",	/* Tera */
            "P",	/* Peta */
            "E",	/* Exa */
            "Z",	/* Zetta */
            "Y"	/* Yotta */
        };

        if (0 > mOutputBlockSize)
            mOutputBlockSize *= -1;

        if ((0 == mOutputBlockSize) || (value < mOutputBlockSize))
        {
            // Most common situation, do it fast and easy!
            char buffer[32];
            if ((value <= ULONG_MAX) && (sizeof(buffer) > colWidth))
            {
                sprintf(buffer, "%*u ", colWidth, (unsigned)value);
                Out << buffer;
                return;
            }
            else
            {
                std::string buffer;
                Out << WarItoa(buffer, value);
                return;
            }
        }

        std::string buffer;
        double blockszie_in_power = mOutputBlockSize, next_power = 0;
        
        for(int power = 1; power < sizeof(suffixes); power++)
        {
            if (value < (next_power = (blockszie_in_power * mOutputBlockSize)))
            {
                char buf[512];
                double human_value = (double)(value) / blockszie_in_power;

                if (10 > human_value)
                    sprintf(buf, "%*.1lf%s ", colWidth -1, 
                        human_value,
                        suffixes[power]);
                else
                    sprintf(buf, "%*.0lf%s ", colWidth -1, 
                        human_value,
                        suffixes[power]);

                Out << buf;
                return;
            }
            
            blockszie_in_power = next_power;
        }

        Out << "########";
    }

    void OutInode(outputT& Out, const WarDirListNode& Node)
    {
        char buffer[16];
    
        sprintf(buffer, "%*ld ", INODE_DIGITS, Node.mInode);
        Out << buffer;
    }

    void OutBlocksize(outputT& Out, const WarDirListNode& Node)
    {
        char buffer[16];
    
        sprintf(buffer, "%*ld ", mpDir->GetBlockSizeLen(), (long)(Node.mSize / 1024));
        Out << buffer;
    }

    void OutLinks(outputT& Out, const WarDirListNode& Node)
    {
        char buffer[16];
    
        sprintf(buffer, "%3u ", (unsigned)Node.mLinks);
        Out << buffer;
    }

    void OutMode(outputT& Out, const WarDirListNode& Node)
    {   
        char str[12];

        switch(Node.mType)
        {
        case WarDirListNode::FT_DIR:
            str[0] = 'd';
            break;
        case WarDirListNode::FT_LINK:
            str[0] = 'l';
            break;
        default:
            str[0] = '-';
        }

        str[1] = Node.mMode & S_IRUSR ? 'r' : '-';
        str[2] = Node.mMode & S_IWUSR ? 'w' : '-';
        str[3] = Node.mMode & S_IXUSR ? 'x' : '-';
        str[4] = Node.mMode & S_IRGRP ? 'r' : '-';
        str[5] = Node.mMode & S_IWGRP ? 'w' : '-';
        str[6] = Node.mMode & S_IXGRP ? 'x' : '-';
        str[7] = Node.mMode & S_IROTH ? 'r' : '-';
        str[8] = Node.mMode & S_IWOTH ? 'w' : '-';
        str[9] = Node.mMode & S_IXOTH ? 'x' : '-';
        str[10] = ' '; // ACL
        str[11] = 0;

        Out << str;
    }

    template <class outputT>
    void OutDate(outputT& Out, const WarDirListNode& Node)
    {
        WarTime when, Now;
        war_ccstr_t fmt;

        switch (mTimeType)
        {
        case TIME_CTIME:
            when = Node.mCreationTime;
            break;
        case TIME_MTIME:
            when = Node.mModifyTime;
            break;
        case TIME_ATIME:
            when = Node.mAccessTime;
            break;
        }
        
        if (mDoFullTime)
        {
            fmt = "%a %b %d %H:%M:%S %Y ";
        }
        else
        {
            if (Now.GetTime() > when.GetTime() + 6L * 30L * 24L * 60L * 60L	/* Old. */
                || Now.GetTime() < when.GetTime() - 60L * 60L)	/* In the future. */
            {
            /* The file is fairly old or in the future.
            POSIX says the cutoff is 6 months old;
            approximate this by 6*30 days.
            Allow a 1 hour slop factor for what is considered "the future",
            to allow for NFS server/client clock disagreement.
                Show the year instead of the time of day.  */
                fmt = "%b %d  %Y ";
            }
            else
            {
                fmt = "%b %d %H:%M ";
            }
        }

        Out << when.FormatGMT(fmt);
    }
};

/* INLINE METHODS */

/* EXTERNAL REFERENCES */

#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* WAR_DIRFMT_UNIX_LS_H_ */

