/** @name Error codes
*
*  #include "WarErrorCodes.h"
*/
/*@{ */

#ifndef WARERRORCODES_H
#define WARERRORCODES_H

/* SYSTEM INCLUDES */
#if defined(_WIN32) && !defined(_WINSOCK2API_)
#  include <winsock2.h>
#endif

#include "WarTypes.h"

/* Error strings */
typedef struct war_error_text
{
    const int	  mErrorCode;
    war_ccstr_t	  mErrotText;
}
war_error_text_t;


#ifdef __cplusplus
extern "C"
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/
    
#ifdef _WIN32
enum war_winsock_to_unix_map 
{
    EWOULDBLOCK     = WSAEWOULDBLOCK,
    EINPROGRESS     = WSAEINPROGRESS,
    EALREADY	    = WSAEALREADY,
    ENOTSOCK        = WSAENOTSOCK,
    EDESTADDRREQ    = WSAEDESTADDRREQ,
    EMSGSIZE        = WSAEMSGSIZE,
    EPROTOTYPE      = WSAEPROTOTYPE,
    ENOPROTOOPT     = WSAENOPROTOOPT,
    EPROTONOSUPPORT = WSAEPROTONOSUPPORT,
    ESOCKTNOSUPPORT = WSAESOCKTNOSUPPORT,
    EOPNOTSUPP      = WSAEOPNOTSUPP,
    EPFNOSUPPORT    = WSAEPFNOSUPPORT,
    EAFNOSUPPORT    = WSAEAFNOSUPPORT,
    EADDRINUSE      = WSAEADDRINUSE,
    EADDRNOTAVAIL   = WSAEADDRNOTAVAIL,
    ENETDOWN        = WSAENETDOWN,
    ENETUNREACH     = WSAENETUNREACH,
    ENETRESET       = WSAENETRESET,
    ECONNABORTED    = WSAECONNABORTED,
    ECONNRESET      = WSAECONNRESET,
    ENOBUFS         = WSAENOBUFS,
    EISCONN         = WSAEISCONN,
    ENOTCONN        = WSAENOTCONN,
    ESHUTDOWN       = WSAESHUTDOWN,
    ETOOMANYREFS    = WSAETOOMANYREFS,
    ETIMEDOUT       = WSAETIMEDOUT,
    ECONNREFUSED    = WSAECONNREFUSED,
    ELOOP           = WSAELOOP,
    EHOSTDOWN       = WSAEHOSTDOWN,
    EHOSTUNREACH    = WSAEHOSTUNREACH,
};
#endif // _WIN32
    
/* Place-holders for ragnes of error-numbers */
enum war_error_def_classes 
{
    WAR_ERRC_GENERAL = 1000,
    WAR_ERRC_NETWORK = 1100,
    WAR_ERRC_THREAD  = 1200,
    WAR_ERRC_CHRCNV  = 1300,
    WAR_ERRC_FSYS    = 1400,
    WAR_ERRC_FTPD    = 1500,
    WAR_ERRC_PLUGIN  = 1600
};
    
/** @name Error values */ /*@{*/
/** Enum of error values */
enum war_error_definitions {
    /** No error, value 0 */
    WAR_ERR_OK = 0,
            
    /**@name General error codes */
    /**@{ */
            
    /**  */
    WAR_ERR_SYSTEM_ERROR = WAR_ERRC_GENERAL,
    /**  */
    WAR_ERR_NOT_IMPLEMENTED,
    /**  */
    WAR_ERR_UNEXCPECTED_REPLY_CODE,
    /**  */
    WAR_ERR_INVALID_ARGUMENT,
    /**  */        
    WAR_ERR_MISSING_REQ_FILE,
    /**  */
    WAR_ERR_PARSER_ERROR,
    /**  */
    WAR_ERR_LEXICAL_ERRROR,
    /**  */
    WAR_ERR_DNS_LOOKUP_FAILED,
    /**  */
    WAR_ERR_INTERNAL_DATA_NOT_INITIALIZED,
    /**  */
    WAR_ERR_ALREADY_INITIALIZED,
    /**  */
    WAR_ERR_INVALID_CASE_VALUE,
    /**  */
    WAR_ERR_SYSTEM_CALL_FAILED,
    /**  */
    WAR_ERR_COMMAND_FAILED,
    /**  */
    WAR_ERR_OUT_OF_MEMORY,
    /**  */
    WAR_ERR_BUFFER_OVERFLOW,
    /**  */
    WAR_ERR_NAME_EXIST,
    /**  */
    WAR_ERR_NAME_DONT_EXIST,
    /**  */
    WAR_ERR_NO_OBJECT,
    /**  */
    WAR_ERR_OBJECT_EXIST,
    /**  */
    WAR_ERR_OBJECT_NOT_FOUND,
    /**  */
    WAR_ERR_TIMED_OUT,
    /**  */
    WAR_ERR_BAD_SYSTEM_STATUS,
    /**  */
    WAR_ERR_FEATURE_NOT_SUPPORTED,
    /**  */
    WAR_ERR_OBJECT_TOO_LONG,
    /**  */
    WAR_ERR_INVALID_HEADER,
    /**  */
    WAR_ERR_COULD_NOT_OPEN_FILE,
    /**  */
    WAR_ERR_USAGE,
    /**  */
    WAR_ERR_VERSION,
    /**  */
    WAR_ERR_ABORT_THREAD,
    /** */
    WAR_ERR_INVALID_PATH,
    /** */
    WAR_ERR_INVALID_URL,
    /** */
    WAR_ERR_NOT_AN_URL,
     /** */
    WAR_ERR_NOT_OPEN,
     /** */
    WAR_ERR_ALREADY_OPEN,
    /** */
    WAR_ERR_NOT_OPEN_FOR_READ,
    /** */
    WAR_ERR_NOT_OPEN_FOR_WRITE,
    /** */
    WAR_ERR_ACCESS_DENIED,
    /** */
    WAR_ERR_OUT_OF_RANGE,
    /** */
    WAR_ERR_SECURITY_VIOLATION,
    /** */
    WAR_ERR_SHUTDOWN,
    /** */
    WAR_ERR_DISABLED,
    /** */
    WAR_ERR_IP_IS_DENIED,
    /** */
    WAR_ERR_FAILED_TO_LOCK_OBJECT,
    /**@} */

    
    /**@name Network errors */
    /**@{ */
    /**  */
    WAR_NETERR_INVALID_HOSTADDRESS = WAR_ERRC_NETWORK,
    /** */
    WAR_NETERR_INVALID_PORT_OR_SERVICE,
    /** */
    WAR_NETERR_UNSUPPORTED_FAMILY,
    /** */
    WAR_NETERR_NO_AVAIL_PORTS_IN_RANGE,
    /**@} */


    /**@name WarThread errors */
    /**@{ */
    /**  */
    WAR_THREADERR_FAILED_TO_CREATE = WAR_ERRC_THREAD,
    /**  */
    WAR_THREADERR_FAILED_TO_INITIALIZE,
    /**  */
    WAR_THREADERR_FAILED_TO_INITIALIZE_TLS,
    /**  */
    WAR_THREADERR_FAIED_TO_INITIALIZE_MUTEX,
    /**  */
    WAR_THREADERR_MUTEX_LOCK_FAILED,
    /**  */
    WAR_THREADERR_MUTEX_UNLOCK_FAILED,
    /**  */
    WAR_THREADERR_FAIED_TO_INITIALIZE_EVENT,
    /**  */
    WAR_THREADERR_EVENT_WAIT_FAILED,
    /**  */
    WAR_THREADERR_EVENT_SIGNAL_FAILED,
    /** */
    WAR_THREADERR_NO_TSD,

    /**@} */

    /**@name Conversiopns errors */
    /**@{ */
    /**  */
    WAR_CHRCNV_NO_CONVERSION = WAR_ERRC_CHRCNV,
    /** */
    WAR_CHRCNV_NOT_UTF8,
    /**@} */

    /**@name File anf filesystem errors */
    /**@{ */
    /**  */
    WAR_FERR_PATH_IS_A_DIR = WAR_ERRC_FSYS,
    /** */
    WAR_FERR_END_OF_FILE,
    /** */
    WAR_FERR_WRITE_FAILED,
    /** */
    WAR_FERR_NOT_A_DRIVE,
    /** */
    WAR_FERR_CALLBACK_FAILED,
    /** */
    WAR_FERR_NOT_OPENED_WPTH_CALLBACK,
    /** */
    WAR_FERR_NO_SUCH_PATH,
    /** */
    WAR_FERR_NOT_OPEN_FOR_READ,
    /** */
    WAR_FERR_NOT_A_DIRECTORY,
    /** */
    WAR_FERR_NOT_A_FILE,
    /** */
    WAR_FERR_RECURSIVE_LISTINGS_FORBIDDEN,
    /** */
    WAR_FERR_NOT_SAME_PATH,
    
    /**@} */
    /**@name FTP Daemon errors */
    /**@{ */
    /**  */
    WAR_FTPDERR_MISSING_ARGUMENT = WAR_ERRC_FTPD,
    /**  */
    WAR_FTPDERR_ARGUMENT_NOT_ALLOWED,
    /** */
    WAR_FTPDERR_NO_DATA_CONNECTION,
    /** */
    WAR_FTPDERR_ABORTED,
    /** */
    WAR_FTPDERR_NOASCIIREST,
    /** */
    WAR_FTPDERR_REST_OUT_OF_RANGE,
    /** */
    WAR_FTPERR_BAD_SEQUENCE_OF_CMDS,

    /**@} */
    /**@name FTP Daemon errors */
    /**@{ */
    /** Do not process more plogins, proceed */
    WAR_ERR_PLUGIN_NONEXT = WAR_ERRC_PLUGIN,
    /** No default processing */
    WAR_ERR_PLUGIN_DONE, 
    
    /**@} */
    WAR_ERROR_INVALID_ERROR_VALUE
};
    
/*@} Error values*/

/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus
}
#endif


extern const war_error_text_t WarErrorText[];
extern const war_error_text_t WarSysErrorText[];

#endif  /* _WARERRORCODES_H_ */
/*@} */
