/** The WarLibrary's exception object.
 *
 * #include "WarException.h" <BR>
 * -llib 
 *
 * @doc Any exceptions thrown by the 
 * WarLibrary is of this type (or derievd
 * from this type). 
 *  
 * @see WarError
 */

#ifndef WAR_EXCEPTION_H
#define WAR_EXCEPTION_H

/* SYSTEM INCLUDES */


/* PROJECT INCLUDES */

#ifndef WAR_ERROR_H
#   include "WarError.h"
#endif


/* LOCAL INCLUDES */


/* FORWARD REFERENCES */



#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/


/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/

#ifdef __cplusplus

class WarException : public WarError
{
public:
    // LIFECYCLE
    
    /**
    * Default constructor.
    */
    WarException(const war_error_definitions local_error,
             const int system_error = 0,
             war_ccstr_t message = NULL,
             war_ccstr_t source_file = __FILE__,
             const int source_line = __LINE__);

    /**
    * Inherit WarError constructor.
    */
    WarException(const WarError& err,
             war_ccstr_t message = NULL);


    /** Copy constructor
    */
    WarException(const WarException& from);
        
    /**
    * Destructor.
    */
    ~WarException(void);
    
    // OPERATORS
    // OPERATIONS                       

    virtual std::string Explain() const;

    // ACCESS
    // INQUIRY
    
protected:
    war_ccstr_t mMessage;
private:
};

/* INLINE METHODS */

/* EXTERNAL REFERENCES */

///
#define WarThrow(err, msg)\
throw WarException(WarError(err.LocalError(), err.SystemError(), __FILE__, __LINE__), msg)

#ifdef WIN32
#   define WarThrowSyserr(msg) \
    throw WarException(WarError(WAR_ERR_SYSTEM_ERROR, ::GetLastError(), __FILE__, __LINE__), msg)
#else
#   define WarThrowSyserr(msg) \ 
    throw WarException(WarError(WAR_ERR_SYSTEM_ERROR, errno, __FILE__, __LINE__), msg)
#endif

#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* _WAR_EXCEPTION_H_ */

