/** 
*/

#ifndef WAR_FUNCTION_PROFILER_H
#define WAR_FUNCTION_PROFILER_H

/* SYSTEM INCLUDES */


/* PROJECT INCLUDES */

#ifndef WAR_TYPES_H
#   include "WarTypes.h"
#endif

/* LOCAL INCLUDES */


/* FORWARD REFERENCES */



#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/


#define WAR_PROFILE_FUNC(id) \
    WarCodeTimer my_func_timer(id)

/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/

#ifdef __cplusplus

enum WarProfilerItemE
{
    WAR_FP_FTPDATA_PUMP,
    WAR_FP_FTPDATA_PUMP_FREAD_ASYNCH,
    WAR_FP_FTPDATA_PUMP_FREAD_SYNCH,
    WAR_FP_FTPDATA_PUMP_FREAD_CHKEOF,
    WAR_FP_FTPDATA_PUMP_SCK_OUT,
    WAR_FP_FTPDATA_PUMP_RECEIVING,
    WAR_FP_WarSvrProtocolFtpDataConn_ProcessPendingBuffers,
    WAR_FP_WarSvrProtocolFtpDataConn_OnTransferDone,
    WAR_FP_WarSvrProtocolFtpDataConn_OnAccept,
    WAR_FP_WarSvrProtocolFtpDataConn_OnConnect,
    WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo,
    WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_SendWithCallback,
    WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_ERROR,
    WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_QUEUE,
    WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_RECV,
    WAR_FP_WarSvrProtocolFtpDataConn_OnReceived,
    WAR_FP_WarSvrProtocolFtpDataConn_OnSent,
    WAR_FP_WarFileDriverFileWin32_ReadWithCallback,
    WAR_FP_WarFileDriverFileWin32_ReadWithCallback_ALLOC,
    WAR_FP_WarFileDriverFileWin32_ReadWithCallback_PREP,
    WAR_FP_WarFileDriverFileWin32_ReadWithCallback_READFILE,
    WAR_FP_WarFileDriverFileWin32_ReadWithCallback_POSTPRC,
    WAR_FP_WarFileDriverFileWin32_WriteWithCallback,
    WAR_FP_WarFileDriverFileWin32_AllocResult_NEW,
    WAR_FP_WarFileDriverFileWin32_AllocResult_ASSIGN,
    WAR_FP_WarSocketIoWin32Nt_SendWithCallback,
    WAR_FP_WarSocketIoWin32Nt_SendWithCallback_ALLOC,
    WAR_FP_WarSocketIoWin32Nt_SendWithCallback_READFILE,
    WAR_FP_WarSocketIoWin32Nt_SendWithCallback_POSTPRC,
    WAR_FP_WarSocketIoWin32Nt_RecvWithCallback,
    WAR_FP_WarSocketIoWin32Nt_AllocResult_NEW,
    WAR_FP_WarSocketIoWin32Nt_AllocResult_ASSIGN,
    WAR_FP_WarUserAuthDataWin32Nt_BeginImpersonate,
    WAR_FP_WarUserAuthDataWin32Nt_EndImpersonate,
    WAR_FP_WarSvrProtocolFtpDataConn_OnReceived_SIMPLE,
    WAR_FP_WarSvrProtocolFtpDataConn_OnReceived_QUEUE,

    WAR_FT_INVALID
};

class WarFunctionProfiler 
{
public:

    typedef struct profile_data_def
    {
        war_int64_t pd_total_time; // Time in milliseconds
        war_int64_t pd_counter; // Counter
    } data_t;
    
    // LIFECYCLE
    
    /**
    * Default constructor.
    */
    WarFunctionProfiler();
     
    /**
    * Destructor.
    */
    ~WarFunctionProfiler();
    
    // OPERATORS
    // OPERATIONS

    inline Record(WarProfilerItemE indexId, 
        const war_int64_t& msElapsed)
    {
        ::EnterCriticalSection(&mhCritSect);

        data_t *p = &mData[indexId];
        p->pd_total_time += msElapsed;
        p->pd_counter++;

        ::LeaveCriticalSection(&mhCritSect);
    }   

    // CALLBACK
    // ACCESS
    // INQUIRY

    std::string Explain() const;
    
protected:
    friend class WarCodeTimer;
    static WarFunctionProfiler *mspProfiler;

private:
    data_t mData[WAR_FT_INVALID];
    CRITICAL_SECTION mhCritSect;
    war_int64_t mTimerResolution; // factor of 1 second
};


class WarCodeTimer
{
public:
    inline WarCodeTimer(WarProfilerItemE itemIndex) : 
      mItemIndex(itemIndex)
      {
        if (!QueryPerformanceCounter(
            (LARGE_INTEGER *)&mStartTime))
        {
            mStartTime.QuadPart = 0;
        }
      }

      inline ~WarCodeTimer()
      {
          if (WarFunctionProfiler::mspProfiler)
          {
              LARGE_INTEGER elapsed = {0,0};
              QueryPerformanceCounter(&elapsed);

              elapsed.QuadPart -= mStartTime.QuadPart;
              
              WarFunctionProfiler::mspProfiler->Record(
                  mItemIndex, elapsed.QuadPart);
          }
      }

private:
    LARGE_INTEGER mStartTime;
    WarProfilerItemE mItemIndex;
};

#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* WAR_FUNCTION_PROFILER_H_ */

