/** Memory organizer for small data blocks that can be 
    freed together. Typically used for file-names, 
    symbol table names and temporary storage.
 
    This method does not work for classes. The advantage 
    is that memory allocation is extreemly fast, and 
    that small blocks woun't defragment the main 
    memory. The disadvantage is that individual blocks 
    of memory can't be freed. The memory remains 'used' 
    until the WarMemoryBuffer object is destroyed.

    Classes that are allocated using this scheme must
    be derived from WarImplementMemory
*/

#ifndef WAR_MEMORY_BUFFER_H
#define WAR_MEMORY_BUFFER_H

/* SYSTEM INCLUDES */

#ifndef WAR_LIST_INCLUDED
#   define WAR_LIST_INCLUDED
#   include <list>
#endif

/* PROJECT INCLUDES */


/* LOCAL INCLUDES */


/* FORWARD REFERENCES */



#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/


/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/

#ifdef __cplusplus
template <class T>
class WarMemoryBuffer 
{
public:
    typedef std::list<T *> block_list_t;
    // LIFECYCLE
    
    /// Constructor
    WarMemoryBuffer(size_t AllocSize = (1024 * 16))
        : mEndOfBlock(NULL),
        mpNextPtr(NULL),
        mAllocSize(AllocSize),
        mTotalAllocatedSize(0)
    {}
    
    ~WarMemoryBuffer()
    {
        Reset();
    }
    
    // OPERATORS
    // OPERATIONS

    ///
    void Reset()
    {
        while(!mBlocks.empty())
        {
            delete mBlocks.front();
            mBlocks.pop_front();
        }
        mpNextPtr = NULL;
        mAllocSize = 0;
        mTotalAllocatedSize = 0;
        mEndOfBlock = NULL;
    }

    /** @param Number of T's to get 
        (bytes = Size * sizeof(T)
    */
    T *Malloc(size_t Size)
    {
        if ((GetFree() * sizeof(T)) < Size)
            AllocBlock();

        T *p = mpNextPtr;
        mpNextPtr += Size;
        mTotalAllocatedSize += Size * sizeof(T);
        ByteAlign();
        return p;
    }

    ///
    T *StrDup(std::basic_string<T>& from)
    {
        T *p = Malloc(from.size() +1);
        memcpy(p, from.c_str(), from.size() * sizeof(T));
        p[from.size()] = 0;
        return p;
    }

    /// Same as the C API function malloc()
    char *MallocBytes(size_t bytes)
    {
        if (GetFree() < bytes)
            AllocBlock();

        char *p = (char *)mpNextPtr;
        mpNextPtr = (T *)(char *)(p + bytes);
        mTotalAllocatedSize += bytes;
        ByteAlign();
        return p;
    }

    // ACCESS
    // INQUIRY
    
protected:
private:
    void ByteAlign()
    {
        if ((int)mpNextPtr % sizeof(int))
        {
            int AddBytes = sizeof(int) - ((int)mpNextPtr % sizeof(int));
            mpNextPtr = (T *)(char *)((char *)mpNextPtr) + AddBytes;
            mTotalAllocatedSize += AddBytes;
        }
    }

    inline size_t GetFree()
    {
        return mEndOfBlock - mpNextPtr;
    }

    void AllocBlock()
    {
        mBlocks.push_back(mpNextPtr = new T[mAllocSize]);
        mEndOfBlock = (war_ccstr_t)mpNextPtr + mAllocSize;
    }

    war_ccstr_t mEndOfBlock;
    T *mpNextPtr;
    size_t mAllocSize;
    size_t mTotalAllocatedSize;
    block_list_t mBlocks;
};

/* INLINE METHODS */

/* EXTERNAL REFERENCES */


///
#define WarMallocStruct(type)\
(type *)m_Mem.MallocBytes(sizeof(type))


#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* WAR_MEMORY_BUFFER_H_ */

