/** 
*/

#ifndef WAR_PATH_H
#define WAR_PATH_H

/* SYSTEM INCLUDES */

#ifndef WAR_IOSTREAM_INCLUDED
#   define WAR_IOSTREAM_INCLUDED
#   include <iostream>
#endif

#ifndef WAR_ASSERT_H_INCLUDED
#   define WAR_ASSERT_H_INCLUDED
#   include <assert.h>
#endif

/* PROJECT INCLUDES */

#ifndef WAR_COLLECTOR_H
#   include "WarCollector.h"
#endif

#ifndef WAR_EXCEPTION_H
#   include "WarException.h"
#endif

#ifndef WAR_FILE_ENUMS_H
#   include "WarFileEnums.h"
#endif

/* LOCAL INCLUDES */


/* FORWARD REFERENCES */



#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/


/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/

#ifdef __cplusplus


enum WarPathE
{
    WAR_SYSSLASH, /// SLASH OR BACKSLASH
    WAR_DSYSSLASH, /// DOUBLE SLASH OR BACKSLASH
    WAR_SLASH, /// '/'
    WAR_DSLASH, /// '//'
    WAR_CDIR /// './'
};

class WarSlash
{
public:
    enum SlashE
    {
        SYS_SLASH = '/'
    };
};

class WarSysSlash : public WarSlash
{
public:
    enum SlashE
    {
#ifdef WIN32
        SYS_SLASH = '\\'
#else
        SYS_SLASH = '/'
#endif    
    };

    static bool IsSlash(const char ch) 
    {
        if (SYS_SLASH != '/')
        {
            if (SYS_SLASH == ch)
                return true;
        }
        return '/' == ch;
    }

};

template <class charT, class slashT = WarSysSlash>
class WarPath : public WarCollector<charT>,
    public WarFileEnums
{
public:
    typedef std::basic_string<charT> path_string_t;
    typedef charT const * path_ccstring_t;
    typedef charT path_char_t; 
    typedef slashT sys_slash_t;
    typedef WarPath<charT, slashT> path_t;

    WarPath<charT, slashT>()
    {}

    template <class fcT, class fsT>
    WarPath<charT, slashT>(const WarPath<fcT, fsT>& from)
    {
        *this << (WarCollector<fcT>)from;
    }

    template <class fcT>
    WarPath<charT, slashT>(const WarCollector<fcT>& from)
    {
        *this << (WarCollector<fcT>)from;
    }

    template <class fcT>
    WarPath<charT, slashT>(const std::basic_string<fcT>& from)
    {
        *this << from;
    }

    WarPath<charT, slashT>(war_ccstr_t from)
    {
        *this << from;
    }

    WarPath<charT, slashT>(war_cwstr_t from)
    {
        *this << from;
    }

    // LIFECYCLE 
    // OPERATORS

    path_char_t operator [] (int offset) const
    {
        return mString[offset];
    }

    template <class T>
    WarPath<charT, slashT>& operator = (const T& from)
    {
        mString.erase();
        *this << from;
        return *this;
    }

    bool operator == (const WarPath<charT, slashT>& from)
    {
        return mString == from.mString;
    }

    // OPERATIONS
    
    bool CutAtLastCh (path_char_t ch)
    {
        path_string_t::size_type 
            len = mString.find_last_of(ch);
        if (len != mString.npos)
            mString.resize(len);
        return 0 < len;
    }

    bool CutAtFirstCh(path_char_t ch)
    {
        path_string_t::size_type 
            len = mString.find_first_of(ch);
        if (len != mString.npos)
            mString.resize (len);
        return 0 < len;
    }

    void ClipAfterLevel(size_t level) 
        throw(WarException)
    {
        path_ccstring_t p = mString.c_str(),
            p_start = p;
        for(int i = 0; i < level; i++)
        {
            while(*p && ('\\' != *p))
                ++p;
            if ('\\' == *p)
                ++p;
        }

        if (i != level)
            WarThrow(WarError(WAR_ERR_OUT_OF_RANGE), NULL);

        mString.resize(p - p_start);
    }
    
    /// Prepere for use. Set slashes the right way etc.
    void Normalize()
    {
        if (sys_slash_t::SYS_SLASH != '/')
        {
            for(path_string_t::iterator P = mString.begin()
                ; P != mString.end()
                ; P++)
            {
                if (*P == '/')
                    *P = (path_char_t)sys_slash_t::SYS_SLASH;
            }
        }
    }

    /// Convert any backslashes to posix slashes (/)
    void MakePosixPath()
    {
        for(path_string_t::iterator P = mString.begin()
            ; P != mString.end()
            ; P++)
        {
            if (*P == '\\')
                *P = '/';
        }
    }

    void RemovePaddingSlash()
    {
        while(IsSlash(GetLastCh()))
            mString.erase(mString.length() -1);
    }

    /** Deparses a path by interperating and removing
      * "./" and "../"
      * Known exploits, like "..." under Win32 throws an
      * exception. So does paths that tries to CD up beoynd
      * the root.
      */
    void Deparse() throw(WarException)
    {
        std::vector<path_char_t> buffer(mString.size() + 1);
        const path_char_t *p = mString.c_str();
        path_char_t *pp = &buffer[0];
        int cnt = 0;

        while (*p)
        {
            if (IsSlash(*p))
            {
                // /C/.. /C/./any  /C/any/. /C/any/../.any /C/any\..
                if (p[1] == '.')
                {
                    if (!p[2] || IsSlash(p[2]))
                    {
                        // /C/any/. style. Just ignore
                        p += 2;
                        continue;
                    }

                    if (p[2] == '.')
                    {
                        if (!p[3] || IsSlash(p[3]))
                        {
                            // /C/any/.. /C/any/../test style. Move back one
                            if (cnt == 0)
                                WarThrow(WarError(WAR_ERR_INVALID_PATH), NULL);

                            while (--cnt && !IsSlash(pp[cnt]))
                            {
                                assert(cnt >= 0);
                            }
                            p += 3;
                            continue;
                        }
                        else
                        {
                            const path_char_t *ppp = p;
                            while(*ppp == '.')
                                ++ppp;

                                // Deny /......[/]
                            if (!*ppp 
                                || IsSlash(*ppp) 
                                // We deny some common characters with special meanings as well!
                                // /..*../
                                || (*ppp == '*') 
                                || (*ppp == '?')
                                || (*ppp == '~'))
                            {
                                // Possible exploit
                                WarThrow(WarError(WAR_ERR_INVALID_PATH), NULL);
                            }
                        }
                    }
                }
            }
            // Default
            pp[cnt++] = *p++;
        }
        
        pp[cnt++] = 0;
        mString = &buffer[0];
    }
    
    // ACCESS
    // INQUIRY

    /** Return the number of letters in the path */
    size_t GetLength() const
    {
        return mString.size();
    }

    inline char GetSlash() const
    {
        return (char)sys_slash_t::SYS_SLASH;
    }

    path_ccstring_t GetPath() const
    {
        return mString.c_str();
    }

    WarPath<charT, slashT> GetFilename() const
    {
        path_string_t::size_type 
            position = mString.find_last_of(
                sys_slash_t::SYS_SLASH);
        return mString.c_str() + (int)((position == mString.npos) 
            ? 0 : (position + 1));
    }

    WarPath<charT, slashT> GetPathname() const
    {
        path_string_t::size_type 
            position = mString.find_last_of(
                sys_slash_t::SYS_SLASH);
        
        if (position == mString.npos)
        {
            charT buf[sizeof(int)];
            buf[0] = '.';
            buf[1] = sys_slash_t::SYS_SLASH;
            buf[2] = 0;
            return buf;
        }
        
        if (position == 0)
        {
            charT buf[sizeof(int)];
            buf[0] = sys_slash_t::SYS_SLASH;
            buf[1] = 0;
            return buf;
        }
        
        return mString.substr(0, position);
    }

    WarPath<charT, slashT> GetExtension() const
    {
        path_string_t::size_type index = 
            mString.find_last_of('.');
        if (index == path_string_t::npos)
            return WarPath(); // Blank
        return mString.c_str() + (index +1);
    }

    bool IsEmpty() const
    {
        return mString.empty();
    }

    path_char_t GetLastCh() const
    {
        if (!mString.empty())
            return mString[mString.size() -1];
        return 0;
    }

    bool IsRoot() const
    {
        return ((GetLength() == 1) 
            && (mString[0] == '/'));
    }

    static bool IsSlash(const path_char_t ch) 
    {
        if (sys_slash_t::SYS_SLASH != '/')
        {
            if (sys_slash_t::SYS_SLASH == ch)
                return true;
        }
        return '/' == ch;
    }

    int Strnicmp(const char * right, size_t length) const
    {
        assert(sizeof(path_char_t) == sizeof(char));
        return ::strnicmp((char *)GetPath(), 
            right, length);
    }
    
    int Strnicmp(const wchar_t * right, size_t length) const
    {
        assert(sizeof(path_char_t) == sizeof(wchar_t));
        return ::wcsnicmp((wchar_t *)GetPath(), 
            right, length);
    }
    
    int Stricmp(const char * right) const
    {
        assert(sizeof(path_char_t) == sizeof(char));
        return ::strnicmp((char *)GetPath(), right);
    }
    
    int Stricmp(const wchar_t * right) const
    {
        assert(sizeof(path_char_t) == sizeof(wchar_t));
        return ::wcsicmp((wchar_t *)GetPath(), right);
    }
    
    int Strcmp(const char * right) const
    {
        assert(sizeof(path_char_t) == sizeof(char));
        return ::strnicmp((char *)GetPath(), right);
    }
    
    int Strcmp(const wchar_t * right) const
    {
        assert(sizeof(path_char_t) == sizeof(wchar_t));
        return ::wcsnicmp((wchar_t *)GetPath(), right);
    }
    
    /** Gets the first part of the path. Typically
    used by lame WIN32 calls that needs the
    initial C:\ or \\SERVER\SHARE\ to return
    interesting information.
    
      @return Win32: The file system part of the path. 
      UNIX : Returns the full path.
    */
    WarPath<charT, slashT> GetMountPath(bool do_expand = true) const
    {
        if (IsEmpty())
            WarThrow(WarError(WAR_ERR_INVALID_ARGUMENT), "empty path");
        
#ifdef WIN32
        //WarUrl MyUrl = mPath;
        //WarPath RealPath = MyUrl.GetFilePath().GetPath();
        WarPath<charT, slashT>RealPath = *this;
        path_ccstring_t p = RealPath.mString.c_str();
        WarPath<charT, slashT> MyPath;
        
        if ((p[0] == '\\') 
            && (p[1] == '\\') 
            && ((unsigned)(p[0]) < 0xff)
            && isalpha((char)p[2]))
        {
            // UNC path
            MyPath.mString += *p++;
            MyPath.mString += *p++;
            while(*p && !IsSlash(*p))
                MyPath.mString += *p++;
            if (IsSlash(*p))
                MyPath.mString += *p++;
            while(*p && !IsSlash(*p))
                MyPath.mString += *p++;
            return MyPath;
        }
        else if (((unsigned)(p[0]) < 0xff) 
            && isalpha(p[0]) && (p[1] == ':'))
        {
            MyPath.mString += *p++;
            MyPath.mString += ':';
        }
        else if (do_expand)
        {
            path_char_t cwd_buf[MAX_PATH];
            if (::GetCurrentDirectory(MAX_PATH, cwd_buf))
            {
                WarPath retry_path = cwd_buf;
                retry_path 
                    << WAR_SYSSLASH 
                    << MyPath.GetPath();
                return retry_path.GetMountPath(false);
            }
            else
            {
                WarError system_error;
                system_error.Capture();
                WarThrow(system_error, NULL);
            }
        }
        else
            WarThrow(WarError(WAR_ERR_INVALID_PATH), "funny path");
        
        MyPath.mString += '\\';
        
        return MyPath;
#else // WIN32
        if (mString[0] != WarSysSlash::SYS_SLASH)
            WarThrow(WAR_ERR_INVALID_PATH), "funny path");
        
        return *this;
#endif // WIN32
    }

    /** Compare this path with another path.*/
    bool CmpPath(const path_t& otherPath, 
        const PathCmpTypeE CmpMode,
        bool allowOtherToBeLonger = true,
        int *pCmpResult = NULL) const
    {
        size_t cmp_length = GetLength(),
            other_length = otherPath.GetLength();

        bool return_val = false;
        int cmp_result = 0;
        if (IsSlash(GetLastCh()))
            --cmp_length;
        if (otherPath.IsSlash(otherPath.GetLastCh()))
            --other_length;
        
        if (pCmpResult)
        {
            *pCmpResult = 0;
        }
        else
        {
            if (!allowOtherToBeLonger)
            {
                if (cmp_length != other_length)
                    return false;
            }
            else if ((other_length != cmp_length)
                && !IsSlash(otherPath[cmp_length]))
                return false;
        }
        
        if (CmpMode == PCMP_CASE_INSENSITIVE)
        {

            return_val = (cmp_result = Strnicmp(
                otherPath.GetPath(), cmp_length)) == 0;
        }
        else
            return_val = (cmp_result = memcmp(
            GetPath(), otherPath.GetPath(), 
            cmp_length *sizeof(path_char_t))) == 0;
        
        if (pCmpResult)
            *pCmpResult = cmp_result;

        return return_val;
    }

protected:
private:
};

/* INLINE METHODS */

/* EXTERNAL REFERENCES */

template <class charT, class slashT, class fcT, class fsT>
WarPath<charT, slashT>& operator << (WarPath<charT, slashT>&c, 
                                     const WarPath<fcT, fsT>& from)
{
    WarCollector<charT> &rc = c;
    rc <<  (WarCollector<fcT>)from
    return c;
}

template <class charT, class slashT>
WarPath<charT, slashT>& operator << (WarPath<charT, slashT>&c, 
                                     const WarPathE from)
{
    charT buf[4];
    charT *p = buf;

    if (from != WAR_CDIR)
        c.RemovePaddingSlash();
    
    switch(from)
    {
    case WAR_DSYSSLASH:
         *p++ = (charT)slashT::SYS_SLASH;
         // Fall trough
    case WAR_SYSSLASH:
        *p++ = (charT)slashT::SYS_SLASH;
        break;
    case WAR_DSLASH:
        *p++ =  '/'; 
        // Fall trough
    case WAR_SLASH:
        *p++ =  '/';
        break;
    case WAR_CDIR:
        *p++ =  '.';
        *p++ =  '/';
        break;
    default:
        WarThrow(WarException(WAR_ERR_INVALID_CASE_VALUE), NULL);
    }
    
    *p = 0;
    c.mString += buf;

    return c;
}

template <class charT, class slashT, class T>
WarPath<charT, slashT>& operator << (WarPath<charT, slashT>&c, 
                                     const T& from)
{
    WarCollector<charT> &rc = c;
    rc << from;
    return c;
}

template <class slashT>
std::ostream& operator << (std::ostream& c, 
    const WarPath<char,slashT>& from)
{
    c << from.GetPath();
    return c;
}

template <class slashT>
std::ostream& operator << (std::ostream& c, 
    const WarPath<wchar_t,slashT>& from)
{
    try
    {
        WarPath<char,WarSlash> charPath(from.GetPath());
        c << charPath;
    }
    catch(WarException& ex)
    {
        c << "[filename can not be expressed:" 
            << "(" << ex.Explain() << ")]";
    }
    return c;
}

/** */
template <class charT, class fcT, class fsT>
WarCollector<charT>& operator << (WarCollector<charT>& c, 
                           const WarPath<fcT, fsT>& val)
{
    const WarCollector<fcT>& rc = val;
    return c << rc.mString;
}



#ifdef UNICODE
    typedef WarPath<wchar_t,WarSysSlash> war_syspath_t;
#else
    typedef WarPath<char,WarSysSlash> war_syspath_t;
#endif

typedef WarPath<wchar_t, WarSlash> war_svrpath_t; 
typedef war_svrpath_t::path_ccstring_t war_svrpath_ccstr_t;
typedef WarPath<war_fsysch_t, WarSysSlash> war_registrypath_t;
typedef war_registrypath_t::path_ccstring_t war_registrypath_ccstr_t;
typedef std::basic_string<war_registrypath_t::path_char_t> war_regstr_t;
typedef const war_registrypath_t::path_char_t * war_regstr_ccstr_t;

#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* WAR_PATH_H_ */

