/** 

  The server definition is a template that
  tells WarSvrEngine a host:port to listen to,
  and how to create the appropriate WarServerProtocol
  object when a connection is made.

  Derived classes may also store extra configuration
  information, like default virtual-server properties,
  security-settings ol.
*/

#ifndef WAR_SVR_DEFINITION_H
#define WAR_SVR_DEFINITION_H

/* SYSTEM INCLUDES */

#ifndef WAR_SET_INCLUDED
#   define WAR_SET_INCLUDED
#   include <set>
#endif


/* PROJECT INCLUDES */

#ifndef WAR_PTR_WRAPPER_H
#   include "WarPtrWrapper.h"
#endif

#ifndef WAR_NET_ADDRESS_H
#   include "WarNetAddress.h"
#endif

#ifndef WAR_SMART_POINTER_H
#   include "WarSmartPointer.h"
#endif

#ifndef WAR_SVR_PROPERTIES_H
#   include "WarSvrProperties.h"
#endif

#ifndef WAR_SOCKET_IO_H
#   include "WarSocketIo.h"
#endif

#ifndef WAR_USER_AUTH_H
#   include "WarUserAuth.h"
#endif

#ifndef WAR_USER_ENGINE_H
#   include "WarUserEngine.h"
#endif

/* LOCAL INCLUDES */


/* FORWARD REFERENCES */

class WarSvrProtocol;
class WarSocketIo;

#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/


/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/

#ifdef __cplusplus

class WarSvrDefinition : public WarSmartPointer
{
public:
    // LIFECYCLE
    
    /**
    * Default constructor.
    */
    WarSvrDefinition()
        throw(WarException);
    
    // OPERATORS

    inline bool operator == (const WarSvrDefinition& from)
    {
        return mLocalHost == from.mLocalHost;
    }

    inline bool operator < (const WarSvrDefinition& from)
    {
        return mLocalHost < from.mLocalHost;
    }
 
    // OPERATIONS   
    
    /** Create a new instance of the server
      * 
    */
    WarSvrProtocol *CreateNewInstance(
        war_socket_io_ptr_t& companionPtr)
        throw(WarException);

    // ACCESS

    WarNetAddress& GetHost()
    {
        return mLocalHost;
    }

    WarUserEngine& GetAuth() const
    {
        return (WarUserEngine&)mAuthEngine;
    }

    // INQUIRY

    std::string Explain() const;

    const std::string GetSiteName() const;

    const std::string GetNativeSiteName() const;

    // Properties to be inherited by children
    WarSvrProperties mProperties;
    
protected:
    // Call by CreateNewInstance overrides!!
    void OnCreateNew(WarSvrProtocol *newChild);

    // Override !!
    virtual WarSvrProtocol *GetNewSvr(
        war_socket_io_ptr_t& companionPtr)
        throw(WarException) = 0;

private:
    friend class WarIfSite;
    WarNetAddress mLocalHost;
    WarUserEngine mAuthEngine; // User authentication
    std::string mSiteName; // Jgaa's fan club ...
    std::string mNativeSiteName; // Default site
};

template <class serverT>
class WarSvrDefinitionImpl : public WarSvrDefinition
{
public:
    typedef serverT server_t;
    
    WarSvrDefinitionImpl()
        throw(WarException)
    {
    }
    
protected:
    
    virtual WarSvrProtocol *GetNewSvr(
        war_socket_io_ptr_t& companionPtr)
        throw(WarException)
    {
        return new server_t(companionPtr, *this);
    }
    
};

/* INLINE METHODS */

/* EXTERNAL REFERENCES */

typedef WarPtrWrapper<WarSvrDefinition> war_svrdef_ptr_t;
typedef std::set<war_svrdef_ptr_t> war_svrdef_list_t;

#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* WAR_SVR_DEFINITION_H_ */

