/** 

    This implements the data connection of a FTP session.

    The data connection is used for passicve and 
    active transfers. In the case of passive transfers, 
    the socket will first listen for a connection, and
    then reassign the internal socket to the connected
    socket when the other side connects.
*/

#ifndef WAR_SVR_PROTOCOL_FTP_DATA_CONN_H
#define WAR_SVR_PROTOCOL_FTP_DATA_CONN_H

/* SYSTEM INCLUDES */


/* PROJECT INCLUDES */

#ifndef WAR_SOCKET_H
#   include "WarSocket.h"
#endif

#ifndef WAR_SVR_FILE_H
#   include "WarSvrFile.h"
#endif

/* LOCAL INCLUDES */


/* FORWARD REFERENCES */

class WarSocketIo;
class WarSvrProtocolFtp;

#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/


/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/

#ifdef __cplusplus

class WarSvrProtocolFtpDataConn : public WarSocket
{
public:
    typedef std::list<war_transfer_buffer_ptr_t> bufq_t;
    typedef std::set<war_transfer_buffer_ptr_t> bufq_set_t;

    // LIFECYCLE
    enum DirecionE
    {
        D_SENDING,
        D_RECEIVING,
        D_INVALID
    };
    
    /**
    * Default constructor.
    */
    WarSvrProtocolFtpDataConn(
        war_socket_io_ptr_t& companionPtr,
        WarSvrProtocolFtp& rServer);
      
    // OPERATORS
    // OPERATIONS   

    void SendFile(war_svrfile_ptr_t& filePtr);
    void ReceiveFile(war_svrfile_ptr_t& filePtr);
    void Abort(const war_error_definitions errReason = WAR_FTPDERR_ABORTED);

    // ACCESS
    // INQUIRY

   war_uint32_t GetFileFlags() const
       throw(WarException)
   {
       return mFilePtr->GetFlags();
   }

   // Checks if the dataconnection has exeeded the time limit
   bool IsTimeOut() const;
    
protected:
    ~WarSvrProtocolFtpDataConn(void);
    void OnTransferDone(const WarError& status);

    void StartTransfer(war_svrfile_ptr_t& filePtr);
    void PumpData();
    void KillBuffers();

    virtual void OnAccept(const WarError& status,
        war_socket_t newSocket,
        const WarNetAddress& remoteAddress,
        const WarNetAddress& localAddress);
    virtual void OnConnect(const WarError& status);
    virtual void OnReceived(const WarError& status, 
        war_transfer_buffer_ptr_t& buffer);
    virtual void OnSent(const WarError& status,
        war_transfer_buffer_ptr_t& buffer);

    static void CallbackFunc(
        war_cptr_t ptrToObject, 
        const WarError&, 
        war_transfer_buffer_ptr_t& bufferPtr);

    void QueueToFreeBuffer(WarTransferBuffer *buffer);

    void OnFileIo(const WarError&, 
        war_transfer_buffer_ptr_t& bufferPtr);
    void WriteToFile(war_transfer_buffer_ptr_t& buffer);
    void ProcessPendingBuffers();
    int GetNumBuffers();
   
    bool QueueNetRead();
    void QueueNetWrite(war_transfer_buffer_ptr_t& buffer);
    bool QueueFileRead();
    void QueueFileWrite(war_transfer_buffer_ptr_t& buffer);
  
    WarSvrProtocolFtp& mrServer;
    war_svrfile_ptr_t mFilePtr; // The file we wortk with
    DirecionE mDirection;
    bufq_t mFreeQueue; // Free buffers
    bufq_set_t mPendingInQueue; // Input fragments that are received out of order
    bool mHaveSeenEof;
    int mIoSerialLast; // Higest number in use
    int mIoSerialFirst; // Lowest number in use
    int mNumBuffers; // Buffers in use
    int mNumBufSegments; // Segments in data buffer
    int mBufferLen; // Lenggth of data buffers
    int mSegmentLen; // Length of eachh buffer segment
    int mTimeOutSeconds;
private:
    bool mHasFinished;
    bool mUseFileCallback;
    bool mIsInitialized;

    int mNumPendingNetIoRequests;
    int mNumPendingFileIoRequests;
};

/* INLINE METHODS */

/* EXTERNAL REFERENCES */

typedef WarPtrWrapper<WarSvrProtocolFtpDataConn> war_datasck_ptr_t;

#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* WAR_SVR_PROTOCOL_FTP_DATA_CONN_H_ */

