/** 
    
      Implementation file for macros for the FTP server.
*/

#ifndef WAR_SVR_PROTOCOL_FTP_MACROS_H
#define WAR_SVR_PROTOCOL_FTP_MACROS_H

/* SYSTEM INCLUDES */


/* PROJECT INCLUDES */

#ifndef WAR_SVR_PROTOCOL_FTP_H
#   include "WarSvrProtocolFtp.h"
#endif

#ifndef WAR_OS_H
#   include "WarOs.h"
#endif

#ifndef WAR_ESCAPE_STR_H
#   include "WarEscapeStr.h"
#endif

#ifndef WAR_FSYS_H
#   include "WarFsys.h"
#endif

/* LOCAL INCLUDES */


/* FORWARD REFERENCES */



#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/


/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/


#if 0 // Code template for macros
class macro : public WarTextMacro
{
public:
    macro()
        : WarTextMacro("", "")
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {

    }
};
#endif // 0

class MacroProgramname : public WarTextMacro
{
public:
    MacroProgramname()
        : WarTextMacro("programname", "Name of the FTP daemon software")
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return WAR_LIBRARY_NAME;
    }
};

class MacroProgramversion : public WarTextMacro
{
public:
    MacroProgramversion()
        : WarTextMacro("programversion", "")
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return WAR_LIBRARY_VERSION;
    }
};

class MacroPrgcopyright : public WarTextMacro
{
public:
    MacroPrgcopyright()
        : WarTextMacro("prgcopyright", "Copyright notification of the FTP daemon software")
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        WarCollector<char> msg;
        msg << WAR_LIBRARY_COPYRIGHT
            << WAR_FTPD_DISCLAIMER
            << war_endl;

        return msg.GetValue();
    }
};

class MacroOrigin : public WarTextMacro
{
public:
    MacroOrigin(const WarNetAddress& address)
        : WarTextMacro("origin", "IP address of the current user"),
        mrAddress(address)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mrAddress.Explain();
    }

private:
    const WarNetAddress& mrAddress;
};

class MacroOsname : public WarTextMacro
{
public:
    MacroOsname()
        : WarTextMacro("osname", "The name of the operating system (Linux, Unix, Win95, NT etc...)", true)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return WarOs::GetOs().GetOsName();
    }
};

class MacroPwd : public WarTextMacro
{
public:
    MacroPwd(const WarSvrProtocol& rsvrProtocol)
        : WarTextMacro("pwd", "The current working directory", true),
        mrSvrProtocol(rsvrProtocol)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        WarUtf8 my_cwd = (argument && *argument) 
            ? WarEscapeStr<wchar_t>(mrSvrProtocol.GetCwd().GetAlias().GetPath(), *argument).c_str()
            : mrSvrProtocol.GetCwd().GetAlias().GetPath();
        
        return my_cwd.GetUtf8();
    }

private:
    const WarSvrProtocol& mrSvrProtocol;
};

class MacroCurrentfilename : public WarTextMacro
{
public:
    MacroCurrentfilename(const WarSvrPath& nameBuf)
        : WarTextMacro("currentfilename", "The file name used by the current command"),
        mrPath(nameBuf)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
         WarUtf8 my_filename = (argument && *argument) 
            ? WarEscapeStr<wchar_t>(mrPath.GetAlias().GetFilename().GetPath(), *argument).c_str()
            : mrPath.GetAlias().GetFilename().GetPath();
        
        return my_filename;
    }

    const WarSvrPath & mrPath;
};

class MacroCurrentpath : public WarTextMacro
{
public:
    MacroCurrentpath(const WarSvrPath& nameBuf)
        : WarTextMacro("currentpath", "The Path used by the current command"),
        mrPath(nameBuf)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
         WarUtf8 my_path = (argument && *argument) 
            ? WarEscapeStr<wchar_t>(mrPath.GetAlias().GetPath(), *argument).c_str()
            : mrPath.GetAlias().GetPath();
        

        return my_path;
    }

    const WarSvrPath & mrPath;
};

class MacroRest : public WarTextMacro
{
public:
    MacroRest(const WarSvrProtocolFtpMode& ftpMode)
        : WarTextMacro("rest", "Show the current restart offset"),
        mrMode(ftpMode)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        WarCollector<char> buf;
        buf << mrMode.mRestartPos;
        return buf.GetValue();
    }

private:
    const WarSvrProtocolFtpMode& mrMode;
};

class MacroHelpcmds : public WarTextMacro
{
public:
    MacroHelpcmds(const WarSvrProtocolFtpCommands& rCmds)
        : WarTextMacro("helpcmds", "Show the commands listed by the HELP function"),
        mrCmds(rCmds)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        int row = 1;
        std::string return_val;

        war_cmddef_set_t my_cmds;
        mrCmds.GetCommandDefs(my_cmds);

        for(war_cmddef_set_t::const_iterator P 
            = my_cmds.begin()
            ; P !=  my_cmds.end()
            ; P++)
        {
            static const char spaces[] = "        ";
            std::string cmd_name = P->GetName();
            if (!P->IsImplemented())
                cmd_name += "*";
            
            return_val += cmd_name;
            
            if (!(row++ % 8) && (row != 2))
            {
                return_val += "\n";
            }
            else
            {
                return_val += spaces + (cmd_name.length() % sizeof(spaces));
            }
        }

        if ((++row % 8))
            return_val += "\n";

        return return_val;
    }

    const WarSvrProtocolFtpCommands& mrCmds;
};

class MacroCurrenttype : public WarTextMacro
{
public:
    MacroCurrenttype(const WarSvrProtocolFtpMode& ftpMode)
        : WarTextMacro("currenttype", "Show the current file type"),
        mrMode(ftpMode)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mrMode.GetTypeName();
    }

private:
    const WarSvrProtocolFtpMode& mrMode;
};

class MacroCurrentform : public WarTextMacro
{
public:
    MacroCurrentform(const WarSvrProtocolFtpMode& ftpMode)
        : WarTextMacro("currentform", "Show the current file form"),
        mrMode(ftpMode)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mrMode.GetFormName();
    }

private:
    const WarSvrProtocolFtpMode& mrMode;
};

class MacroCurrentstructure : public WarTextMacro
{
public:
    MacroCurrentstructure(const WarSvrProtocolFtpMode& ftpMode)
        : WarTextMacro("currentstructure", "Show the current structure type"),
        mrMode(ftpMode)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mrMode.GetStructureName();
    }

private:
    const WarSvrProtocolFtpMode& mrMode;
};

class MacroCurrentmode : public WarTextMacro
{
public:
    MacroCurrentmode(const WarSvrProtocolFtpMode& ftpMode)
        : WarTextMacro("currentmode", "Show the current file mode"),
        mrMode(ftpMode)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mrMode.GetModeName();
    }

private:
    const WarSvrProtocolFtpMode& mrMode;
};


/*class macro : public WarTextMacro
{
public:
    macro(const WarSvrProtocol& rSvr)
        : WarTextMacro("", ""),
        mrSvr(rSvr)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {

    }
private:
    const WarSvrProtocolFtp& mrSvr;
};*/

class MacroHostdomainname : public WarTextMacro
{
public:
    MacroHostdomainname(const WarNetAddress& rNet)
        : WarTextMacro("hostdomainname", "Show the domain name for the server"),
        mrNet(rNet)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mrNet.GetDomainname();
    }
private:
    const WarNetAddress& mrNet;
};

class MacroRemotedomainname : public WarTextMacro
{
public:
    MacroRemotedomainname(const WarNetAddress& rNet)
        : WarTextMacro("remotedomainname", "Show the domain name for the client"),
        mrNet(rNet)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mrNet.GetDomainname();
    }
private:
    const WarNetAddress& mrNet;
};

class MacroHostaddress : public WarTextMacro
{
public:
    MacroHostaddress(const WarNetAddress& rNet)
        : WarTextMacro("hostaddress", "Show the Ip address for the server"),
        mrNet(rNet)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mrNet.GetDottedStr();
    }
private:
    const WarNetAddress& mrNet;
};

class MacroRemoteaddress : public WarTextMacro
{
public:
    MacroRemoteaddress(const WarNetAddress& rNet)
        : WarTextMacro("remoteaddress", "Show the IP address for the client"),
        mrNet(rNet)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mrNet.GetDottedStr();
    }
private:
    const WarNetAddress& mrNet;
};

class MacroUser : public WarTextMacro
{
public:
    MacroUser(const WarSvrProtocol& rSvr)
        : WarTextMacro("user", "Returns the login-name for the current user"),
        mrSvr(rSvr)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mrSvr.GetLoginName();
    }
private:
    const WarSvrProtocol& mrSvr;
};

class MacroDataconn : public WarTextMacro
{
public:
    MacroDataconn(const WarSvrProtocolFtp& rSvr)
        : WarTextMacro("dataconn", "returns argument 2 if a dataconnection is active. Else, argument 1 is returned."),
        mrSvr(rSvr)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        // Deparse the argument
        war_ccstr_t p = argument;
        std::string yes, no;

        if (!argument)
            WarThrow(WarError(WAR_ERR_INVALID_ARGUMENT), NULL);

        while(*p)
        {
            if (*p == ',')
            {
                ++p;
                break;
            }
            no += *p++;
        }

        while(*p)
        {
            yes += *p++;
        }

        if (!mrSvr.mDataConnectionPtr.IsEmpty()
            && mrSvr.mDataConnectionPtr->IsOpen())
        {
            return yes;
        }
        else
        {
            return no;
        }
    }
private:
    const WarSvrProtocolFtp& mrSvr;
};

class MacroRcvbytes : public WarTextMacro
{
public:
    MacroRcvbytes(const WarSvrProtocolFtpStat& rStat)
        : WarTextMacro("rcvbytes", "Returns the bytes reveived in normal file transfers during the current session"),
        mrStat(rStat)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        std::string num_buf;
        return WarItoa(num_buf, mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_FILETRANS_BYTES_IN));
    }
private:
    const WarSvrProtocolFtpStat& mrStat;
};

class MacroSndbytes : public WarTextMacro
{
public:
    MacroSndbytes(const WarSvrProtocolFtpStat& rStat)
        : WarTextMacro("sndbytes", "Returns the bytes sent in normal file transfers during the current session"),
        mrStat(rStat)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        std::string num_buf;
        return WarItoa(num_buf, mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_FILETRANS_BYTES_OUT));
    }
private:
    const WarSvrProtocolFtpStat& mrStat;
};

class MacroRcvfiles : public WarTextMacro
{
public:
    MacroRcvfiles(const WarSvrProtocolFtpStat& rStat)
        : WarTextMacro("rcvfiles", "Returns the number of incoming file transfers during the current session"),
        mrStat(rStat)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        std::string num_buf;
        return WarItoa(num_buf, mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_FILETRANS_FILES_IN));
    }
private:
    const WarSvrProtocolFtpStat& mrStat;
};

class MacroSndfiles : public WarTextMacro
{
public:
    MacroSndfiles(const WarSvrProtocolFtpStat& rStat)
        : WarTextMacro("sndfiles", "Returns the number of outgoing file transfers during the current session"),
        mrStat(rStat)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        std::string num_buf;
        return WarItoa(num_buf, mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_FILETRANS_FILES_OUT));
    }
private:
    const WarSvrProtocolFtpStat& mrStat;
};

class MacroNumbytes : public WarTextMacro
{
public:
    MacroNumbytes(const WarSvrProtocolFtpStat& rStat)
        : WarTextMacro("numbytes", "Returns the bytes sent and received in normal file transfers during the current session"),
        mrStat(rStat)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        std::string num_buf;
        return WarItoa(num_buf, mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_FILETRANS_BYTES_IN)
            + mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_FILETRANS_BYTES_OUT));
    }
private:
    const WarSvrProtocolFtpStat& mrStat;
};

class MacroNumfiles : public WarTextMacro
{
public:
    MacroNumfiles(const WarSvrProtocolFtpStat& rStat)
        : WarTextMacro("numfiles", "Returns the files sent and received in normal file transfers during the current session"),
        mrStat(rStat)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        std::string num_buf;
        return WarItoa(num_buf, mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_FILETRANS_FILES_IN)
            + mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_FILETRANS_FILES_OUT));
    }
private:
    const WarSvrProtocolFtpStat& mrStat;
};

class MacroTrafficinbytes : public WarTextMacro
{
public:
    MacroTrafficinbytes(const WarSvrProtocolFtpStat& rStat)
        : WarTextMacro("trafficinbytes", "Returns the bytes received in tmp file transfers and over the control channel during the current session"),
        mrStat(rStat)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        std::string num_buf;
        return WarItoa(num_buf, mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_TMPTRANS_BYTES_IN)
            + mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_CNTR_BYTES_IN));
    }
private:
    const WarSvrProtocolFtpStat& mrStat;
};

class MacroTrafficoutbytes : public WarTextMacro
{
public:
    MacroTrafficoutbytes(const WarSvrProtocolFtpStat& rStat)
        : WarTextMacro("trafficoutbytes", "Returns the bytes sent in tmp file transfers and over the control channel during the current session"),
        mrStat(rStat)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        std::string num_buf;
        return WarItoa(num_buf, mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_TMPTRANS_BYTES_OUT)
            + mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_CNTR_BYTES_OUT));
    }
private:
    const WarSvrProtocolFtpStat& mrStat;
};

class MacroTrafficnumbytes : public WarTextMacro
{
public:
    MacroTrafficnumbytes(const WarSvrProtocolFtpStat& rStat)
        : WarTextMacro("trafficnumbytes", "Returns the bytes sent and received in tmp file transfers and over the control channel during the current session"),
        mrStat(rStat)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        std::string num_buf;
        return WarItoa(num_buf, mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_TMPTRANS_BYTES_IN)
            + mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_CNTR_BYTES_IN)
            + mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_TMPTRANS_BYTES_OUT)
            + mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_CNTR_BYTES_OUT));
    }
private:
    const WarSvrProtocolFtpStat& mrStat;
};

class MacroRcvtransfers : public WarTextMacro
{
public:
    MacroRcvtransfers(const WarSvrProtocolFtpStat& rStat)
        : WarTextMacro("rcvtransfers", "Returns the number of incoming tmp file transfers during the current session"),
        mrStat(rStat)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        std::string num_buf;
        return WarItoa(num_buf, mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_FAILED_TMPTRANS_FILES_IN));
    }
private:
    const WarSvrProtocolFtpStat& mrStat;
};

class MacroSndtransfers : public WarTextMacro
{
public:
    MacroSndtransfers(const WarSvrProtocolFtpStat& rStat)
        : WarTextMacro("sndtransfers", "Returns the number of outgoing tmp file transfers during the current session"),
        mrStat(rStat)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        std::string num_buf;
        return WarItoa(num_buf, mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_FAILED_TMPTRANS_FILES_OUT));
    }
private:
    const WarSvrProtocolFtpStat& mrStat;
};

class MacroNumtransfers : public WarTextMacro
{
public:
    MacroNumtransfers(const WarSvrProtocolFtpStat& rStat)
        : WarTextMacro("numtransfers", "Returns the number of tmp file transfers during the current session"),
        mrStat(rStat)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        std::string num_buf;
        return WarItoa(num_buf, mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_FAILED_TMPTRANS_FILES_OUT)
            + mrStat.GetValue(
            WarSvrProtocolFtpStat::ST_FAILED_TMPTRANS_FILES_IN));
    }
private:
    const WarSvrProtocolFtpStat& mrStat;
};

#if WAR_RFC2389
class MacroFeatures : public WarTextMacro
{
public:
    MacroFeatures(const WarSvrProtocolFtp& rSvr)
        : WarTextMacro("features", "Features response to the FEAT command"),
        mrSvr(rSvr)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mrSvr.GetFeatures();
    }
private:
    const WarSvrProtocolFtp& mrSvr;
};
#endif // WAR_RFC2389

class MacroLocaltime : public WarTextMacro
{
public:
    MacroLocaltime()
        : WarTextMacro("localtime", "Prints the local time.")
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        WarTime now;
        return now.FormatLT();
    }
};

class MacroGmttime : public WarTextMacro
{
public:
    MacroGmttime()
        : WarTextMacro("gmttime", "Prints the current GMT (Greenwhich Mean Time) time.")
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        WarTime now;
        return now.FormatGMT();
    }
};

class MacroCurrentcmd : public WarTextMacro
{
public:
    MacroCurrentcmd(const WarSvrProtocolFtpCommands& cmdRef)
        : WarTextMacro("currentcmd", "Prints the command currently executing."),
        mrCmdRef(cmdRef)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mrCmdRef.GetCurrentCommand().GetName();
    }

private:
    const WarSvrProtocolFtpCommands& mrCmdRef;
};

class MacroCurrentfilelen : public WarTextMacro
{
public:
    MacroCurrentfilelen(const war_flen_t& fileLen)
        : WarTextMacro("currentfilelen", "Length of the current file in bytes."),
        mrLength(fileLen)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        WarCollector<char> buf;

        if (mrLength > 0)
        {
            buf << mrLength;
        
            if (!buf.GetValue().empty() && argument)
                buf << ' ' << argument;
        }

        return buf.GetValue();
    }
private:
    const war_flen_t& mrLength;
};

class MacroCurrenttransbytes : public WarTextMacro
{
public:
    MacroCurrenttransbytes(const war_flen_t& fileLen)
        : WarTextMacro("currenttransbytes", "Bytes transfered in the last transfer."),
        mrLength(fileLen)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        WarCollector<char> buf;
        buf << mrLength;
        
        return buf.GetValue();
    }
private:
    const war_flen_t& mrLength;
};

class MacroCurrenttransrate : public WarTextMacro
{
public:
    MacroCurrenttransrate(const war_int64_t& fileCps)
        : WarTextMacro("currenttransrate", "Speed (characters/sec) of the last transfer."),
        mrCps(fileCps)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {   
        if (argument && ('k' == *argument))
        {
            double speed = mrCps;
            char buf[32];
            sprintf(buf, "%.02lf", speed / 1024.0);
            return buf;
        }
        else if (argument && ('m' == *argument))
        {
            double speed = mrCps;
            char buf[32];
            sprintf(buf, "%.03lf", speed / (1024.0 * 1024.0));
            return buf;
        }
        else if (argument && ('g' == *argument))
        {
            double speed = mrCps;
            char buf[32];
            sprintf(buf, "%.03lf", speed / (1024.0 * 1024.0 * 1024.0));
            return buf;
        }
        
        // Just show bytes/sec
        WarCollector<char> buf;
        buf << mrCps;
        return buf.GetValue();
    }
private:
    const war_int64_t& mrCps;
};

class MacroCurrenttranstime : public WarTextMacro
{
public:
    MacroCurrenttranstime(const WarTime& transTime)
        : WarTextMacro("currenttranstime", "Prints the time used by the last transfer."),
        mrTime(transTime)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        WarCollector<char> buf;
        buf << (mrTime.mTime / 1000);
        
        return buf.GetValue();
    }
private:
    const WarTime& mrTime;
};

class MacroDirmsg : public WarTextMacro
{
public:
    MacroDirmsg(const WarSvrProtocol& rsvrProtocol)
        : WarTextMacro("dirmsg", "Enter directory message (from file)", false),
        mrSvrProtocol(rsvrProtocol)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        try
        {
            std::string msg_file_name = mrSvrProtocol.GetOption("ftpd_DIRMSG");
            if (msg_file_name.empty())
                return "";

            war_syspath_t my_path;
            my_path << mrSvrProtocol.GetCwd().GetPath()
                << WAR_SYSSLASH
                << msg_file_name;

            war_file_ptr_t my_file_ptr = WarFile::Create(my_path);
            my_file_ptr->Open(WarFileEnums::F_READ
                | WarFileEnums::F_SHREAD
                | WarFileEnums::F_MUSTEXIST);

            war_flen_t file_len;

            if ((file_len = my_file_ptr->GetLength()) > (1024 * 32))
            {
                WarLog warn_log(WARLOG_WARNINGS, "MacroDirmsg::Expand()");
                warn_log << "The dir change file \""
                    << my_path
                    << "\" is more than 32 kbytes long. I don't think it's a good idea to "
                    << "throw textfiles this size to the client, - so I don't."
                    << war_endl;
                return "";
            }

            std::string buffer;
            buffer.resize(file_len);
            my_file_ptr->Read(&buffer[0], file_len);
            my_file_ptr = NULL;
            return buffer;
        }
        catch(WarException)
        {
            ;
        }
        
        return "";
    }

private:
    const WarSvrProtocol& mrSvrProtocol;
};

class MacroEmail : public WarTextMacro
{
public:
    MacroEmail(const WarSvrProtocol& rsvrProtocol)
        : WarTextMacro("email", "Rrints the email address to the FTP site administrator.", true),
        mrSvrProtocol(rsvrProtocol)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mrSvrProtocol.GetOption("ftpd_EMAIL");
    }

private:
    const WarSvrProtocol& mrSvrProtocol;
};

class MacroExplainerr : public WarTextMacro
{
public:
    MacroExplainerr(const WarError& lastError)
        : WarTextMacro("explainerr", "Enter directory message (from file)", false),
        mrLastError(lastError)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mrLastError.Explain();
    }

private:
    const WarError& mrLastError;
};

class MacroGreeting : public WarTextMacro
{
public:
    MacroGreeting(const WarSvrProtocol& rsvrProtocol)
        : WarTextMacro("greeting", "Placeholder for greeting message.", false),
        mrSvrProtocol(rsvrProtocol)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return "";
    }

private:
    const WarSvrProtocol& mrSvrProtocol;
};

class MacroDiskfree : public WarTextMacro
{
public:
    MacroDiskfree(const WarSvrProtocol& rsvrProtocol)
        : WarTextMacro("diskfree", "Reports the free space on the current drive",  true),
        mrSvrProtocol(rsvrProtocol)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        WarCollector<char> buf;

        try
        {
            buf << WarGetFreeSpaceOnPath(mrSvrProtocol.GetCwd().GetUrl());
        }
        catch(WarException)
        {
            ;
        }
        
        return buf.GetValue();
    }

private:
    const WarSvrProtocol& mrSvrProtocol;
};

class MacroIdletimelm : public WarTextMacro
{
public:
    MacroIdletimelm(const int& idleLimit)
        : WarTextMacro("idletimelm", "The current idle time limit for the client.",  true),
        mrIdleLimit(idleLimit)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        WarCollector<char> buf;

        buf << mrIdleLimit;
        
        return buf.GetValue();
    }

private:
    const int& mrIdleLimit; // in seconds
};

class MacroSystemname : public WarTextMacro
{
public:
    MacroSystemname(const std::string& siteName)
        : WarTextMacro("systemname", "The name of the site"),
        mSiteName(siteName)
    {}

    std::string Expand(war_ccstr_t argument = NULL) throw(WarException)
    {
        return mSiteName;
    }

    const std::string mSiteName;
};


#ifdef __cplusplus

/* INLINE METHODS */

/* EXTERNAL REFERENCES */

#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* WAR_SVR_PROTOCOL_FTP_MACROS_H_ */

