/** @name Basic datatypes  */
/*@{ */

#ifndef WAR_TYPES_H
#define WAR_TYPES_H

/* SYSTEM INCLUDES */



/* PROJECT INCLUDES */

#ifndef WAR_H
#   include "war.h"
#endif

/* LOCAL INCLUDES */

#if defined(HAVE_LIBPTHREAD) && defined(HAVE_PTHREAD_H) && !defined(WAR_INCLUDED_PTHREAD_H)
#   define WAR_INCLUDED_PTHREAD_H
#   include <pthread.h>
#endif

#ifndef WAR_SYS_TYPES_H_INCLUDED
#   define WAR_SYS_TYPES_H_INCLUDED
#   include <sys/types.h> 
#endif

#ifndef WAR_SYS_STAT_H
#   define WAR_SYS_STAT_H
#   include <sys/stat.h>
#endif

/* FORWARD REFERENCES */


#ifdef __cplusplus
extern "C"
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/
    
#if _MSC_VER
/* Microsoft C++ implementation */
    typedef __int8 war_int8_t;
    typedef __int16 war_int16_t;
    typedef __int32 war_int32_t;
    typedef __int64 war_int64_t;
    typedef unsigned __int8 war_uint8_t;
    typedef unsigned __int16 war_uint16_t;
    typedef unsigned __int32 war_uint32_t;
    typedef unsigned __int64 war_uint64_t;
#else
    /* Generic/GNU implementation */
    
    /** 8 bit integer */
    typedef char war_int8_t;
    /** 16 bits integer */
    typedef short war_int16_t;
    /** 32 it integer */
    typedef int war_int32_t;
    /* 64 bit integer */
    typedef long long war_int64_t;
    
    /** 8 bit unsigned integer */
    typedef unsigned char war_uint8_t;
    /** 16 bits unsigned integer */
    typedef unsigned short war_uint16_t;
    /** 32 it unsigned integer */
    typedef unsigned int war_uint32_t;
    /** 64 bit unsigned integer */
    typedef unsigned long long war_uint64_t;
#endif /* Compiler type/ OS */

/** 
  * The librarys repcacement for time_t
  *
  * 64 bit file-time offset from January 1st, 1970, in
  * 1/1000 seconds resolution.
  */
typedef war_int64_t war_time_t;
    
/** C string type (char *) */
typedef char* war_cstr_t;

/** Wide C string type (char *) */
typedef char* war_wstr_t;

/** Const C string type (const char *) */
typedef const char* war_ccstr_t;

/** Const wide C string type (const char *) */
typedef const wchar_t* war_cwstr_t;

/** C pointer */
typedef void * war_cptr_t;

/** Const C poinrer */
typedef const war_cptr_t war_ccptr_t;

#ifdef UNICODE
    typedef war_wstr_t war_fsysstr_t;
    typedef war_cwstr_t war_cfsysstr_t;
    typedef wchar_t war_fsysch_t;
    typedef wchar_t war_sysch_t;
#else
    typedef war_cstr_t war_fsysstr_t;
    typedef war_ccstr_t war_cfsysstr_t;
    typedef char war_fsysch_t;
    typedef char war_sysch_t;
#endif

/** System string */
typedef war_sysch_t * war_csysstr_t;

/** Const system string */
typedef const war_sysch_t * war_ccsysstr_t;

/** File length and offset type, 64 bit also on 
* systems that only support 32 bit offsets 
*/
typedef war_int64_t war_flen_t;

/** Result set lenghts */
typedef war_int64_t war_resultlen_t;

    
#ifndef HAVE_PORT_T
    /** Socket port */
    typedef war_uint16_t war_port_t;
#else
    typedef port_t war_port_t;
#endif

#ifndef HAVE_MODE_T
    typedef int mode_t;
#endif

#ifndef HAVE_GID_T
    typedef int gid_t;
#endif

#ifndef HAVE_UID_T
    typedef int uid_t;
#endif

#ifndef HAVE_INODE_T
    typedef war_int64_t inode_t;
#endif

#ifdef WIN32
    /** Native file handle for the operating system */
    typedef HANDLE war_filehandle_t;
    typedef struct _stati64 war_stat_t;
#else
    typedef int war_filehandle_t;
    typedef struct stat war_stat_t;
#endif

#if HAVE_LIBPTHREAD
    /** Datatype for critical section (fast mutex) */
    typedef pthread_mutex_t war_critsec_t;
    typedef pthread_cond_t war_event_handle_t;
#elif defined(WIN32)
    typedef CRITICAL_SECTION war_critsec_t;
    typedef HANDLE war_event_handle_t;
#else
#error Need thread implementation!
#endif
    
    /****************** END OLD STYLE C spesific **********/

#ifdef __cplusplus
}
#endif


/* INLINE METHODS */

/* EXTERNAL REFERENCES */

#endif  /* WAR_TYPES_H */

/*@}  Basic datatypes  */
