/** 
*/

#ifndef WAR_URL_TYPE_H
#define WAR_URL_TYPE_H

/* SYSTEM INCLUDES */


/* PROJECT INCLUDES */


/* LOCAL INCLUDES */


/* FORWARD REFERENCES */



#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/


/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/

#ifdef __cplusplus

class WarUrlType 
{
public:
    // LIFECYCLE
    
    /**
    * Default constructor.
    */
    WarUrlType(void)
    {
        Reset();
    }
    
    /**
    * Copy constructor.
    *
    * @param from The value to copy to this object.
    */
    WarUrlType(const WarUrlType& from)
        : mTagName(from.mTagName),
        mDoHostscan(true),
        mCmdDelimiters(from.mCmdDelimiters)
    {}

    WarUrlType(war_ccstr_t tagName)
    {
        std::string tag = tagName;
        if (!strchr(tagName, ':'))
        {
            tag += ":///";
        }
        Create(tag.c_str());
    }
    
    /**
    * Destructor.
    */
    ~WarUrlType(void) 
    {}
    
    // OPERATORS
    
    /**
    * Assignment operator.
    *
    * @param from THe value to assign to this object.
    *
    * @return A reference to this object.
    */
    WarUrlType& operator=(const WarUrlType& from)
    {
        mTagName = from.mTagName;
        mDoHostscan = from.mDoHostscan;
        mCmdDelimiters = from.mCmdDelimiters;
        return *this;
    }

    /** Checks if the tag matches the type
     * ie: if (my_url_type == "http")
     */
    bool operator == (war_ccstr_t urlTag)
    {
        return stricmp(urlTag, mTagName.c_str()) == 0;
    }      
    
    // OPERATIONS
    
    void Create(war_ccstr_t fullUrl) throw(WarException)
    {    
        Reset();

        war_ccstr_t p, pp;
        
        if (!fullUrl || !*fullUrl)
            WarThrow(WarError(WAR_ERR_INVALID_ARGUMENT), NULL);

        for(p = fullUrl; *p; p++)
        {
            if (*p == ':')
            {
                size_t len = p - fullUrl;

                if (len < 2)
                    WarThrow(WarError(WAR_ERR_INVALID_URL), NULL);

                for(pp = fullUrl; pp < p;)
                    mTagName += *pp++;
                
                // Special types...
                if (strnicmp(fullUrl, "ftp", 3) == 0)
                    mCmdDelimiters = ";";
                else if (strnicmp(fullUrl, "http", 4) == 0)
                    mCmdDelimiters = "?#";
                else if (strnicmp(fullUrl, "mailto", 6) == 0)
                    mDoHostscan = false;
                else if (strnicmp(fullUrl, "news", 4) == 0)
                    mDoHostscan = false;

                if (mDoHostscan
                    && (*++p != '/')
                    && (*++p != '/'))
                    WarThrow(WarError(WAR_ERR_INVALID_URL), NULL);

                return;
            }
        }
        
        WarThrow(WarError(WAR_ERR_NOT_AN_URL), NULL);
    }

    void Reset()
    {
        mTagName.erase();
        mDoHostscan = true;
        mCmdDelimiters = NULL;
    }

    // ACCESS
    // INQUIRY

    /** Return the tag part og the fullUrl identifier,
     * ie: "http"
     */
    const std::string& GetUrlTag() const 
        throw(WarException)
    {
        if (mTagName.empty())
            WarThrow(WarError(WAR_ERR_INTERNAL_DATA_NOT_INITIALIZED), NULL);
        return mTagName;
    }

    /** Return the fullUrl identifier
     * ie: "http://"
     */
    const std::string GetUrlName() const
        throw(WarException)
    {
        if (mTagName.empty())
            WarThrow(WarError(WAR_ERR_INTERNAL_DATA_NOT_INITIALIZED), NULL);
        std::string ret_value = mTagName;
        
        if (mDoHostscan)
            ret_value += "://";
        else
            ret_value += ':';

        return ret_value;
    }

    /** Get the length of the full fullUrl identifier,
     * ie: strlen("http://"
     */
    size_t GetLenght() throw(WarException)
    {
        if (mTagName.empty())
            WarThrow(WarError(WAR_ERR_INTERNAL_DATA_NOT_INITIALIZED), NULL);
        return mTagName.size() + 1 + (mDoHostscan ? 2 : 0); // <name>://
    }

    bool IsHostscanRequiered() const
        throw(WarException)
    {
        if (mTagName.empty())
            WarThrow(WarError(WAR_ERR_INTERNAL_DATA_NOT_INITIALIZED), NULL);
        return mDoHostscan;
    }

    inline bool IsCmdDelimiter(const char ch)
    {
        for(war_ccstr_t p = mCmdDelimiters
            ; p && *p
            ; p++)
        {
            if (ch == *p)
                return true;
        }
        return false;
    }

    char GetCmdDelimiter() const
    {
        if (mCmdDelimiters)
            return *mCmdDelimiters;
        return 0;
    }

protected:
private:
    std::string mTagName;
    bool mDoHostscan;
    war_ccstr_t mCmdDelimiters;
};

/* INLINE METHODS */

/* EXTERNAL REFERENCES */

#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* WAR_URL_TYPE_H_ */

