
#include "stdafx.h"
#include "WarFtpdlite.h"
#include "WarFtpdLiteWin32NtService.h"
#include "WarUserAuthWin32Nt.h"
#include "WarWinntLsa.h"

#if WAR_CRYPTO && !defined(WAR_OPENSSL_RSA_H_INCLUDED)
#   define WAR_OPENSSL_RSA_H_INCLUDED
#   include <openssl/rsa.h>
#endif

#if WAR_CRYPTO && !defined(WAR_OPENSSL_PEM_H_INCLUDED)
#   define WAR_OPENSSL_PEM_H_INCLUDED
#   include <openssl/pem.h>
#endif


#ifdef WAR_WINNT
#   include "PrfData.h"
#endif

using namespace std;

void DoInstall()
{
    WarLog err_log(WARLOG_ERROR, "DoInstall()");
    ConfigFlag = 1;
    if (ServiceName.empty())
        ServiceName = "WARSVR";

    WarFtpdLiteWin32NtService my_service(WarCollector<TCHAR>(ServiceName.c_str()).GetValue().c_str());
    if (my_service.IsInstalled())
        my_service.Uninstall();
    
    // Set initial registry values
    war_registrypath_t root_reg_path;
    root_reg_path << "SOFTWARE"
        << WAR_SYSSLASH << WAR_VENDOR
        << WAR_SYSSLASH << WAR_DEBUG_PREFIX _T("Servers")
        << WAR_SYSSLASH << ServiceName;
    
    // Open the Win32 registry
    WarWin32Registry reg_root, reg_defaults,
        reg_sites, reg_ftp, reg_ftp_paths, reg_path,
        reg_auth, reg_auth_nt, reg_log, reg_log_cout,
        reg_log_logfile, reg_log_eventlog,
        reg_modules, reg_modules_fsys, 
        reg_modules_fsys_file, reg_modules_fsys_file_styles,
        reg_modules_fsys_file_styles_denied,
        reg_modules_fsys_file_styles_home,
        reg_modules_fsys_file_styles_public,
        reg_modules_fsys_file_styles_upload,
        reg_ftp_options, reg_protocols, reg_prot_ftp;
    reg_root.Create(HKEY_LOCAL_MACHINE,root_reg_path);
    
    if (InstallFlag > 0)
    {
        // Installation
        TCHAR current_path[MAX_PATH];
        current_path[0] = 0;
        ::GetCurrentDirectory(sizeof(current_path), current_path);

#define KEY(a,b) a.GetRef(b)
#define VAL(a) war_regstr_t(a)

        // Set startup path
        reg_root.SetValue(WAR_WINNT_STARTUP_PATH, war_regstr_t(current_path));
        reg_root.SetValue(WAR_WINNT_REG_CLASSID, _T(WAR_CLSID_ROOT));
        reg_root.SetValue(WAR_WINNT_REG_NAME, _T(PROGRAM));

        {
            TCHAR str_path[MAX_PATH];
            if (::GetModuleFileName(NULL, str_path, 
                sizeof(str_path) / sizeof(TCHAR)))
            {
                war_syspath_t path = str_path;
                reg_root.SetValue(WAR_WINNT_PROGRAM_NAME, 
                    war_regstr_t(path.GetFilename().GetPath()));
            }
        }

#if WAR_CRYPTO

        war_syspath_t key_file;
        bool do_make_encryption_key = false;
        key_file << current_path << WAR_SYSSLASH << "server_key.pam";
        if (GetFileAttributes(key_file.GetPath()) == 0xffffffff)
        {
            // Create a new private key for the server
            RSA *rsa;
            rsa = RSA_generate_key(1024, RSA_F4, NULL, NULL);
            if (NULL == rsa)
            {
                cerr << "Failed to create a RSA key." << endl;
                WarThrow(WarError(WAR_ERR_SYSTEM_CALL_FAILED), "RSA_generate_key()");
            }
           
            FILE *fp = _tfopen(key_file.GetPath(), _T("wt"));
            if (NULL == fp)
            {
                WAR_CERR << _T("Failed to open ")
                    << key_file.GetPath()
                    << _T(" for write.") << endl;
                WarThrow(WarError(WAR_ERR_SYSTEM_CALL_FAILED), "_tfopen()");
            }

            // Write a non-encrypted key for now. The user can encrypt this later on.
            if (PEM_write_RSAPrivateKey(fp, 
                rsa, 
                NULL,/*EVP_des_ede3_cbc()*/
                NULL, 
                0,
                NULL, 
                NULL) == 0)
            {
                RSA_free(rsa);
                fclose(fp);
                cerr << "Failed to write to the RSA key file." << endl;
                WarThrow(WarError(WAR_ERR_SYSTEM_CALL_FAILED), "PEM_write_RSAPrivateKey()");
            }
            RSA_free(rsa);
            fclose(fp);
            do_make_encryption_key = true;
        }
        
        if (do_make_encryption_key
            || !reg_root.HaveValue(WAR_WINNT_REG_CRYPTKEY))
        {
            FILE *fp = _tfopen(key_file.GetPath(), _T("rt"));
            if (NULL == fp)
            {
                WAR_CERR << _T("Failed to reopen ")
                    << key_file.GetPath()
                    << _T(" for read.") << endl;
                WarThrow(WarError(WAR_ERR_SYSTEM_CALL_FAILED), "_tfopen()");
            }

            EVP_PKEY *key = PEM_read_PrivateKey(
                fp, 
                NULL,
                NULL,
                NULL);

            fclose(fp);

            if (NULL == key)
            {
                WarOpenSslError sys_err;
                err_log << "PEM_read_PrivateKey() failed when reading pam file "
                    << key_file.GetPath()
                    << sys_err
                    << war_endl;
                
                WarThrow(sys_err, "PEM_read_PrivateKey()");
            }

            reg_root.SetValue(WAR_WINNT_REG_PKEYFILE, key_file.GetPath());

            char *strkey = BN_bn2hex(key->pkey.dh->priv_key);
            WarCryptText key_crypt(strkey);
            WarCollector<TCHAR> data_key(WarCollector<TCHAR>::SM_ERASE);
            data_key = WarUserAuth::GeneratePasswd(64);
            war_regstr_t encrypded_data_key;
            key_crypt.Encrypt(encrypded_data_key, data_key.GetValue());
            reg_root.SetValue(WAR_WINNT_REG_CRYPTKEY, encrypded_data_key);
            //reg_root.SetValue(_T("DEBUG Data Encryption Key"), data_key.GetValue());

            memset(strkey, 0, strlen(strkey));
            EVP_PKEY_free(key);
        }
        
        // Load the encryption password
        reg_root.LoadPamFile();
        
#endif // WAR_CRYPTO

#ifdef WAR_WINNT

        // Install the NT performance-monitor
        WarPath<WCHAR> dll_path;
        dll_path << current_path 
            << WAR_SYSSLASH 
            << "wfde-perfmon.dll";

        CPrfData::Uninstall();
        CPrfData::Install(dll_path.GetPath());
#endif // WAR_WINNT

        // General options
        WarInstallServerOptionsWin32(reg_root);

        // Logging
        reg_log.Create(KEY(reg_root, WAR_WINNT_REG_LOG_MODULES));
        reg_log_cout.Create(KEY(reg_log, _T("cout")));
        reg_log_cout.SetValue(WAR_WINNT_REG_EVENTS, VAL(_T("WARLOG_ERROR;WARLOG_SECURITY;WARLOG_INOUT;WARLOG_SECURITY;WARLOG_WARNINGS;WARLOG_SYSTEM;WARLOG_SNDFILE;WARLOG_RCVFILE;WARLOG_CREDIR;WARLOG_DELDIR;WARLOG_DELFILE;WARLOG_DELFILE;WARLOG_LOGIN;WARLOG_LOGOUT;WARLOG_CREACC;WARLOG_DELACC;WARLOG_INFO")));
        reg_log_cout.SetValue(WAR_WINNT_REG_CLASSID, _T(WAR_CLSID_LOG_COUT));
        reg_log_cout.SetValue(WAR_WINNT_REG_ENABLE, true);
        reg_log_logfile.Create(KEY(reg_log, _T("logfile")));
        reg_log_logfile.SetValue(WAR_WINNT_REG_EVENTS, VAL(_T("WARLOG_ERROR;WARLOG_SECURITY;WARLOG_INOUT;WARLOG_SECURITY;WARLOG_WARNINGS;WARLOG_SYSTEM;WARLOG_SNDFILE;WARLOG_RCVFILE;WARLOG_CREDIR;WARLOG_DELDIR;WARLOG_DELFILE;WARLOG_DELFILE;WARLOG_LOGIN;WARLOG_LOGOUT;WARLOG_CREACC;WARLOG_DELACC;WARLOG_INFO")));
        reg_log_logfile.SetValue(WAR_WINNT_REG_FILENAME, VAL(_T("server.log")));
        reg_log_logfile.SetValue(WAR_WINNT_REG_CLASSID, _T(WAR_CLSID_LOG_FILE));
        reg_log_logfile.SetValue(WAR_WINNT_REG_ENABLE, true);
        reg_log_eventlog.Create(KEY(reg_log, WAR_WINNT_REG_LOG_EVENTLOG));
        reg_log_eventlog.SetValue(WAR_WINNT_REG_EVENTS, VAL(_T("WARLOG_ERROR;WARLOG_SECURITY;WARLOG_WARNINGS;WARLOG_SYSTEM;WARLOG_INFO")));
        reg_log_eventlog.SetValue(WAR_WINNT_REG_TAGNAME, VAL(_T(NT_EVENT_TAG)));
        reg_log_eventlog.SetValue(WAR_WINNT_REG_CLASSID, _T(WAR_CLSID_LOG_NTEVLOG));
        reg_log_eventlog.SetValue(WAR_WINNT_REG_ENABLE, true);

        // Modules
        reg_modules.Create(KEY(reg_root, WAR_WINNT_REG_MODULES));
        reg_modules_fsys.Create(KEY(reg_modules, WAR_WINNT_REG_FILE_SYSTEMS));
        reg_modules_fsys_file.Create(KEY(reg_modules_fsys, _T("file")));
        reg_modules_fsys_file.SetValue(WAR_WINNT_REG_ENABLE, 1);
        reg_modules_fsys_file_styles.Create(KEY(reg_modules_fsys_file, WAR_WINNT_REG_STYLES));
        reg_modules_fsys_file_styles_denied.Create(KEY(reg_modules_fsys_file_styles, _T("Denied")));
        reg_modules_fsys_file_styles_denied.SetValue(WAR_WINNT_REG_PERMISSIONS, 0x3800);
        reg_modules_fsys_file_styles_home.Create(KEY(reg_modules_fsys_file_styles, _T("Home")));
        reg_modules_fsys_file_styles_home.SetValue(WAR_WINNT_REG_PERMISSIONS, 0x1d3b9);
        reg_modules_fsys_file_styles_public.Create(KEY(reg_modules_fsys_file_styles, _T("Public")));
        reg_modules_fsys_file_styles_public.SetValue(WAR_WINNT_REG_PERMISSIONS, 0x528a);
        reg_modules_fsys_file_styles_upload.Create(KEY(reg_modules_fsys_file_styles, _T("Upload")));
        reg_modules_fsys_file_styles_upload.SetValue(WAR_WINNT_REG_PERMISSIONS, 0x3d3b0);

        // Sites
        reg_sites.Create(KEY(reg_root, WAR_WINNT_REG_SITES));

        reg_ftp.Create(KEY(reg_sites, _T("Default site")));
        reg_ftp.SetValue(WAR_WINNT_REG_NAME, VAL(_T(DEFAULT_FTP_SVR_NAME)));
        reg_ftp.SetValue(WAR_WINNT_REG_CLASSID, _T(WAR_CLSID_SITE));
        reg_ftp.SetValue(WAR_WINNT_REG_ENABLE, true);

        // Auth modules

        WarCollector<TCHAR> anon_passwd_buf(WarCollector<wchar_t>::SM_ERASE);
        anon_passwd_buf = WarUserAuth::GeneratePasswd();
        reg_auth.Create(KEY(reg_ftp, WAR_WINNT_REG_AUTH_MODULES));
        reg_auth_nt.Create(KEY(reg_auth, _T("Windows NT")));
        reg_auth_nt.SetValue(WAR_WINNT_REG_CLASSID, _T(WAR_CLSID_AUTH_NT));
        reg_auth_nt.SetValue(WAR_WINNT_AUTH_FTP_GROUP, VAL(_T("FTP-Users")));
        reg_auth_nt.SetValue(WAR_WINNT_AUTH_ANON_USER, VAL(_T("FTP-Guest")));
        reg_auth_nt.SetValue(WAR_WINNT_AUTH_ANON_PASSWD, anon_passwd_buf.GetValue().c_str(), true);
        reg_auth_nt.SetValue(WAR_WINNT_AUTH_ALLOW_ANON, false);
        reg_auth_nt.SetValue(WAR_WINNT_AUTH_ANON_PWDEMAIL, true);
        reg_auth_nt.SetValue(WAR_WINNT_AUTH_SERVER_LOCAL, true);

        // Install the desired FTP-group and FTP-user
        {
            war_user_auth_win32_nt_ptr_t auth_ptr = new WarUserAuthWin32Nt;
            auth_ptr->Create(reg_auth, false);
            try
            {
                auth_ptr->CreateGroup(_T("FTP-Users"));
            }
            catch(WarException& ex)
            {
                if (WAR_ERR_OBJECT_EXIST != ex.LocalError())
                    throw ex;
            }
            
            try
            {
                auth_ptr->CreateUser(_T("FTP-Guest"), 
                    anon_passwd_buf.GetValue().c_str());
            }
            catch(WarException& ex)
            {
                if (WAR_ERR_OBJECT_EXIST != ex.LocalError())
                    throw ex;
            }
            
            try
            {
                auth_ptr->AddUserToGroup(_T("FTP-Guest"), _T("Guests"));
            }
            catch(WarException)
            {
            }

            try
            {
                if (UserName.GetValue().empty())
                    UserName = DEFAULT_USER_NAME;
                
                if (!auth_ptr->HaveLocalUser(UserName.GetValue().c_str()))
                {
                    // Generate a random password
                    if (UserPasswd.GetValue().empty())
                        UserPasswd = WarUserAuth::GeneratePasswd();
                    
                    // Create the server's user account
                    auth_ptr->CreateUser(UserName.GetValue().c_str(), 
                        UserPasswd.GetValue().c_str(), true);
                    
                    // Add to "guest" group
                    auth_ptr->AddUserToGroup(UserName.GetValue().c_str(), _T("Guests"));                    
                    reg_root.SetValue(WAR_WINNT_RUN_AS_USER_PWD, UserPasswd.GetValue().c_str(), true);
                }

                reg_root.SetValue(WAR_WINNT_RUN_AS_USER, UserName.GetValue().c_str());
                
                // Give requiered privilegies
                LSA_HANDLE lsa = NULL;
                DWORD error = WarOpenPolicy(NULL, POLICY_ALL_ACCESS, &lsa);
                WarLog err_log(WARLOG_ERROR, "DoInstall()");
                if (0 != error)
                {
                    WarError err(WAR_ERR_SYSTEM_ERROR, error);
                    
                    err_log << "Failed to open LSA policy handle"
                        << err
                        << war_endl;

                } else 
                {
                    if (error = WarAddUserRightToAccount(lsa, 
                        UserName.GetValue().c_str(), SE_SERVICE_LOGON_NAME))
                    {
                        WarError err(WAR_ERR_SYSTEM_ERROR, error);
                        err_log << "Failed to set the SE_SERVICE_LOGON_NAME LSA policy"
                            << err
                            << war_endl;
                    }
                    
                    if (error = WarAddUserRightToAccount(lsa, 
                        UserName.GetValue().c_str(), SE_TCB_NAME))
                    {
                        WarError err(WAR_ERR_SYSTEM_ERROR, error);
                        err_log << "Failed to set the SE_TCB_NAME LSA policy"
                            << err
                            << war_endl;
                    }

                    if (error = WarAddUserRightToAccount(lsa, 
                        _T("FTP-Guest"), SE_BATCH_LOGON_NAME))
                    {
                        WarError err(WAR_ERR_SYSTEM_ERROR, error);
                        err_log << "Failed to set the SE_BATCH_LOGON_NAME LSA policy on FTP-Guest"
                            << err
                            << war_endl;
                    }
                    
                    WarCloseLsa(&lsa);
                }
                
            }
            catch(WarException& ex)
            {
                if (WAR_ERR_OBJECT_EXIST != ex.LocalError())
                    throw ex;
            }
        }

        // Create the NT servic
        // Update the properties for the NT user account we are
        // using!
        WarCollector<TCHAR> service_args, 
            user_name = UserName, 
            user_passwd(UserPasswd.GetValue(), WarCollector<TCHAR>::SM_ERASE),
            visual_name;

        service_args << "--daemon "
            << "--name=" << ServiceName;

        visual_name << PROGRAM << " [" << ServiceName << ']';

        my_service.Install(visual_name.GetValue().c_str(),
            service_args.GetValue().c_str(),
            AutostartFlag,
            user_name.GetValue().c_str(),
            user_passwd.GetValue().c_str());

        // Erase the password buffers
        UserPasswd.Reset();
        user_passwd.Reset();

        // FTP server
        reg_protocols.Create(KEY(reg_ftp, WAR_WINNT_REG_PROTOCOLS));
        reg_prot_ftp.Create(KEY(reg_protocols, WAR_WINNT_REG_FTP));
        reg_prot_ftp.SetValue(WAR_WINNT_REG_ENABLE, true);

#ifdef DEBUG
        // For debug situations, we use a low port number that normally
        // will be blocked from external networks by access-lists in
        // routers, or by firewalls.
        reg_prot_ftp.SetValue(WAR_WINNT_REG_ADDRESS, VAL(_T("*:10")));
#else
        // Listen to all interfaces on the FTP port (normally port 21)
        reg_prot_ftp.SetValue(WAR_WINNT_REG_ADDRESS, VAL(_T("*:ftp")));
#endif
        reg_ftp_paths.Create(KEY(reg_ftp, WAR_WINNT_REG_FILE_SYSTEMS));

        // Virtual server options
        reg_ftp_options.Create(KEY(reg_ftp,WAR_WINNT_REG_OPTIONS));
        
        // Add some standard paths
        reg_path.Create(KEY(reg_ftp_paths, _T("/")));

        war_syspath_t my_path = _T("file:///");
        if (!RootPath.empty())
            my_path << RootPath;
        else
            my_path << current_path << WAR_SYSSLASH << "root";

        reg_path.SetValue(WAR_WINNT_SHARE_URL, war_regstr_t(my_path.GetPath()));
        reg_path.SetValue(WAR_WINNT_REG_PERMISSIONS, WarSvrEnums::DEFAULT_PERMS);

        reg_path.Create(KEY(reg_ftp_paths, _T("/upload")));

        my_path = "file:///";
        if (!RootPath.empty())
            my_path << RootPath << WAR_SYSSLASH << "upload";
        else
            my_path << current_path 
            << WAR_SYSSLASH << "root" 
            << WAR_SYSSLASH << "upload";

        reg_path.SetValue(WAR_WINNT_SHARE_URL, war_regstr_t(my_path.GetPath()));
        reg_path.SetValue(WAR_WINNT_REG_PERMISSIONS, 0x3d3b0);

#undef KEY
#undef VAL
    }
    else
    {
        // Uninstall
        // The NT service is already removed.

#ifdef WAR_WINNT
        // remove the performance-monitor
        CPrfData::Uninstall();
#endif

        // Remove registry settings
        reg_root.DeleteKey();

        AutostartFlag = -1; // Schedule removal of autostart
    }
}

void DoAutostart()
{
    ConfigFlag = 1;

    if (AutostartFlag > 0)
    {
        // Set up autostart
    }
    else
    {
        // Remove autostart
    }
}
