/** 

    Virtual Base class for macros (typically in text-
    strings) that are to be expanded. 
*/

#ifndef WAR_TEXT_MACRO_H
#define WAR_TEXT_MACRO_H

/* SYSTEM INCLUDES */


#ifndef WAR_SET_INCLUDED
#   define WAR_SET_INCLUDED
#   include <set>
#endif


/* PROJECT INCLUDES */


#ifndef WAR_SMART_POINTER_H
#   include "WarSmartPointer.h"
#endif

#ifndef WAR_PTR_WRAPPER
#   include "WarPtrWrapper.h"
#endif

/* LOCAL INCLUDES */


/* FORWARD REFERENCES */



#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/


/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/

#ifdef __cplusplus

class WarTextMacroEnums
{
public:
    enum ExpandModesE
    {
        EXPAND_ALL,
        EXPAND_ALL_BUT_FILES,
        EXPAND_SAFE,
        EXPAND_NONE
    };
};

class WarTextMacro : public WarSmartPointer,
    public WarTextMacroEnums
{
public:
    // LIFECYCLE
   
    WarTextMacro(war_ccstr_t name, 
        war_ccstr_t explanation = NULL,
        bool isSafe = false)
        : mName(name), 
        mExplanation(explanation ? explanation : ""),
        mIsSafe(isSafe)
    {}
    
    // OPERATORS

    bool operator < (const WarTextMacro& from) const
    {
        return mName < from.mName;
    }
    
    bool operator == (const WarTextMacro& from) const
    {
        return mName == from.mName;
    }

    bool operator < (const std::string& from) const
    {
        return mName < from;
    }
    
    bool operator == (const std::string& from) const
    {
        return mName == from;
    }

    // OPERATIONS                       
    // ACCESS
    // INQUIRY

    virtual std::string Expand(war_ccstr_t argument = NULL)
        throw(WarException)
    {
        WarThrow(WarError(WAR_ERR_NOT_IMPLEMENTED), NULL);
    }

    virtual const std::string GetName() const
    {
        return mName;
    }

    virtual bool IsSafe() const
    {
        return mIsSafe;
    }

    virtual bool IsFile() const
    {
        // Return true if the macro reads a file. 
        // This imposes problems with security, where
        // users may try to display system-files,
        // or recursive loops where files includes
        // each others!
        return false;
    }

    /** Get the default mode to expand the macro to.
      * This concerns security, as macros returned
      * by the macros will be expanded according
      * to this level. 
      */
    virtual ExpandModesE GetMode() const
    {
        return EXPAND_SAFE;
    }

protected:
    ~WarTextMacro(void)
    {}
    bool mIsSafe; // Safe to expand - not exploitable

private:
    const std::string mName;
    const std::string mExplanation;
};

/* INLINE METHODS */

/* EXTERNAL REFERENCES */

typedef WarPtrWrapper<WarTextMacro> war_macro_ptr_t;
typedef std::set<war_macro_ptr_t> war_macro_set_t;

#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* WAR_SVR_PROTOCOL_FTP_MACROS_H_ */

